// ***************************************************************************************
// File: session.cpp
// Description: session management
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************


#include "nvs.h"

#ifdef NV_SERVER
	#ifdef WIN32
		#ifdef _DEBUG
			#include <stdlib.h>
			#include <crtdbg.h>
			#define NVNEW new( _NORMAL_BLOCK, __FILE__, __LINE__)
		#else // _DEBUG
			#define NVNEW new
		#endif // _DEBUG
	#else	//WIN32
		#define NVNEW new
	#endif	//WIN32
#else	//NV_SERVER
	#define NVNEW new
#endif	//NV_SERVER

using namespace std;


// Unique session counter
static int SessionCounter = 0;				// Session counter	
static NvMutex ProtectSessionIdent;			// Global protection for session identifier
string GetSessionIdent(int &Counter);		// Used to get the session identifier
void RemoveSessionIdent(string sIdent);		// Remove a session identifier from the global list
static NvObjIndStringList SessionIdents;	// Session identifiers
#ifndef WIN32
unsigned int GetMilliSeconds();
unsigned int GetSeconds();
static unsigned int InitialSeconds = 0;
#endif

extern NvMutex ProtectConsoleLog;				// protect console output defeined in nvs.cpp

// Table of function pointers for fast access
typedef bool (NvSession::*NVS_SYS_COMMAND)(NvCommand*,char*,int,int*,int);
map <string,NVS_SYS_COMMAND> CommandTable;

map <string,string> CWCommandTable; // Container Write Command Table
bool IsInCWTable(string CommandClass, string CommandName);

bool IsInCWTable(string CommandSignature)
{
	if(CWCommandTable.find(CommandSignature) == CWCommandTable.end())
		return false;
	return true;
}

#ifndef WIN32
// get num milliseconds
unsigned int GetMilliSeconds() 
{
	struct timeval now;
	gettimeofday(&now, NULL);
	int RetVal = now.tv_sec - InitialSeconds;
	RetVal *= 1000;
	RetVal += now.tv_usec/1000;
	return RetVal;
}

unsigned int GetSeconds() 
{
	struct timeval now;
	gettimeofday(&now, NULL);
	int RetVal = now.tv_sec;
	return RetVal;
}
#endif

// Intialize the pointer command table
// Called in the nvs server start
void InitCommandTable()
{

#ifndef WIN32
	InitialSeconds = GetSeconds();
#endif

#ifndef AIX

	// SYSTEM CLASS
	CommandTable["SYSTEM:STOP"] = &NvSession::StopServer;
	CommandTable["SYSTEM:SET_PARAM"] = &NvSession::SetSystemParam;
	CommandTable["SYSTEM:GET_PARAM"] = &NvSession::GetSystemParam;
	CommandTable["SYSTEM:DEBUG_STATE"] = &NvSession::SystemDebugState;
	CommandTable["SYSTEM:DEBUG_DUMP"] = &NvSession::SystemDebugDump;
	CommandTable["SYSTEM:GET_MIME_TYPES"] = &NvSession::SystemGetMimeTypes;
	CommandTable["SYSTEM:SET_MIME_TYPE"] = &NvSession::SystemSetMimeType;
	CommandTable["SYSTEM:GET_MIME_TYPE"] = &NvSession::SystemGetMimeType;
	CommandTable["SYSTEM:REMOVE_MIME_TYPE"] = &NvSession::SystemRemoveMimeType;
	CommandTable["SYSTEM:GET_ALIASES"] = &NvSession::SystemGetAliases;
	CommandTable["SYSTEM:SET_ALIAS"] = &NvSession::SystemSetAlias;
	CommandTable["SYSTEM:GET_ALIAS"] = &NvSession::SystemGetAlias;
	CommandTable["SYSTEM:REMOVE_ALIAS"] = &NvSession::SystemRemoveAlias;
	CommandTable["SYSTEM:GET_VHOSTS"] = &NvSession::SystemGetVhosts;
	CommandTable["SYSTEM:ADD_VHOST"] = &NvSession::SystemAddVhost;
	CommandTable["SYSTEM:REMOVE_VHOST"] = &NvSession::SystemRemoveVhost;
	CommandTable["SYSTEM:GET_VERSION"] = &NvSession::SystemGetVersion;
	CommandTable["SYSTEM:GET_VERSION_TYPE"] = &NvSession::SystemGetVersionType;
	CommandTable["SYSTEM:GET_NUM_PROCS"] = &NvSession::SystemGetNumProcs;
	CommandTable["SYSTEM:GET_PLATFORM"] = &NvSession::SystemGetPlatform;
	CommandTable["SYSTEM:GET_ENCODING"] = &NvSession::SystemGetEncoding;
	CommandTable["SYSTEM:THREAD_LIST"] = &NvSession::SystemThreadList;
	CommandTable["SYSTEM:GET_NUM_EXCEPTIONS"] = &NvSession::SystemGetNumExceptions;

	// DEBUG CLASS
	CommandTable["DEBUG:DISPLAY_MESSAGE"] = &NvSession::DebugDisplayMessage;
	CommandTable["DEBUG:DISPLAY_OBJECT"] = &NvSession::DebugDisplayObject;
	CommandTable["DEBUG:DISPLAY_CONTAINER"] = &NvSession::DebugDisplayContainer;
	CommandTable["DEBUG:DISPLAY_VARIABLES"] = &NvSession::DebugDisplayVariables;

	// COMMAND CLASS
	CommandTable["COMMAND:SET_PARAMETER"] = &NvSession::CommandSetParameter;
	CommandTable["COMMAND:REMOVE_PARAMETER"] = &NvSession::CommandRemoveParameter;
	CommandTable["COMMAND:SET_HTTP_HEADER"] = &NvSession::CommandSetHttpHeader;
	CommandTable["COMMAND:SET_HTTP_RET_CODE"] = &NvSession::CommandSetHttpRetCode;
	CommandTable["COMMAND:GET_HTTP_HEADERS"] = &NvSession::CommandGetHttpHeaders;
	CommandTable["COMMAND:SET_OUTPUT_BUFFER"] = &NvSession::CommandSetOutputBuffer;

	// VARIABLE CLASS
	CommandTable["VARIABLE:SET"] = &NvSession::SetVariable;
	CommandTable["VARIABLE:GET"] = &NvSession::GetVariable;
	CommandTable["VARIABLE:GET_MULTI"] = &NvSession::GetMultiVariable;
	CommandTable["VARIABLE:REMOVE"] = &NvSession::RemoveVariable;
	CommandTable["VARIABLE:EXIST"] = &NvSession::VariableExist;
	CommandTable["VARIABLE:ENCODE"] = &NvSession::EncodeVariable;

	// LOCK CLASS
	CommandTable["LOCK:LOCK"] = &NvSession::LockLock;
	CommandTable["LOCK:UNLOCK"] = &NvSession::LockUnlock;
	CommandTable["LOCK:ISLOCKED"] = &NvSession::LockIsLocked;
	CommandTable["LOCK:INFO"] = &NvSession::LockInfo;

	// SEMAPHORE CLASS
	CommandTable["SEMAPHORE:LOCK"] = &NvSession::SemaLock;
	CommandTable["SEMAPHORE:UNLOCK"] = &NvSession::SemaUnlock;
	CommandTable["SEMAPHORE:REMOVE"] = &NvSession::SemaRemove;
	CommandTable["SEMAPHORE:INFO"] = &NvSession::SemaInfo;

	// OBJECT CLASS
	CommandTable["OBJECT:SEND"] = &NvSession::SendObject;
	CommandTable["OBJECT:GET"] = &NvSession::GetObject;
	CommandTable["OBJECT:CREATE"] = &NvSession::CreateObject;
	CommandTable["OBJECT:GET_NUM"] = &NvSession::GetNumObjects;
	CommandTable["OBJECT:GET_NAME"] = &NvSession::GetObjectName;
	CommandTable["OBJECT:GET_TYPE"] = &NvSession::GetObjectType;
	CommandTable["OBJECT:CLEAR_ALL"] = &NvSession::ClearAllObjects;
	CommandTable["OBJECT:COPY"] = &NvSession::CopyObject;
	CommandTable["OBJECT:MOVE"] = &NvSession::MoveObject;
	CommandTable["OBJECT:SET_NAME"] = &NvSession::SetObjectName;
	CommandTable["OBJECT:REMOVE"] = &NvSession::RemoveObject;
	CommandTable["OBJECT:EXIST"] = &NvSession::IsObjectExist;
	CommandTable["OBJECT:ENCODE"] = &NvSession::EncodeObject;
	CommandTable["OBJECT:BOOLEAN_SET_VALUE"] = &NvSession::ObjBooleanSetValue;
	CommandTable["OBJECT:BOOLEAN_GET_VALUE"] = &NvSession::ObjBooleanGetValue;
	CommandTable["OBJECT:INTEGER_SET_VALUE"] = &NvSession::ObjIntegerSetValue;
	CommandTable["OBJECT:INTEGER_GET_VALUE"] = &NvSession::ObjIntegerGetValue;
	CommandTable["OBJECT:STRING_SET_VALUE"] = &NvSession::ObjStringSetValue;
	CommandTable["OBJECT:STRING_GET_VALUE"] = &NvSession::ObjStringGetValue;
	CommandTable["OBJECT:STRINGLIST_SET_VALUE"] = &NvSession::ObjStringListSetValue;
	CommandTable["OBJECT:STRINGLIST_GET_VALUE"] = &NvSession::ObjStringListGetValue;
	CommandTable["OBJECT:STRINGLIST_GET_SIZE"] = &NvSession::ObjStringListGetSize;
	CommandTable["OBJECT:STRINGLIST_REMOVE"] = &NvSession::ObjStringListRemove;
	CommandTable["OBJECT:STRINGLIST_INSERT"] = &NvSession::ObjStringListInsert;
	CommandTable["OBJECT:STRINGLIST_ADD_STRINGLIST"] = &NvSession::ObjStringListAddStringList;
	CommandTable["OBJECT:STRINGLIST_SWAP"] = &NvSession::ObjStringListSwap;
	CommandTable["OBJECT:STRINGLIST_SORT"] = &NvSession::ObjStringListSort;
	CommandTable["OBJECT:STRINGLIST_GET_VALUES"] = &NvSession::ObjStringListGetValues;
	CommandTable["OBJECT:STRINGLIST_SEARCH"] = &NvSession::ObjStringListSearch;
	CommandTable["OBJECT:INDSTRINGLIST_SET_VALUE"] = &NvSession::ObjIndStringListSetValue;
	CommandTable["OBJECT:INDSTRINGLIST_GET_VALUE"] = &NvSession::ObjIndStringListGetValue;
	CommandTable["OBJECT:INDSTRINGLIST_GET_VALUES"] = &NvSession::ObjIndStringListGetValues;
	CommandTable["OBJECT:INDSTRINGLIST_GET_SIZE"] = &NvSession::ObjIndStringListGetSize;
	CommandTable["OBJECT:INDSTRINGLIST_REMOVE"] = &NvSession::ObjIndStringListRemove;
	CommandTable["OBJECT:INDSTRINGLIST_ADD_INDSTRINGLIST"] = &NvSession::ObjIndStringListAddIndStringList;
	CommandTable["OBJECT:INDSTRINGLIST_KEY_EXIST"] = &NvSession::ObjIndStringListKeyExist;
	CommandTable["OBJECT:INDSTRINGLIST_GET_KEY"] = &NvSession::ObjIndStringListGetKey;
	CommandTable["OBJECT:TABLE_SET_DESCRIPTION"] = &NvSession::ObjTableSetDescription;
	CommandTable["OBJECT:TABLE_GET_DESCRIPTION"] = &NvSession::ObjTableGetDescription;
	CommandTable["OBJECT:TABLE_CLEAR"] = &NvSession::ObjTableClear;
	CommandTable["OBJECT:TABLE_CLEAR_DATA"] = &NvSession::ObjTableClearData;
	CommandTable["OBJECT:TABLE_SEARCH"] = &NvSession::ObjTableSearch;
	CommandTable["OBJECT:TABLE_SORT"] = &NvSession::ObjTableSort;
	CommandTable["OBJECT:TABLE_CREATE_FROM"] = &NvSession::ObjTableCreateFrom;
	CommandTable["OBJECT:TABLE_SAVE"] = &NvSession::ObjTableSave;
	CommandTable["OBJECT:TABLE_LOAD"] = &NvSession::ObjTableLoad;
	CommandTable["OBJECT:TABLE_IMPORT"] = &NvSession::ObjTableImport;
	CommandTable["OBJECT:TABLE_EXPORT"] = &NvSession::ObjTableExport;
	CommandTable["OBJECT:TABLE_SELECT_FROM"] = &NvSession::ObjTableSelectFrom;
	CommandTable["OBJECT:TABLE_JOIN"] = &NvSession::ObjTableJoin;
	CommandTable["OBJECT:TABLE_ADD_TABLE"] = &NvSession::ObjTableAddTable;
	CommandTable["OBJECT:TABLE_CLEAR_CELL"] = &NvSession::ObjTableClearCell;
	CommandTable["OBJECT:TABLE_SORT_CELL"] = &NvSession::ObjTableSortCell;
	CommandTable["OBJECT:TABLE_GET_CELL_NUM_LINES"] = &NvSession::ObjTableGetCellNumLines;
	CommandTable["OBJECT:TABLE_GET_CELL_LINE"] = &NvSession::ObjTableGetCellLine;
	CommandTable["OBJECT:TABLE_SET_CELL_LINE"] = &NvSession::ObjTableSetCellLine;
	CommandTable["OBJECT:TABLE_GET_CELL"] = &NvSession::ObjTableGetCell;
	CommandTable["OBJECT:TABLE_SET_CELL"] = &NvSession::ObjTableSetCell;
	CommandTable["OBJECT:TABLE_INSERT_CELL_LINE"] = &NvSession::ObjTableInsertCellLine;
	CommandTable["OBJECT:TABLE_REMOVE_CELL_LINE"] = &NvSession::ObjTableRemoveCellLine;
	CommandTable["OBJECT:TABLE_SWAP_CELL_LINES"] = &NvSession::ObjTableSwapCellLines;
	CommandTable["OBJECT:TABLE_GET_NUM_ROWS"] = &NvSession::ObjTableGetNumRows;
	CommandTable["OBJECT:TABLE_REMOVE_ROW"] = &NvSession::ObjTableRemoveRow;
	CommandTable["OBJECT:TABLE_REMOVE_ROWS"] = &NvSession::ObjTableRemoveRows;
	CommandTable["OBJECT:TABLE_CLEAR_ROW"] = &NvSession::ObjTableClearRow;
	CommandTable["OBJECT:TABLE_CLEAR_ROWS"] = &NvSession::ObjTableClearRows;
	CommandTable["OBJECT:TABLE_INSERT_ROWS"] = &NvSession::ObjTableInsertRows;
	CommandTable["OBJECT:TABLE_SWAP_ROWS"] = &NvSession::ObjTableSwapRows;
	CommandTable["OBJECT:TABLE_GET_ROWS"] = &NvSession::ObjTableGetRows;
	CommandTable["OBJECT:TABLE_GET_ROW"] = &NvSession::ObjTableGetRow;
	CommandTable["OBJECT:TABLE_GET_COLUMN"] = &NvSession::ObjTableGetColumn;
	CommandTable["OBJECT:TABLE_SET_COLUMN"] = &NvSession::ObjTableSetColumn;
	CommandTable["OBJECT:TABLE_GET_NUM_COLUMNS"] = &NvSession::ObjTableGetNumColums;
	CommandTable["OBJECT:TABLE_GET_COLUMN_NAME"] = &NvSession::ObjTableGetColumnName;
	CommandTable["OBJECT:TABLE_GET_COLUMN_NAMES"] = &NvSession::ObjTableGetColumnNames;
	CommandTable["OBJECT:TABLE_GET_COLUMN_DESCRIPTION"] = &NvSession::ObjTableGetColumnDescription;
	CommandTable["OBJECT:TABLE_IS_COLUMN_NUMERIC"] = &NvSession::ObjTableIsColumnNumeric;
	CommandTable["OBJECT:TABLE_GET_COLUMN_INDEX"] = &NvSession::ObjTableGetColumnIndex;
	CommandTable["OBJECT:TABLE_CLEAR_COLUMN"] = &NvSession::ObjTableClearColumn;
	CommandTable["OBJECT:TABLE_REMOVE_COLUMN"] = &NvSession::ObjTableRemoveColumn;
	CommandTable["OBJECT:TABLE_REMOVE_COLUMNS"] = &NvSession::ObjTableRemoveColumns;
	CommandTable["OBJECT:TABLE_INSERT_COLUMN"] = &NvSession::ObjTableInsertColumn;
	CommandTable["OBJECT:TABLE_ADD_COLUMNS"] = &NvSession::ObjTableAddColumns;
	CommandTable["OBJECT:TABLE_MODIFY_COLUMN"] = &NvSession::ObjTableModifyColumn;
	CommandTable["OBJECT:TABLE_SWAP_COLUMNS"] = &NvSession::ObjTableSwapColumns;
	CommandTable["OBJECT:TABLE_FILTER_COLUMNS"] = &NvSession::ObjTableFilterColumns;
	CommandTable["OBJECT:TABLE_SET_PRIMARY_COLUMN"] = &NvSession::ObjTableSetPrimaryColumn;
	CommandTable["OBJECT:TABLE_SET_COLUMN_NAME"] = &NvSession::ObjTableSetColumnName;
	CommandTable["OBJECT:TABLE_SET_COLUMN_TYPE"] = &NvSession::ObjTableSetColumnType;
	CommandTable["OBJECT:TABLE_SET_COLUMN_DESCRIPTION"] = &NvSession::ObjTableSetColumnDescription;
	CommandTable["OBJECT:TABLE_GET_PRIMARY_INFO"] = &NvSession::ObjTableGetPrimaryInfo;
	CommandTable["OBJECT:TABLE_GET_ROW_INDEX"] = &NvSession::ObjTableGetRowIndex;
	CommandTable["OBJECT:TABLE_SET_ROW"] = &NvSession::ObjTableSetRow;
	CommandTable["OBJECT:TABLE_ADD_ROWS"] = &NvSession::ObjTableAddRows;
	CommandTable["OBJECT:FILE_SET_FILENAME"] = &NvSession::ObjFileSetFileName;
	CommandTable["OBJECT:FILE_SET_PERSIST"] = &NvSession::ObjFileSetPersist;
	CommandTable["OBJECT:FILE_GET_PATHNAME"] = &NvSession::ObjFileGetPathName;
	CommandTable["OBJECT:FILE_GET_DIRNAME"] = &NvSession::ObjFileGetDirName;
	CommandTable["OBJECT:FILE_GET_FILENAME"] = &NvSession::ObjFileGetFileName;
	CommandTable["OBJECT:FILE_GET_EXTENSION"] = &NvSession::ObjFileGetExtension;
	CommandTable["OBJECT:FILE_GET_SIZE"] = &NvSession::ObjFileGetSize;
	CommandTable["OBJECT:FILE_REMOVE"] = &NvSession::ObjFileRemove;
	CommandTable["OBJECT:FILE_CREATE"] = &NvSession::ObjFileCreate;
	CommandTable["OBJECT:FILE_CLEAR"] = &NvSession::ObjFileClear;
	CommandTable["OBJECT:FILE_APPEND"] = &NvSession::ObjFileAppend;
	CommandTable["OBJECT:FILE_EXIST"] = &NvSession::ObjFileExist;
	CommandTable["OBJECT:FILE_SET_ORIGIN"] = &NvSession::ObjFileSetOrigin;
	CommandTable["OBJECT:FILE_GET_ORIGIN"] = &NvSession::ObjFileGetOrigin;
	CommandTable["OBJECT:FILE_SET_ASCII"] = &NvSession::ObjFileSetAscii;
	CommandTable["OBJECT:FILE_COMPRESS"] = &NvSession::ObjFileCompress;
	CommandTable["OBJECT:FILE_DECOMPRESS"] = &NvSession::ObjFileDecompress;
	CommandTable["OBJECT:BINARY_GET_SIZE"] = &NvSession::ObjBinaryGetSize;
	CommandTable["OBJECT:BINARY_CLEAR"] = &NvSession::ObjBinaryClear;
	CommandTable["OBJECT:BINARY_APPEND"] = &NvSession::ObjBinaryAppend;
	CommandTable["OBJECT:BINARY_SAVE"] = &NvSession::ObjBinarySave;
	CommandTable["OBJECT:BINARY_LOAD"] = &NvSession::ObjBinaryLoad;

	// TEST CLASS
	CommandTable["TEST:TEST_BROWSER"] = &NvSession::TestBrowser;
	CommandTable["TEST:CRASH"] = &NvSession::Crash;
	CommandTable["TEST:SET_LIST"] = &NvSession::TestSetList;
	CommandTable["TEST:LIST"] = &NvSession::TestList;
	CommandTable["TEST:ZUT"] = &NvSession::Zut;

	// MISC CLASS
	CommandTable["MISC:NOP"] = &NvSession::Nop;
	CommandTable["MISC:SLEEP"] = &NvSession::SleepSession;
	CommandTable["MISC:EXEC"] = &NvSession::ExecFilter;
	CommandTable["MISC:INFOEX"] = &NvSession::InfoEx;
	CommandTable["MISC:GET_CRASH"] = &NvSession::GetCrash;
	CommandTable["MISC:HASH"] = &NvSession::MiscHash;
	CommandTable["MISC:UNIQ"] = &NvSession::MiscUniq;
	CommandTable["MISC:SEQUENCE"] = &NvSession::MiscSequence;
	CommandTable["MISC:REST"] = &NvSession::MiscRest;

	// WEBSERVICE CLASS
	CommandTable["WEBSERVICE:EXECUTE"] = &NvSession::WebserviceExecute;
	CommandTable["WEBSERVICE:REMOVE"] = &NvSession::WebserviceRemove;
	CommandTable["WEBSERVICE:START"] = &NvSession::WebserviceStart;
	CommandTable["WEBSERVICE:STOP"] = &NvSession::WebserviceStop;
	CommandTable["WEBSERVICE:INFO"] = &NvSession::WebserviceInfo;
	CommandTable["WEBSERVICE:INFOEX"] = &NvSession::WebserviceInfoEx;
	CommandTable["WEBSERVICE:CREATE"] = &NvSession::WebserviceCreate;
	CommandTable["WEBSERVICE:PACKAGE"] = &NvSession::WebservicePackage;
	CommandTable["WEBSERVICE:INSTALL"] = &NvSession::WebserviceInstall;
	CommandTable["WEBSERVICE:SET_EDIT_MODE"] = &NvSession::WebserviceSetEditMode;
	CommandTable["WEBSERVICE:EDIT_UPDATE"] = &NvSession::WebserviceEditUpdate;
	CommandTable["WEBSERVICE:IS_EDIT"] = &NvSession::WebserviceIsEdit;
	CommandTable["WEBSERVICE:CAN_EDIT"] = &NvSession::WebserviceCanEdit;
	CommandTable["WEBSERVICE:OPERATION_LIST"] = &NvSession::WebserviceOperationList;
	CommandTable["WEBSERVICE:OPERATION_ENABLE"] = &NvSession::WebserviceOperationEnable;
	CommandTable["WEBSERVICE:OPERATION_CREATE"] = &NvSession::WebserviceOperationCreate;
	CommandTable["WEBSERVICE:OPERATION_REMOVE"] = &NvSession::WebserviceOperationRemove;
	CommandTable["WEBSERVICE:OPERATION_SET_PARAM"] = &NvSession::WebserviceOperationSetParam;
	CommandTable["WEBSERVICE:OPERATION_SOAP"] = &NvSession::WebserviceOperationSoap;
	CommandTable["WEBSERVICE:MESSAGE_LIST"] = &NvSession::WebserviceMessageList;
	CommandTable["WEBSERVICE:MESSAGE_CREATE"] = &NvSession::WebserviceMessageCreate;
	CommandTable["WEBSERVICE:MESSAGE_REMOVE"] = &NvSession::WebserviceMessageRemove;
	CommandTable["WEBSERVICE:MESSAGE_SET_PARAM"] = &NvSession::WebserviceMessageSetParam;
	CommandTable["WEBSERVICE:MESSAGE_SET_CONTENT"] = &NvSession::WebserviceMessageSetContent;
	CommandTable["WEBSERVICE:MESSAGE_GET_CONTENT"] = &NvSession::WebserviceMessageGetContent;
	CommandTable["WEBSERVICE:GET_HEADERS"] = &NvSession::WebserviceGetHeaders;
	CommandTable["WEBSERVICE:SET_HEADERS"] = &NvSession::WebserviceSetHeaders;
	CommandTable["WEBSERVICE:WSDL"] = &NvSession::WebserviceWsdl;

	// TIME CLASS
	CommandTable["TIME:GET"] = &NvSession::GetTime;

	// MAIL CLASS
	CommandTable["MAIL:SEND"] = &NvSession::SendMail;

	// SESSION CLASS
	CommandTable["SESSION:GET_SESSION_ID"] = &NvSession::GetSessionId;
	CommandTable["SESSION:GET_USER"] = &NvSession::GetSessionUser;
	CommandTable["SESSION:GET_PASSWORD"] = &NvSession::GetSessionPassword;
	CommandTable["SESSION:GET_NUM"] = &NvSession::SessionGetNum;
	CommandTable["SESSION:SERVICES"] = &NvSession::SessionServices;
	CommandTable["SESSION:LIST"] = &NvSession::SessionList;
	CommandTable["SESSION:CLOSE"] = &NvSession::Nop;
	CommandTable["SESSION:CLOSE_OTHER"] = &NvSession::SessionCloseOther;
	CommandTable["SESSION:CREATE"] = &NvSession::SessionCreate;
	CommandTable["SESSION:GET_NAMED"] = &NvSession::SessionGetNamed;
	CommandTable["SESSION:FREE_NAMED"] = &NvSession::SessionFreeNamed;
	CommandTable["SESSION:CHECK_CLOSE_REQUEST"] = &NvSession::SessionCheckCloseRequest;
	CommandTable["SESSION:SET_DEFAULT_ERROR_PROC"] = &NvSession::SessionSetDefaultErrorProc;
	CommandTable["SESSION:SET_DEFAULT_ERROR_XSL"] = &NvSession::SessionSetDefaultErrorXsl;
	CommandTable["SESSION:SET_LANGUAGE"] = &NvSession::SessionSetLanguage;
	CommandTable["SESSION:USE_ID_COOKIE"] = &NvSession::SessionUseIdCookie;
	CommandTable["SESSION:SET_DEFAULT_VAR_IDENT"] = &NvSession::SessionSetDefaultVarIdent;

	// REQUEST CLASS
	CommandTable["REQUEST:OPEN"] = &NvSession::OpenRequest;
	CommandTable["REQUEST:CLOSE"] = &NvSession::CloseRequest;

	// SERVICE CLASS
	CommandTable["SERVICE:START"] = &NvSession::ServiceStart;
	CommandTable["SERVICE:STOP"] = &NvSession::ServiceStop;
	CommandTable["SERVICE:SESSIONS"] = &NvSession::ServiceSessions;
	CommandTable["SERVICE:MOUNT"] = &NvSession::ServiceMount;
	CommandTable["SERVICE:UNMOUNT"] = &NvSession::ServiceUnmount;
	CommandTable["SERVICE:SKELETON"] = &NvSession::ServiceSkeleton;
	CommandTable["SERVICE:INFO"] = &NvSession::ServiceInfo;
	CommandTable["SERVICE:INFOEX"] = &NvSession::ServiceInfoEx;
	CommandTable["SERVICE:CONFIG"] = &NvSession::ServiceConfig;
	CommandTable["SERVICE:IDENT"] = &NvSession::ServiceIdent;
	CommandTable["SERVICE:PACKAGE"] = &NvSession::ServicePackage;
	CommandTable["SERVICE:INSTALL"] = &NvSession::ServiceInstall;
	CommandTable["SERVICE:PROTECT_CONTAINERS"] = &NvSession::ServiceProtectContainers;

	// LICENSE CLASS
	CommandTable["LICENSE:GET"] = &NvSession::LicenseGet;
	CommandTable["LICENSE:SET"] = &NvSession::LicenseSet;
	CommandTable["LICENSE:REMOVE"] = &NvSession::LicenseRemove;
	CommandTable["LICENSE:INSTALL"] = &NvSession::LicenseInstall;
	CommandTable["LICENSE:EXPORT"] = &NvSession::LicenseExport;
	CommandTable["LICENSE:INFO"] = &NvSession::LicenseInfo;
	CommandTable["LICENSE:ISDEMO"] = &NvSession::LicenseIsDemo;
	CommandTable["LICENSE:CHECK_EXPIRE_SOON"] = &NvSession::LicenseCheckExpireSoon;
	CommandTable["LICENSE:GET_TYPE"] = &NvSession::LicenseGetType;

	// TRANSACTION CLASS
	CommandTable["TRANSACTION:START"] = &NvSession::TransactionStart;
	CommandTable["TRANSACTION:RESTART"] = &NvSession::TransactionReStart;
	CommandTable["TRANSACTION:ROLLBACK"] = &NvSession::TransactionRollback;
	CommandTable["TRANSACTION:VALIDATE"] = &NvSession::TransactionValidate;
	CommandTable["TRANSACTION:REMOVE"] = &NvSession::TransactionRemove;
	CommandTable["TRANSACTION:INFO"] = &NvSession::TransactionInfo;
	CommandTable["TRANSACTION:INFOEX"] = &NvSession::TransactionInfoEx;
	CommandTable["TRANSACTION:SET_STATUS"] = &NvSession::TransactionSetStatus;
	CommandTable["TRANSACTION:SET_OPTIONS"] = &NvSession::TransactionSetOptions;
	CommandTable["TRANSACTION:GET_CONTAINER"] = &NvSession::TransactionGetContainer;
	CommandTable["TRANSACTION:GET_VARIABLES"] = &NvSession::TransactionGetVariables;

	// SCHEDULER CLASS
	CommandTable["SCHEDULER:CREATE_TASK"] = &NvSession::SchedulerCreateTask;
	CommandTable["SCHEDULER:REMOVE_TASK"] = &NvSession::SchedulerRemoveTask;
	CommandTable["SCHEDULER:ENABLE_TASK"] = &NvSession::SchedulerEnableTask;
	CommandTable["SCHEDULER:RUN_TASK"] = &NvSession::SchedulerRunTask;
	CommandTable["SCHEDULER:SET_TASK_PARAM"] = &NvSession::SchedulerSetTaskParam;
	CommandTable["SCHEDULER:GET_TASK_PARAM"] = &NvSession::SchedulerGetTaskParam;
	CommandTable["SCHEDULER:TASK_LIST"] = &NvSession::SchedulerTaskList;

	// LISTENER CLASS
	CommandTable["LISTENER:CREATE"] = &NvSession::ListenerCreate;
	CommandTable["LISTENER:REMOVE"] = &NvSession::ListenerRemove;
	CommandTable["LISTENER:START"] = &NvSession::ListenerStart;
	CommandTable["LISTENER:STOP"] = &NvSession::ListenerStop;
	CommandTable["LISTENER:SET_PARAM"] = &NvSession::ListenerSetParam;
	CommandTable["LISTENER:LIST"] = &NvSession::ListenerList;

	// THREAD CLASS
	CommandTable["THREAD:CREATE"] = &NvSession::ThreadCreate;

	// PACKAGE CLASS
	CommandTable["PACKAGE:PACKAGE"] = &NvSession::PackagePackage;
	CommandTable["PACKAGE:INSTALL"] = &NvSession::PackageInstall;
	CommandTable["PACKAGE:INFO"] = &NvSession::PackageInfo;

	// LOG CLASS
	CommandTable["LOG:CREATE"] = &NvSession::LogCreate;
	CommandTable["LOG:REMOVE"] = &NvSession::LogRemove;
	CommandTable["LOG:INFO"] = &NvSession::LogInfo;
	CommandTable["LOG:GET_LEVEL"] = &NvSession::LogGetLevel;
	CommandTable["LOG:WRITE"] = &NvSession::LogWrite;
	CommandTable["LOG:SET_OPTIONS"] = &NvSession::LogSetOptions;
	CommandTable["LOG:SEARCH"] = &NvSession::LogSearch;
	CommandTable["LOG:ERASE"] = &NvSession::LogErase;

	// SECURITY CLASS
	CommandTable["SECURITY:INFO"] = &NvSession::SecurityInfo;
	CommandTable["SECURITY:SET_PARAM"] = &NvSession::SecuritySetParam;
	CommandTable["SECURITY:CHECK"] = &NvSession::SecurityCheck;
	CommandTable["SECURITY:ADD_USER"] = &NvSession::SecurityUserAdd;
	CommandTable["SECURITY:ENABLE_USER"] = &NvSession::SecurityUserEnable;
	CommandTable["SECURITY:REMOVE_USER"] = &NvSession::SecurityUserRemove;
	CommandTable["SECURITY:USER_LIST"] = &NvSession::SecurityUserList;
	CommandTable["SECURITY:USER_EXIST"] = &NvSession::SecurityUserExist;
	CommandTable["SECURITY:SET_USER_DESCRIPTION"] = &NvSession::SecurityUserSetDescription;
	CommandTable["SECURITY:SET_USER_FULLNAME"] = &NvSession::SecurityUserSetFullname;
	CommandTable["SECURITY:SET_USER_PARAM"] = &NvSession::SecurityUserSetParam;
	CommandTable["SECURITY:SET_USER_PASSWORD"] = &NvSession::SecurityUserSetPassword;
	CommandTable["SECURITY:SET_OTHER_USER_PASSWORD"] = &NvSession::SecurityUserSetOtherPassword;
	CommandTable["SECURITY:SET_USER_ROLES"] = &NvSession::SecurityUserSetRoles;
	CommandTable["SECURITY:REMOVE_USER_ROLES"] = &NvSession::SecurityUserRemoveRoles;
	CommandTable["SECURITY:USER_PERMISSIONS"] = &NvSession::SecurityUserPermissions;
	CommandTable["SECURITY:SESSION_PERMISSIONS"] = &NvSession::SecuritySessionPermissions;
	CommandTable["SECURITY:ADD_ROLE"] = &NvSession::SecurityRoleAdd;
	CommandTable["SECURITY:REMOVE_ROLE"] = &NvSession::SecurityRoleRemove;
	CommandTable["SECURITY:ROLE_LIST"] = &NvSession::SecurityRoleList;
	CommandTable["SECURITY:ROLE_EXIST"] = &NvSession::SecurityRoleExist;
	CommandTable["SECURITY:SET_ROLE_DESCRIPTION"] = &NvSession::SecurityRoleSetDescription;
	CommandTable["SECURITY:SET_ROLE_INHERITS"] = &NvSession::SecurityRoleSetInherits;
	CommandTable["SECURITY:REMOVE_ROLE_INHERITS"] = &NvSession::SecurityRoleRemoveInherits;
	CommandTable["SECURITY:SET_ROLE_PERMISSIONS"] = &NvSession::SecurityRoleSetPermissions;
	CommandTable["SECURITY:REMOVE_ROLE_PERMISSIONS"] = &NvSession::SecurityRoleRemovePermissions;
	CommandTable["SECURITY:PERMISSION_LIST"] = &NvSession::SecurityPermissionList;
	CommandTable["SECURITY:ADD_SESSION_PERMISSION"] = &NvSession::SecurityAddSessionPermission;
	CommandTable["SECURITY:REMOVE_SESSION_PERMISSION"] = &NvSession::SecurityRemoveSessionPermission;
	CommandTable["SECURITY:LOAD_USER_CONTEXT"] = &NvSession::SecurityLoadUserContext;
	CommandTable["SECURITY:SAVE_USER_CONTEXT"] = &NvSession::SecuritySaveUserContext;

	// APPLICATION CLASS
	CommandTable["APPLICATION:GET_NAME"] = &NvSession::ApplicationGetName;
	CommandTable["APPLICATION:GET_DIR"] = &NvSession::ApplicationGetDir;
	CommandTable["APPLICATION:GET_WORK_DIR"] = &NvSession::ApplicationGetWorkDir;
	CommandTable["APPLICATION:GET_FILE_DIR"] = &NvSession::ApplicationGetFileDir;
	CommandTable["APPLICATION:GET_WROOT_DIR"] = &NvSession::ApplicationGetWrootDir;
	CommandTable["APPLICATION:REMOVE_DIR"] = &NvSession::ApplicationRemoveDir;
	CommandTable["APPLICATION:CREATE_DIR"] = &NvSession::ApplicationCreateDir;
	CommandTable["APPLICATION:DIR"] = &NvSession::ApplicationDir;
	CommandTable["APPLICATION:REMOVE"] = &NvSession::ApplicationRemove;
	CommandTable["APPLICATION:START"] = &NvSession::ApplicationStart;
	CommandTable["APPLICATION:STOP"] = &NvSession::ApplicationStop;
	CommandTable["APPLICATION:SESSIONS"] = &NvSession::ApplicationSessions;
	CommandTable["APPLICATION:INFO"] = &NvSession::ApplicationInfo;
	CommandTable["APPLICATION:INFOEX"] = &NvSession::ApplicationInfoEx;
	CommandTable["APPLICATION:CREATE"] = &NvSession::ApplicationCreate;
	CommandTable["APPLICATION:CONFIG"] = &NvSession::ApplicationConfig;
	CommandTable["APPLICATION:PACKAGE"] = &NvSession::ApplicationPackage;
	CommandTable["APPLICATION:INSTALL"] = &NvSession::ApplicationInstall;
	CommandTable["APPLICATION:REDIRECTION_ADD"] = &NvSession::ApplicationRedirectionAdd;
	CommandTable["APPLICATION:REDIRECTION_REMOVE"] = &NvSession::ApplicationRedirectionRemove;
	CommandTable["APPLICATION:REDIRECTION_ENABLE"] = &NvSession::ApplicationRedirectionEnable;
	CommandTable["APPLICATION:REDIRECTION_INFO"] = &NvSession::ApplicationRedirectionInfo;

	// XML CLASS
	CommandTable["XML:SET_XML"] = &NvSession::XmlSetXml;
	CommandTable["XML:GET_XML"] = &NvSession::XmlGetXml;
	CommandTable["XML:SET_FILE"] = &NvSession::XmlSetXml;
	CommandTable["XML:TRANSFORM"] = &NvSession::XmlTransform;
	CommandTable["XML:LOAD_XSL"] = &NvSession::XmlLoadXsl;
	CommandTable["XML:SET_XSL"] = &NvSession::XmlSetXsl;
	CommandTable["XML:SEND"] = &NvSession::XmlSend;
	CommandTable["XML:VALIDATE"] = &NvSession::XmlValidate;
	CommandTable["XML:CLEAR_CACHE"] = &NvSession::XmlClearCache;

	// CONTAINER CLASS
	CommandTable["CONTAINER:REMOVE"] = &NvSession::RemoveContainer;
	CommandTable["CONTAINER:COPY"] = &NvSession::CopyContainer;
	CommandTable["CONTAINER:MOVE"] = &NvSession::MoveContainer;
	CommandTable["CONTAINER:LIST"] = &NvSession::ListContainers;
	CommandTable["CONTAINER:GET_NUM_SUBCONTAINERS"] = &NvSession::GetNumSubContainers;
	CommandTable["CONTAINER:GET_SUBCONTAINER_NAME"] = &NvSession::GetSubContainerName;
	CommandTable["CONTAINER:GET_PARENT_NAME"] = &NvSession::GetContainerParentName;
	CommandTable["CONTAINER:EXPORT"] = &NvSession::ExportContainer;
	CommandTable["CONTAINER:IMPORT"] = &NvSession::ImportContainer;
	CommandTable["CONTAINER:VIEW"] = &NvSession::ViewContainer;
	CommandTable["CONTAINER:ENCODE"] = &NvSession::EncodeContainer;

	// REGISTRY CLASS
	CommandTable["REGISTRY:REMOVE"] = &NvSession::RemoveRegistryKey;
	CommandTable["REGISTRY:GET"] = &NvSession::GetRegistryKey;
	CommandTable["REGISTRY:SET"] = &NvSession::SetRegistryKey;
	CommandTable["REGISTRY:CREATE"] = &NvSession::CreateRegistryKey;
	CommandTable["REGISTRY:EXIST"] = &NvSession::RegistryKeyExist;
	CommandTable["REGISTRY:EXPORT"] = &NvSession::ExportRegistryKey;
	CommandTable["REGISTRY:IMPORT"] = &NvSession::ImportRegistryKey;
	CommandTable["REGISTRY:CLEAR_CACHE"] = &NvSession::ClearRegistryCache;
	CommandTable["REGISTRY:OPEN_REGISTRY"] = &NvSession::OpenRegistry;
	CommandTable["REGISTRY:CLOSE_REGISTRY"] = &NvSession::CloseRegistry;

	// MQ CLASS
	CommandTable["MQ:ISAVAILABLE"] = &NvSession::MqIsAvailable;
	CommandTable["MQ:CREATE"] = &NvSession::MqCreate;
	CommandTable["MQ:SET_PARAM"] = &NvSession::MqSetParam;
	CommandTable["MQ:REMOVE"] = &NvSession::MqRemove;
	CommandTable["MQ:START"] = &NvSession::MqStart;
	CommandTable["MQ:STOP"] = &NvSession::MqStop;
	CommandTable["MQ:LIST"] = &NvSession::MqList;
	CommandTable["MQ:EXIST"] = &NvSession::MqExist;

	// FILEDB CLASS
	CommandTable["FILEDB:OPEN"] = &NvSession::FileDbOpen;
	CommandTable["FILEDB:CLOSE"] = &NvSession::FileDbClose;
	CommandTable["FILEDB:SQL"] = &NvSession::FileDbSql;
#endif

}
void CleanupCommandTable()
{
	CommandTable.clear();
	CWCommandTable.clear();
}

string GetSessionIdent(int &Counter)
{
	string sIdent = "";
	NV_LOCKLOCK(&ProtectSessionIdent);
	char Buffer[100];
	while(1)
	{
		SessionCounter ++;
		sIdent = GetMain()->GetMachineName();
		sprintf(Buffer, ":%08x%08x", (int)time(NULL), SessionCounter);
		sIdent += Buffer;
		sIdent = MD5_string(sIdent);
		if(!GetMain()->GetLongSessionId())
			sIdent = sIdent.substr(0, 10);
		if(GetMain()->GetMachineIdForSessions().length())
		{
			sIdent += ".";
			sIdent += GetMain()->GetMachineIdForSessions();
		}
		MakeUpper(sIdent);
		if(!SessionIdents.KeyExist(sIdent))
			break;
	}
	SessionIdents.SetValue(sIdent, "1");
	Counter = SessionCounter;
	NV_UNLOCKLOCK(&ProtectSessionIdent);

	return sIdent;
}

void RemoveSessionIdent(string sIdent)
{
	NV_LOCKLOCK(&ProtectSessionIdent);
	if(SessionIdents.KeyExist(sIdent))
		SessionIdents.Remove(sIdent);
	NV_UNLOCKLOCK(&ProtectSessionIdent);
}

/////////////////////////////////////////////////////////////////////////////
// NvSessionList

NvSessionList::NvSessionList()
{
}

NvSessionList::~NvSessionList()
{
	Clear();
}



void NvSessionList::Clear(const char *AppName)
{
	int i;
	vector<NvSession *> m_vSessionsToClear;
	vector<NvSession *> m_vNamedSessionsToClear;

	string AppNameHere = "";
	if(AppName != NULL)
		AppNameHere = AppName;
	NvApplication::CleanAppName(AppNameHere);

	NV_LOCKOBJECT(this);
	
	map<string,void *>::iterator pos;
	for(pos = m_mSessions.begin(); pos != m_mSessions.end(); ++pos)
	{
		NvSession *Session = (NvSession *)(pos->second);
		if(Session == NULL)
			continue;
		if(!AppNameHere.length())
		{
			if(Session->m_sName.length())
				m_vNamedSessionsToClear.push_back(Session);
			else
				m_vSessionsToClear.push_back(Session);
			continue;
		}
		string AppNameInSession= Session->GetAppName();
		NvApplication::CleanAppName(AppNameInSession);
		if(AppNameInSession == AppNameHere)
		{
			if(Session->m_sName.length())
				m_vNamedSessionsToClear.push_back(Session);
			else
				m_vSessionsToClear.push_back(Session);
			continue;
		}
	}

	Sort(m_vSessionsToClear);
	Sort(m_vNamedSessionsToClear);

	// First remove not named sessions
	for(i=(int)(m_vSessionsToClear.size()-1); i>=0; i--)
	{
		NvSession *Session = m_vSessionsToClear[i];
		if(Session != NULL)
		{
			// Remove the session from the list
			m_mSessions.erase(Session->m_sIdent);
		}
	}

	NV_UNLOCKOBJECT(this);
	for(i=(int)(m_vSessionsToClear.size()-1); i>=0; i--)
	{
		NvSession *Session = m_vSessionsToClear[i];
		if(Session != NULL)
		{
			// Remove the session from the list if this is a named session
			delete Session;
		}
	}

	NV_LOCKOBJECT(this);
	// Then remove the named sessions
	for(i=(int)(m_vNamedSessionsToClear.size()-1); i>=0; i--)
	{
		NvSession *Session = m_vNamedSessionsToClear[i];
		if(Session != NULL)
		{
			// Remove the session from the list
			m_mSessions.erase(Session->m_sIdent);
		}
	}

	NV_UNLOCKOBJECT(this);
	for(i=(int)(m_vNamedSessionsToClear.size()-1); i>=0; i--)
	{
		NvSession *Session = m_vNamedSessionsToClear[i];
		if(Session != NULL)
		{
			// Remove the session from the list if this is a named session
			delete Session;
		}
	}

	// To be sure we delete all sessions if global cleanup
	if(!AppNameHere.length())
	{
		NV_LOCKOBJECT(this);
		m_mSessions.clear();
		NV_UNLOCKOBJECT(this);
	}
}



NvSession* NvSessionList::InternalGetSession(const char *Identifier, bool bCheckTimeOut)
{
	if(Identifier == NULL)
		return NULL;

	string IdentifierHere = Identifier;
	CleanString(IdentifierHere);

	if(!IdentifierHere.length())
		return NULL;

	MakeUpper(IdentifierHere);
	NvSession *FoundSession = NULL;
	
	if(m_mSessions.find(IdentifierHere) == m_mSessions.end())
		return NULL;

	FoundSession = (NvSession *)(m_mSessions[IdentifierHere]);

	if(bCheckTimeOut && FoundSession->IsTimeOut())
		return NULL;

	return FoundSession;
}


NvSession* NvSessionList::GetSession(const char *Identifier, bool bCheckMax)
{
	NvSession *ReturnedSession = NULL;
	// We update only for a new session for performance reason
	if(Identifier == NULL)
		Update();

	NV_LOCKOBJECT(this);
	if(Identifier == NULL)
	{
		// Check the maximum number of sessions allowed
		if(bCheckMax)
		{
			int MaxSessions = GetMain()->GetMaxSessions();
			if(MaxSessions != -1)
			{
				if((int)(m_mSessions.size()) >= MaxSessions)
				{
					NV_UNLOCKOBJECT(this);
					return NULL;
				}
			}
		}
		NvSession *Session = NVNEW NvSession;
		if(Session == NULL)
		{
			NV_UNLOCKOBJECT(this);
			return NULL;
		}
		if(!Session->Init())
		{
			delete Session;
			NV_UNLOCKOBJECT(this);
			return NULL;
		}
		ReturnedSession = Session;
		m_mSessions[Session->m_sIdent] = (void *)Session;
	}
	else
	{
		ReturnedSession = InternalGetSession(Identifier);
	}

	if(ReturnedSession != NULL)
		ReturnedSession->IncInuse();
	NV_UNLOCKOBJECT(this);

	// We update also in this case
	if(ReturnedSession == NULL)
		Update();

	return ReturnedSession;
}


NvSession* NvSessionList::GetSessionByName(const char *Name, NvSession* CallingSession, NvCommand *CommandToExecute)
{
	NvSession *ReturnedSession = NULL;
	string NameHere = Name;
	CleanString(NameHere);
	MakeLower(NameHere);

	CommandToExecute->m_bInitNamedSessionCommand = false;

	NV_LOCKOBJECT(this);

	// Try to see if the named session is already owned by the calling session
	if(CallingSession != NULL)
	{
		string Identifier = CallingSession->m_oChilds.GetValue(NameHere);
		if(Identifier.length())
		{
			ReturnedSession = InternalGetSession(Identifier.c_str());
			if(ReturnedSession != NULL)
			{
				// Yes it's already owned by this session
				ReturnedSession->IncInuse();
				ReturnedSession->IncInuseNamed();	// Increase the occurence counter
				CommandToExecute->m_pParentSession = CallingSession;
				NV_UNLOCKOBJECT(this);
				return ReturnedSession;
			}
		}
	}

	// The calling session request this named session for the first time

	string AppName = "";
	if(CallingSession != NULL)
		AppName = CallingSession->GetAppName();

		// First search directly for a session having the same name
	// and free
	map<string,void *>::iterator pos;
	vector<NvSession*> Sessions;
	for(pos = m_mSessions.begin(); pos != m_mSessions.end(); ++pos)
	{
		NvSession *Session = (NvSession *)(pos->second);
		if(Session == NULL)
			continue;
		if(Session->m_sName == NameHere)
		{
			if(AppName.length() && (AppName != Session->GetAppName()))
				continue;	// The session must be in the same application
			Sessions.push_back(Session);
			if(!Session->IsInUse())
			{
				// We directly found a free session having the same name
				// And not used by anybody
				ReturnedSession = Session;
				break;
			}
		}
	}

	// Well, we didn't find yet any free session so we wait for one to be free.
	// We get this one having the minimum number of waiting users
	if(ReturnedSession == NULL && Sessions.size())
	{
		// Session not yet found
		// We must search for the session with the same name having the
		// minimum number of waiting users
		int i;
		ReturnedSession = Sessions[0];
		int MinInUse = ReturnedSession->GetInUse();
		for(i=1; i<(int)(Sessions.size()); i++)
		{
			NvSession *Session = Sessions[i];
			if(Session->GetInUse() < MinInUse)
			{
				MinInUse = Session->GetInUse();
				ReturnedSession = Session;
			}
		}
	}
	
	if(ReturnedSession != NULL)
	{
		if(CallingSession == NULL)
			ReturnedSession->IncInuse();
		else if(ReturnedSession != CallingSession)
		{
			CommandToExecute->m_pParentSession = CallingSession;
			CommandToExecute->m_bInitNamedSessionCommand = true;
			ReturnedSession->IncInuse();
			CallingSession->m_oChilds.SetValue(NameHere, ReturnedSession->m_sIdent);
		}
	}
	NV_UNLOCKOBJECT(this);

	// We update also in this case
	if(ReturnedSession == NULL)
		Update();

	if(ReturnedSession != NULL)
	{
		if(CommandToExecute->m_bInitNamedSessionCommand)
		{
			// We do the lock to the session in order to wait for session to be free
			// the lock is done only when getting the access to the named session for the first time
			NV_LOCKOBJECT(ReturnedSession);
			ReturnedSession->SetInUseNamed(1);	// This is the first call from the calling session to this named session
		}
		ReturnedSession->OriginalCommand = CallingSession->OriginalCommand;
	}

	return ReturnedSession;
	
}


void NvSessionList::FreeSession(const char *Identifier)
{
	NvSession *FoundSession = NULL;
	NV_LOCKOBJECT(this);
	FoundSession = InternalGetSession(Identifier, false);
	if(FoundSession != NULL)
		FoundSession->DecInUse();
	NV_UNLOCKOBJECT(this);
}


void NvSessionList::FreeNamedSession(string& SessionName, NvSession* CallingSession, NvCommand *CommandToExecute)
{
	CommandToExecute->m_bExitNamedSessionCommand = false;
	NvSession *FoundSession = NULL;
	string NameHere = SessionName;
	CleanString(NameHere);
	MakeLower(NameHere);
	string Identifier = CallingSession->m_oChilds.GetValue(NameHere);
	if(!Identifier.length())
		return;
	
	NV_LOCKOBJECT(this);
	FoundSession = InternalGetSession(Identifier.c_str(), false);
	if(FoundSession != NULL)
	{
		FoundSession->DecInUse();
		FoundSession->DecInUseNamed();
		if(FoundSession->GetInUseNamed() == 0)
		{
			CallingSession->m_oChilds.Remove(NameHere);
			CommandToExecute->m_bExitNamedSessionCommand = true;
		}
	}
	CommandToExecute->m_pParentSession = NULL;
	NV_UNLOCKOBJECT(this);
	if(FoundSession != NULL)
	{
		if(CommandToExecute->m_bExitNamedSessionCommand)
		{
			FoundSession->OriginalCommand = NULL;
			FoundSession->SetOriginalCommand("");
			NV_UNLOCKOBJECT(FoundSession);
		}
	}
}


void NvSessionList::FreeAllNamedSession(NvSession* CallingSession)
{
	if(!CallingSession->m_oChilds.GetSize())
		return;

	NvSession *FoundSession = NULL;
	
	NV_LOCKOBJECT(this);
	
	map<string,string>::iterator pos;
	for(pos = CallingSession->m_oChilds.m_mList.begin(); pos != CallingSession->m_oChilds.m_mList.end(); ++pos)
	{
		string Identifier = pos->second.c_str();
		FoundSession = InternalGetSession(Identifier.c_str(), false);
		if(FoundSession != NULL)
		{
			while(FoundSession->GetInUseNamed())
			{
				FoundSession->DecInUse();
				FoundSession->DecInUseNamed();
			}
			FoundSession->OriginalCommand = NULL;
			FoundSession->SetOriginalCommand("");
			NV_UNLOCKOBJECT(FoundSession);
		}
	}
	CallingSession->m_oChilds.RemoveAll();
	NV_UNLOCKOBJECT(this);
}


void NvSessionList::RemoveSession(const char *Identifier, bool Force, int WaitTime)
{
	NvSession *FoundSession = NULL;
	
	NV_LOCKOBJECT(this);
	FoundSession = InternalGetSession(Identifier, false);
	if(FoundSession == NULL)
	{
		// The session doesn't exist
		NV_UNLOCKOBJECT(this);
		return;
	}
	
	if(!FoundSession->IsInUse())
	{
		m_mSessions.erase(FoundSession->m_sIdent);
		NV_UNLOCKOBJECT(this);
		delete FoundSession;
		return;
	}

	// The session exists and is still in use

	// We reset the time out value for this session so it becomes unsusable
	// And will be deleted automatically from the main thread
	FoundSession->SetTimeOut(0);

	if(Force)
	{
		// We must wait but a maximum of wait time
		time_t initial_time = time(NULL);
		while(1)
		{
			NV_UNLOCKOBJECT(this);
			SLEEP(100);
			NV_LOCKOBJECT(this);
			FoundSession = InternalGetSession(Identifier, false);
			if(FoundSession == NULL)
			{
				// The session has been removed by somebody else
				NV_UNLOCKOBJECT(this);
				return;
			}
			if(!FoundSession->IsInUse())
			{
				m_mSessions.erase(FoundSession->m_sIdent);
				NV_UNLOCKOBJECT(this);
				delete FoundSession;
				return;
			}
			if((int)(time(NULL) - initial_time) > WaitTime)
			{
				// Now we must kill the associated connections
				GetMain()->HttpServer.Clients.Kill(FoundSession);
				NV_UNLOCKOBJECT(this);
				return;
			}
		}
	}
	NV_UNLOCKOBJECT(this);
/*
	NV_LOCKOBJECT(this);
	FoundSession = InternalGetSession(Identifier, false);
	if(FoundSession != NULL)
	{
		// We reset the time out value for this session so it becomes unsusable
		// And will be deleted automatically from the main thread
		FoundSession->SetTimeOut(0);
		if(FoundSession->IsInUse())
		{
			if(Force)
			{
				// In force mode, we must kill the attached connections if the session is still in
				// use after the wait time
				time_t initial_time = time(NULL);
				while((int)(time(NULL) - initial_time) < WaitTime)
				{
					if(!(FoundSession->IsInUse()))
						break;	// The session is not more in use
				}
				// Test again if the session is still in use
				if(FoundSession->IsInUse())
				{
					// Now we must kill the associated connections
					GetMain()->HttpServer.Clients.Kill(FoundSession);
				}
			}
			FoundSession = NULL;
		}
		else
		{
			m_mSessions.erase(FoundSession->m_sIdent);
		}
	}
	NV_UNLOCKOBJECT(this);
	if(FoundSession != NULL)
		delete FoundSession;
*/
		
}


void NvSessionList::Update()
{
	int i;
	vector<NvSession *> m_vSessionsToClear;

	NV_LOCKOBJECT(this);
	
	map<string,void *>::iterator pos;
	for(pos = m_mSessions.begin(); pos != m_mSessions.end(); ++pos)
	{
		NvSession *Session = (NvSession *)(pos->second);
		if(Session == NULL)
			continue;
		if((!(Session->IsInUse())) && (Session->IsTimeOut()))
		{
			m_vSessionsToClear.push_back(Session);
			continue;
		}
	}

	Sort(m_vSessionsToClear);

	for(i=(int)(m_vSessionsToClear.size()-1); i>=0; i--)
	{
		NvSession *Session = m_vSessionsToClear[i];
		m_mSessions.erase(Session->m_sIdent);
	}
	NV_UNLOCKOBJECT(this);
	for(i=(int)(m_vSessionsToClear.size()-1); i>=0; i--)
	{
		NvSession *Session = m_vSessionsToClear[i];
		if(Session != NULL)
			delete Session;
	}
}




void NvSessionList::Sort(vector<NvSession *> &Sessions, bool Ascending)
{
	int num;
	int i, j;								// used for loops
	

	// make the bubble sort

	num = (int)Sessions.size();
	if(num <= 1)
		return;

	NvSession *Entry1;
	NvSession *Entry2;
	bool mutate = false;
	
	for(i=1;i<num;++i)
	{
		for(j=num-1;j>=i;--j)
		{
			Entry1 = Sessions[j-1];
			Entry2 = Sessions[j];
			mutate = false;
			int Value1 = Entry1->iCounter;
			int Value2 = Entry2->iCounter;
			if(Ascending)
			{
				if(Value1 > Value2)
					mutate = true;
			}
			else
			{
				if(Value1 < Value2)
					mutate = true;
			}
			if(mutate)
			{
				Sessions[j-1] = Entry2;
				Sessions[j] = Entry1;
			}
		}
	}              
}




void NvSessionList::SortByCreated(vector<NvSession *> &Sessions, bool Ascending)
{
	int num;
	int i, j;								// used for loops


	// make the bubble sort

	num = (int)Sessions.size();
	if (num <= 1)
		return;

	NvSession *Entry1;
	NvSession *Entry2;
	bool mutate = false;

	for (i = 1; i<num; ++i)
	{
		for (j = num - 1; j >= i; --j)
		{
			Entry1 = Sessions[j - 1];
			Entry2 = Sessions[j];
			mutate = false;
			time_t Value1 = Entry1->tCreationTime;
			time_t Value2 = Entry2->tCreationTime;
			if (Ascending)
			{
				if (Value1 > Value2)
					mutate = true;
			}
			else
			{
				if (Value1 < Value2)
					mutate = true;
			}
			if (mutate)
			{
				Sessions[j - 1] = Entry2;
				Sessions[j] = Entry1;
			}
		}
	}
}

bool NvSessionList::List(NvObjTable *Result, const char *WhatToList, NvSession *ActualSession, const char *Type, const char *SortBy, bool Ascending)
{
	// Always update the list first
	Update();
	NV_LOCKOBJECT(this);

	bool WithActual = true;
	if(ActualSession != NULL)
		WithActual = false;

	string WhatToListHere = WhatToList;
	CleanString(WhatToListHere);
	MakeUpper(WhatToListHere);

	NvSession *Session;
	int i;
	vector<NvSession *> SessionsToList;

	if(WhatToListHere.find("ID:") == 0)
	{
		// Specific session ID requested
		string sWhat = WhatToListHere.substr(3);
		CleanString(sWhat);
		Session = InternalGetSession(sWhat.c_str());
		if((WithActual) || (Session != ActualSession))
			if(Session != NULL && (Session->GetTimeOut() != 0))
				SessionsToList.push_back(Session);
	}
	else if(WhatToListHere.find("APP:") == 0)
	{
		// session for specific application requested
		string sWhat = WhatToListHere.substr(4);
		CleanString(sWhat);
		
		map<string,void *>::iterator pos;
		for(pos = m_mSessions.begin(); pos != m_mSessions.end(); ++pos)
		{
			Session = (NvSession *)(pos->second);
			if(Session == NULL)
				continue;
			if((!WithActual) && (Session == ActualSession))
				continue;
			if(Session->GetTimeOut() == 0)
				continue;
			string AppNameInSession= Session->GetAppName();
			CleanString(AppNameInSession);
			MakeUpper(AppNameInSession);
			if(AppNameInSession == sWhat)
				SessionsToList.push_back(Session);
		}
	}
	else if(WhatToListHere.find("SRV:") == 0)
	{
		// session for specific service requested
		string sWhat = WhatToListHere.substr(4);
		CleanString(sWhat);
		
		map<string,void *>::iterator pos;
		for(pos = m_mSessions.begin(); pos != m_mSessions.end(); ++pos)
		{
			Session = (NvSession *)(pos->second);
			if(Session == NULL)
				continue;
			if((!WithActual) && (Session == ActualSession))
				continue;
			if(Session->GetTimeOut() == 0)
				continue;
			int j;
			NV_LOCKLOCK(&(Session->ProtectInUse));	// The access to m_oServices is protected by the InUse Lock for the session reporting
			for(j=0; j<Session->m_oServices.GetSize(); j++)
			{
				string ServiceNameInSession = Session->m_oServices.GetValue(j);
				CleanString(ServiceNameInSession);
				MakeUpper(ServiceNameInSession);
				if(ServiceNameInSession == sWhat)
				{
					SessionsToList.push_back(Session);
					break;
				}
			}
			NV_UNLOCKLOCK(&(Session->ProtectInUse));
		}
	}
	else
	{
		map<string,void *>::iterator pos;
		for(pos = m_mSessions.begin(); pos != m_mSessions.end(); ++pos)
		{
			Session = (NvSession *)(pos->second);
			if(Session == NULL)
				continue;
			if((!WithActual) && (Session == ActualSession))
				continue;
			if(Session->GetTimeOut() == 0)
				continue;
			SessionsToList.push_back(Session);
		}
	}

	string sSortBy = "";
	if (SortBy != NULL)
		sSortBy = SortBy;
	if (sSortBy == "CREATION_TIME")
		SortByCreated(SessionsToList, Ascending);
	Result->InsertColumn("IDENTIFIER");
	Result->InsertColumn("APPLICATION");
	Result->InsertColumn("NAME");
	Result->InsertColumn("USER");
	Result->InsertColumn("IN_USE");
	Result->InsertColumn("LAST_USE_TIME");
	Result->InsertColumn("TIME_OUT");
	Result->InsertColumn("TOTAL_COMMANDS");
	Result->InsertColumn("ACTUAL_COMMAND");
	Result->InsertColumn("ORIGINAL_COMMAND");
	Result->InsertColumn("COMMAND_TIME");
	Result->InsertColumn("CREATION_TIME");
	Result->InsertColumn("SERVICES");
	Result->InsertColumn("TYPE");
	Result->InsertColumn("STACK");
	int j;
	for(i=0, j=0; i<(int)(SessionsToList.size()); i++)
	{
		Session = SessionsToList[i];
		if(Session == NULL)
			continue;

		// Filter by session type
		string sType = SESSION_TYPE_ANY;
		if (Type != NULL)
			sType = Type;
		MakeUpper(sType);
		if ((sType != SESSION_TYPE_ANY) && (Session->m_sSessionType != sType))
			continue;
		Result->InsertRow();
		Result->InsertCellLine(j, 0, Session->m_sIdent);
		Result->InsertCellLine(j, 1, Session->GetAppName());
		Result->InsertCellLine(j, 2, Session->m_sName);
		Result->InsertCellLine(j, 3, Session->GetUserName());
		char Buf[256];
		sprintf(Buf, "%d", Session->GetInUse());
		Result->InsertCellLine(j, 4, Buf);
		sprintf(Buf, "%d", Session->GetTimeFromLastUse());
		if(Session->IsInUse())
		{
			Result->InsertCellLine(j, 5, "0");
			Result->InsertCellLine(j, 10, Buf);
		}
		else
		{
			Result->InsertCellLine(j, 5, Buf);
			Result->InsertCellLine(j, 10, "0");
		}
		sprintf(Buf, "%d", Session->GetTimeOut());
		Result->InsertCellLine(j, 6, Buf);
		sprintf(Buf, "%d", Session->GetNumCommands());
		Result->InsertCellLine(j, 7, Buf);
		Result->InsertCellLine(j, 8, Session->GetActualCommand());
		Result->InsertCellLine(j, 9, Session->GetOriginalCommand());
		time_t tTime = time(NULL);
		struct tm tCurrentTime;
		localtime_r(&tTime, &tCurrentTime);
		int ActualDay = tCurrentTime.tm_mday;
		int ActualMonth = tCurrentTime.tm_mon;
		int ActualYear = tCurrentTime.tm_year;
		localtime_r(&(Session->tCreationTime), &tCurrentTime);
		if((tCurrentTime.tm_year == ActualYear) && (tCurrentTime.tm_mon == ActualMonth) && (tCurrentTime.tm_mday == ActualDay))
		{
			sprintf(Buf, "%02d:%02d:%02d",
							tCurrentTime.tm_hour,
							tCurrentTime.tm_min,
							tCurrentTime.tm_sec); 
		}
		else
		{
			sprintf(Buf, "%4d/%02d/%02d %02d:%02d:%02d",
							tCurrentTime.tm_year + 1900,
							tCurrentTime.tm_mon + 1,
							tCurrentTime.tm_mday,
							tCurrentTime.tm_hour,
							tCurrentTime.tm_min,
							tCurrentTime.tm_sec); 
		}
		Result->InsertCellLine(j, 11, Buf);
		int k;
		NV_LOCKLOCK(&(Session->ProtectInUse));	// The access to m_oServices is protected by the InUse Lock for the session reporting
		for(k=0; k<Session->m_oServices.GetSize(); k++)
		{
			string ServiceNameInSession = Session->m_oServices.GetValue(k);
			Result->InsertCellLine(j, 12, ServiceNameInSession);
		}
		NV_UNLOCKLOCK(&(Session->ProtectInUse));
		Result->InsertCellLine(j, 13, Session->m_sSessionType);
		vector<string> Stack;
		Session->GetCommandStack(Stack);
		for(k=0; k<(int)Stack.size(); k++)
			Result->InsertCellLine(j, 14, Stack[k]);
		j++;
	}

	SessionsToList.clear();
	if (sSortBy.length() && (sSortBy != "CREATION_TIME"))
	{
		string sSortByHere = sSortBy;
		if (!Ascending)
			sSortByHere = "-" + sSortBy;
		Result->Sort(sSortBy, true);
	}
	NV_UNLOCKOBJECT(this);

	return true;
}


void NvSessionList::List(NvObjStringList *Result)
{
	Result->RemoveAll();
	// Always update the list first
	Update();
	NV_LOCKOBJECT(this);

	NvSession *Session;

	map<string,void *>::iterator pos;
	for(pos = m_mSessions.begin(); pos != m_mSessions.end(); ++pos)
	{
		Session = (NvSession *)(pos->second);
		if(Session == NULL)
			continue;
		Result->Add(Session->m_sIdent);
	}
	NV_UNLOCKOBJECT(this);
}

int NvSessionList::GetNumSessions()
{
	int Retval = 0;
	NV_LOCKOBJECT(this);
	Retval = (int)(m_mSessions.size());
	NV_UNLOCKOBJECT(this);
	return Retval;
}

void NvSessionList::UpdateSessionTimeOut(const char *Identifier)
{
	NV_LOCKOBJECT(this);
	NvSession *Session = InternalGetSession(Identifier);
	if(Session != NULL)
		Session->SetLastUseTime();
	NV_UNLOCKOBJECT(this);
}


/////////////////////////////////////////////////////////////////////////////
// NvSession

NvSession::NvSession()
{
	Initialized = false;
	InUse = 0;
	InUseNamed = 0;
	iTimeOut = NV_SESSION_DEFAULT_TIMEOUT;
	tCreationTime = time(NULL);
	tLastUseTime = tCreationTime;

	// Set the session identifier
	// It has 16 hexadecimal digits
	// The first 8 are the number of seconds since 01/01/1970
	// The second 8 are the session counter initialized at 0 when the process starts
	m_sIdent = GetSessionIdent(iCounter);
	m_sName = "";
	m_sLanguage = "ENGLISH";

	Application = NULL;
	OriginalCommand = NULL;
	mOpenParameters = NULL;
	ActualCommand = NULL;
	m_sUser = NV_SESSION_DEFAULT_USER;
	m_sUserFullName = "";
	m_sPassword = "";
	m_sOpenProc = "";
	m_sCloseProc = "";
	m_sHash = "";
	m_sAppName = "";
	m_sActualCommand = "";
	m_sOriginalCommand = "";
	m_iNumCommands = 0;
	m_bFromTransaction = false;
	m_iThreadHandleOwner = 0;
	m_bListenerCloseRequest = false;
	m_sErrorXsl = "";
	m_sSSOUser = "";
	m_sSSODomain = "";
	m_bUseSessionCookie = false;
	m_iSessionCookieTimeOut = 0;
	m_bSessionCookieHttpOnly = false;
	m_bSessionCookieSecure = false;
	m_sVariableIdent = "#";

	m_bAdministrator = false;
	m_bOverMaxSession = false;
	bInOpenProc = false;
	bHidden = false;


	//Parent = NULL;


	InContainer = NULL;
	OutContainer = NULL;

	CurrentTransaction = NULL;

	PerlInUseCounter = 0;

	ActualSystemLogLevel = GetMain()->Logs.GetLevel("System");

	bDebug = GetMain()->GetDebug();

	m_sProcName = "";	
	m_sErrorProcName = "";
	m_sDefErrorProcName = "";
	m_bInErrorProc = false;

	m_sSessionType = SESSION_TYPE_CLIENT;
	hServerWritePipehandle = 0;
	m_sPipeFile = "";
	bFirstAttachedCommand = false;
	m_bHasPipe = false;
	m_iAttachedConsolePid = 0;
	m_iAttachedLevel = 1;
}

NvSession::~NvSession()
{
	SetTimeOut(0);
	NV_LOCKOBJECT(this);
	try
	{
		Cleanup();
	}
	catch(...)
	{
		GetMain()->IncNumExceptions();
		GetMain()->Logs.Write("Exceptions", "Exception in session destructor", m_sIdent.c_str(), NV_LOG_ERROR);
	}
	NV_UNLOCKOBJECT(this);
	RemoveSessionIdent(m_sIdent);
}


bool NvSession::Init()
{
	bool RetVal = true;
	if(!Initialized)
	{
		// Do init code here
		m_oVariables.SetValue("NV_NIRVADIR", GetMain()->GetNirvaDir());
		m_oVariables.SetValue("NV_HOME_LOGDIR", GetMain()->GetLogHomeDir());
		m_oVariables.SetValue("NV_HOME_REGDIR", GetMain()->GetRegistryHomeDir());
		m_oVariables.SetValue("NV_HOME_APPWORKDIR", GetMain()->GetAppWorkHomeDir());
		string MachineNameEncoded = GetMain()->GetMachineName();
		EncodeString(MachineNameEncoded, true, GetMain()->IsUnicode());
		m_oVariables.SetValue("NV_HOSTNAME", MachineNameEncoded);
		if(GetMain()->IsUnicode())
			m_oVariables.SetValue("NV_UNICODE", "YES");
		else
			m_oVariables.SetValue("NV_UNICODE", "NO");
		if(bDebug)
			m_oVariables.SetValue("NV_DEBUG", "YES");
		else
			m_oVariables.SetValue("NV_DEBUG", "NO");
		string sPlatform = "WIN32";
#ifdef WIN32
#ifdef X64
		sPlatform = "WIN64";
#endif
#endif
#ifdef AIX
		sPlatform = "AIX";
#endif
#ifdef LINUX
		sPlatform = "LINUX";
#ifdef X64
		sPlatform = "LINUX64";
#endif
#endif
#ifdef HPUX
		sPlatform = "HPUX";
#endif
#ifdef HPUXI
		sPlatform = "HPUXI";
#endif
#ifdef SOLARIS
		sPlatform = "SOLARIS";
#endif
		m_oVariables.SetValue("NV_PLATFORM", sPlatform);

		if(ActualSystemLogLevel && !bHidden)
		{
			string Message = "creating session ";
			if(Application != NULL)
			{
				Message += Application->GetName();
				Message += ":";
			}
			Message += m_sIdent;
			GetMain()->Logs.Write("System", Message.c_str(), "", NV_LOG_INFO, 1, m_sIdent.c_str(), "system");
		}
		SqLite.hlib = &(GetMain()->SqLiteInterface);

		Initialized = true;
	}

	return RetVal;
}

void NvSession::Cleanup()
{
	if(Initialized)
	{
		// Do cleanup code here
		if(ActualSystemLogLevel && !bHidden)
		{
			string Message = "closing session ";
			if(Application != NULL)
			{
				Message += Application->GetName();
				Message += ":";
			}
			Message += m_sIdent;
			GetMain()->Logs.Write("System", Message.c_str(), "", NV_LOG_INFO, 1, m_sIdent.c_str(), "system");
		}

		// Close any requests to external NIRVA servers
		Requests.Clear();


		// Call the application close procedure
		if(Application != NULL)
		{
			// Free eventual transaction
			if(CurrentTransaction != NULL)
			{
				Application->Transactions.FreeTransaction(CurrentTransaction->GetIdent().c_str());
				CurrentTransaction = NULL;
			}
			NvCommand Command;
			Command.m_pSession = this;
			Command.m_iCommandSource = NV_SOURCE_SERVER;
			Command.m_sService = "SYSTEM";
			Command.m_sClass = "MISC";
			Command.m_sCommand = "NOP";
			Command.m_sLanguage = m_sLanguage;
			OriginalCommand = &Command;
			ExecuteProcedure(&Command, m_sCloseProc, false);
			OriginalCommand = NULL;
		}

		// We send a SYSTEM NV_EXIT_SESSION command for all session attached services
		int i;
		NV_LOCKLOCK(&ProtectInUse);	// The access to m_oServices is protected by the InUse Lock for the session reporting
		NvObjStringList m_oServicesBis;		// We work with a copy becacuse of possible deadlock problem between service lock and protectinuse lock
		m_oServicesBis.CopyFrom(&m_oServices);
		m_oServices.RemoveAll();
		NV_UNLOCKLOCK(&ProtectInUse);
		for(i=0; i<m_oServicesBis.GetSize(); i++)
		{
			string ServiceInList = m_oServicesBis.GetValue(i);
			NvService *Service = GetMain()->Services.GetService(ServiceInList.c_str(), true, true);
			if(Service == NULL)
				continue;
			string ServiceNameHere = Service->GetName();
			NvCommand RealCommand;
			RealCommand.m_pSession = this;
			MyNvServiceCommand Command;
			if(Service->m_iLanguage == LANGUAGE_JAVA)
				Command.i_Encoding = ENCODING_UTF8;
			else if(Service->m_iLanguage == LANGUAGE_DOTNET)
				Command.i_Encoding = ENCODING_UTF8;
			else
				Command.i_Encoding = Service->m_iEncoding;
			Command.m_pCommand = &RealCommand;
			RealCommand.m_iCommandSource = NV_SOURCE_SERVER;
			RealCommand.m_sService = ServiceInList;
			RealCommand.m_sClass = "SYSTEM";
			RealCommand.m_sCommand = "NV_EXIT_SESSION";
			RealCommand.m_sLanguage = m_sLanguage;
			Service->Command(&Command);
			GetMain()->Services.FreeService(ServiceNameHere.c_str());
		}
		m_oServicesBis.RemoveAll();

		// We remove all attachments to named sessions
		GetMain()->Sessions.FreeAllNamedSession(this);

		// Do SqLite cleanup
		SqLite.Cleanup();

		// Close all opened user registries
		map<string,int>::iterator pos;
		for(pos = m_oRegistries.begin(); pos != m_oRegistries.end(); ++pos)
		{
			string RegistryName = pos->first;
			string RegistryService = "";
			string::size_type index = RegistryName.find("::::");
			if(index != string::npos)
			{
				RegistryService = RegistryName.substr(0, index);
				RegistryName = RegistryName.substr(index+4);
			}
			CleanString(RegistryService);
			MakeUpper(RegistryService);
			int NumInstances =  pos->second;
			int i;

			NvRegistryContainerList *Registries = &(Application->Registries);
			NvService *Service = NULL;
			if(RegistryService == "SYSTEM")
				Registries = &(GetMain()->Registries);
			else if(RegistryService.length())
			{
				Service = GetMain()->Services.GetService(RegistryService.c_str(), true, true);
				if(Service == NULL)
					continue;
				Registries = &(Service->Registries);
			}
			for(i=0;i<NumInstances;i++)
				Registries->CloseRegistry(RegistryName);
			if(Service != NULL)
				GetMain()->Services.FreeService(RegistryService.c_str());
		}
		m_oRegistries.clear();
	
		Logout();
		
		
		Containers.Clear();		// Removes the session containers
		
		// Unlock all eventual system locked objects by this session
		GetMain()->LockList.UnlockAll(m_sIdent);
		GetMain()->FilterLockList.UnlockAll(m_sIdent);

		// Clear the web service in edit mode locked by the session
		GetMain()->Webservices.FreeEdit(this);

		// Close an eventual attached console pipe
		CleanupAttachedConsole();

		Initialized = false;
	}
}

void NvSession::IncInuse()
{
	NV_LOCKLOCK(&ProtectInUse);
	InUse++;
	tLastUseTime = time(NULL);
	NV_UNLOCKLOCK(&ProtectInUse);
}

void NvSession::DecInUse()
{
	NV_LOCKLOCK(&ProtectInUse);
	if(InUse)
	{
		InUse--;
		tLastUseTime = time(NULL);
	}
	NV_UNLOCKLOCK(&ProtectInUse);
}

bool NvSession::IsInUse()
{
	bool RetVal = false;
	NV_LOCKLOCK(&ProtectInUse);
	if(InUse)
		RetVal = true;
	NV_UNLOCKLOCK(&ProtectInUse);

	return RetVal;
}


int NvSession::GetInUse()
{
	int RetVal = 0;
	NV_LOCKLOCK(&ProtectInUse);
	RetVal = InUse;
	NV_UNLOCKLOCK(&ProtectInUse);

	return RetVal;
}



void NvSession::IncInuseNamed()
{
	NV_LOCKLOCK(&ProtectInUse);
	InUseNamed++;
	NV_UNLOCKLOCK(&ProtectInUse);
}

void NvSession::DecInUseNamed()
{
	NV_LOCKLOCK(&ProtectInUse);
	if(InUseNamed>0)
	{
		InUseNamed--;
	}
	NV_UNLOCKLOCK(&ProtectInUse);
}


int NvSession::GetInUseNamed()
{
	int RetVal = 0;
	NV_LOCKLOCK(&ProtectInUse);
	RetVal = InUseNamed;
	NV_UNLOCKLOCK(&ProtectInUse);

	return RetVal;
}


void NvSession::SetInUseNamed(int Value)
{
	NV_LOCKLOCK(&ProtectInUse);
	InUseNamed = Value;
	NV_UNLOCKLOCK(&ProtectInUse);
}


int NvSession::GetTimeFromLastUse()
{
	time_t tActualTime = time(NULL);
	NV_LOCKLOCK(&ProtectInUse);
	int RetVal = (int)(tActualTime - tLastUseTime);
	NV_UNLOCKLOCK(&ProtectInUse);
	return RetVal;
}

void NvSession::SetLastUseTime()
{
	NV_LOCKLOCK(&ProtectInUse);
	tLastUseTime = time(NULL);
	NV_UNLOCKLOCK(&ProtectInUse);
}


bool NvSession::IsTimeOut()
{
	// A 0 time out means that the session should be deleted so it is always in time out condition
	NV_LOCKLOCK(&ProtectInUse);
	if(iTimeOut == 0)
	{
		NV_UNLOCKLOCK(&ProtectInUse);
		return true;
	}
	// A -1 time out means infinite (application life). THis is only autorized for session created with SessionCreate command
	if(iTimeOut == -1)
	{
		NV_UNLOCKLOCK(&ProtectInUse);
		return false;
	}
	bool RetVal = false;
	time_t tActualTime = time(NULL);
	int NumSeconds = (int)(tActualTime - tLastUseTime);
	if(NumSeconds > iTimeOut)
		RetVal = true;
	NV_UNLOCKLOCK(&ProtectInUse);
	return RetVal;
}


void NvSession::SetTimeOut(int TimeOut)
{
	// A 0 time out means that the session should be deleted so it is always in time out condition
	if(TimeOut <-1)
		TimeOut = -1;
	NV_LOCKLOCK(&ProtectInUse);
	iTimeOut = TimeOut;
	NV_UNLOCKLOCK(&ProtectInUse);
}


int NvSession::GetTimeOut()
{
	NV_LOCKLOCK(&ProtectInUse);
	int RetVal = iTimeOut;
	NV_UNLOCKLOCK(&ProtectInUse);

	return RetVal;
}


void NvSession::SetUserName(const char *UserName)
{
	NV_LOCKLOCK(&ProtectInUse);
	m_sUser = UserName;
	NV_UNLOCKLOCK(&ProtectInUse);
}


string NvSession::GetUserName()
{
	NV_LOCKLOCK(&ProtectInUse);
	string RetVal = m_sUser;
	NV_UNLOCKLOCK(&ProtectInUse);

	return RetVal;
}


void NvSession::SetSSOUser(const char *SSOUser)
{
	NV_LOCKLOCK(&ProtectInUse);
	m_sSSOUser = SSOUser;
	NV_UNLOCKLOCK(&ProtectInUse);
}


string NvSession::GetSSOUser()
{
	NV_LOCKLOCK(&ProtectInUse);
	string RetVal = m_sSSOUser;
	NV_UNLOCKLOCK(&ProtectInUse);

	return RetVal;
}


void NvSession::SetSSODomain(const char *SSODomain)
{
	NV_LOCKLOCK(&ProtectInUse);
	m_sSSODomain = SSODomain;
	NV_UNLOCKLOCK(&ProtectInUse);
}


string NvSession::GetSSODomain()
{
	NV_LOCKLOCK(&ProtectInUse);
	string RetVal = m_sSSODomain;
	NV_UNLOCKLOCK(&ProtectInUse);

	return RetVal;
}


void NvSession::SetUserFullName(const char *UserFullName)
{
	NV_LOCKLOCK(&ProtectInUse);
	m_sUserFullName = UserFullName;
	NV_UNLOCKLOCK(&ProtectInUse);
}


string NvSession::GetUserFullName()
{
	NV_LOCKLOCK(&ProtectInUse);
	string RetVal = m_sUserFullName;
	NV_UNLOCKLOCK(&ProtectInUse);

	return RetVal;
}


void NvSession::SetHash(const char *Hash)
{
	NV_LOCKLOCK(&ProtectInUse);
	m_sHash = Hash;
	NV_UNLOCKLOCK(&ProtectInUse);
}


string NvSession::GetHash()
{
	NV_LOCKLOCK(&ProtectInUse);
	string RetVal = m_sHash;
	NV_UNLOCKLOCK(&ProtectInUse);

	return RetVal;
}


void NvSession::SetAppName(const char *AppName)
{
	NV_LOCKLOCK(&ProtectInUse);
	m_sAppName = AppName;
	NV_UNLOCKLOCK(&ProtectInUse);
}


string NvSession::GetAppName()
{
	NV_LOCKLOCK(&ProtectInUse);
	string RetVal = m_sAppName;
	NV_UNLOCKLOCK(&ProtectInUse);

	return RetVal;
}


void NvSession::SetActualCommand(const char *ActualCommandHere)
{
	NV_LOCKLOCK(&ProtectInUse);
	m_sActualCommand = ActualCommandHere;
	NV_UNLOCKLOCK(&ProtectInUse);
}


string NvSession::GetActualCommand()
{
	NV_LOCKLOCK(&ProtectInUse);
	string RetVal = m_sActualCommand;
	NV_UNLOCKLOCK(&ProtectInUse);

	return RetVal;
}


void NvSession::SetOriginalCommand(const char *OriginalCommandHere)
{
	NV_LOCKLOCK(&ProtectInUse);
	m_sOriginalCommand = OriginalCommandHere;
	NV_UNLOCKLOCK(&ProtectInUse);
}


string NvSession::GetOriginalCommand()
{
	NV_LOCKLOCK(&ProtectInUse);
	string RetVal = m_sOriginalCommand;
	NV_UNLOCKLOCK(&ProtectInUse);

	return RetVal;
}


void NvSession::SetNumCommands(int NumCommands)
{
	if(NumCommands <0)
		NumCommands = 0;
	NV_LOCKLOCK(&ProtectInUse);
	m_iNumCommands = NumCommands;
	NV_UNLOCKLOCK(&ProtectInUse);
}


int NvSession::GetNumCommands()
{
	NV_LOCKLOCK(&ProtectInUse);
	int RetVal = m_iNumCommands;
	NV_UNLOCKLOCK(&ProtectInUse);

	return RetVal;
}

void NvSession::IncNumCommands()
{
	NV_LOCKLOCK(&ProtectInUse);
	m_iNumCommands ++;
	NV_UNLOCKLOCK(&ProtectInUse);
}



void NvSession::SetListenerCloseRequest()
{
	NV_LOCKLOCK(&ProtectInUse);
	m_bListenerCloseRequest = true;
	NV_UNLOCKLOCK(&ProtectInUse);
}


bool NvSession::GetListenerCloseRequest()
{
	bool RetVal = false;
	NV_LOCKLOCK(&ProtectInUse);
	RetVal = m_bListenerCloseRequest;
	NV_UNLOCKLOCK(&ProtectInUse);
	return RetVal;
}


void NvSession::SetErrorXsl(const char *ErrorXsl)
{
	NV_LOCKLOCK(&ProtectInUse);
	m_sErrorXsl = ErrorXsl;
	NV_UNLOCKLOCK(&ProtectInUse);
}


string NvSession::GetErrorXsl()
{
	NV_LOCKLOCK(&ProtectInUse);
	string RetVal = m_sErrorXsl;
	NV_UNLOCKLOCK(&ProtectInUse);

	return RetVal;
}

// return -1 if the interpretor cannot be loaded or memory error
// return 0 if the running of script failed
// return 1 in case of success
int NvSession::PerlRunScript(const char* ScriptFileName, NvCommand *CommandToExecute)
{
	MyNvPerlCommand *PerlCommand = NVNEW MyNvPerlCommand;
	if(PerlCommand == NULL)
		return -1;
	PerlCommand->m_pCommand = (void*)CommandToExecute;

	bool bSameThread = true;
	// Since perl 5.8, we have included recursivity management directly
	// in eperl module so it not necessary any more to call perl scripts with a recursivity
	// greater than 0 in a separate thread.
//	if(PerlInUseCounter)
//		bSameThread = false;
	PerlInUseCounter++;
	int RetCode = GetMain()->PerlInterface.PerlThreadRunScript(ScriptFileName, (void*)(PerlCommand), bSameThread);
	if(!RetCode)
	{
		//PM Bug #30 If there is no error code get the last error code in copy error
		if(CommandToExecute->Error.m_iErrorCode == 0)
		{
			if(PerlCommand->CopyError.m_iErrorCode)
				CommandToExecute->Error.SetError(PerlCommand->CopyError.m_sErrorService, PerlCommand->CopyError.m_sErrorClass, PerlCommand->CopyError.m_iErrorCode, PerlCommand->CopyError.m_sErrorInfo);
		}

	}
	delete PerlCommand;
	if(PerlInUseCounter)
		PerlInUseCounter--;

	//PM Bug #30 we comment this line because if the retcode is ok, we should not have any error 
//	if(CommandToExecute->Error.m_iErrorCode != 0)
//		RetCode = 0;

	return RetCode;

}


// return -1 if memory error
// return 0 if the running of method failed
// return 1 in case of success
int NvSession::JavaRunMethod(const char* Method, NvCommand *CommandToExecute)
{
	MyNvJavaCommand *JavaCommand = NVNEW MyNvJavaCommand;
	if(JavaCommand == NULL)
		return -1;

	JavaCommand->m_pCommand = (void*)CommandToExecute;

//	int RetCode = GetMain()->JavaInterface.ThreadRunMethod(Method, (void*)(JavaCommand), true);
	int RetCode = GetMain()->JavaInterface.RunMethod(Method, (void*)(JavaCommand));
	if(!RetCode)
	{
		//PM Bug #30 If there is no error code get the last error code in copy error
		if(CommandToExecute->Error.m_iErrorCode == 0)
		{
			if(JavaCommand->CopyError.m_iErrorCode)
				CommandToExecute->Error.SetError(JavaCommand->CopyError.m_sErrorService, JavaCommand->CopyError.m_sErrorClass, JavaCommand->CopyError.m_iErrorCode, JavaCommand->CopyError.m_sErrorInfo);
		}

	}
	delete JavaCommand;

	//PM Bug #30 we comment this line because if the retcode is ok, we should not have any error 
	//if(CommandToExecute->Error.m_iErrorCode != 0)
	//	RetCode = 0;

	return RetCode;

}


// return -1 if memory error
// return 0 if the running of method failed
// return 1 in case of success
int NvSession::DotnetRunMethod(const char* Method, NvCommand *CommandToExecute)
{
	MyNvDotnetCommand *DotnetCommand = NVNEW MyNvDotnetCommand;
	if(DotnetCommand == NULL)
		return -1;

	DotnetCommand->m_pCommand = (void*)CommandToExecute;

//	int RetCode = GetMain()->DotnetInterface.ThreadRunMethod(Method, (void*)(DotnetCommand), true);
	int RetCode = GetMain()->DotnetInterface.RunMethod(Method, (void*)(DotnetCommand));
	if(!RetCode)
	{
		//PM Bug #30 If there is no error code get the last error code in copy error
		if(CommandToExecute->Error.m_iErrorCode == 0)
		{
			if(DotnetCommand->CopyError.m_iErrorCode)
				CommandToExecute->Error.SetError(DotnetCommand->CopyError.m_sErrorService, DotnetCommand->CopyError.m_sErrorClass, DotnetCommand->CopyError.m_iErrorCode, DotnetCommand->CopyError.m_sErrorInfo);
		}

	}
	delete DotnetCommand;

	//PM Bug #30 we comment this line because if the retcode is ok, we should not have any error 
//	if(CommandToExecute->Error.m_iErrorCode != 0)
//		RetCode = 0;

	return RetCode;

}
	

// return 1 if OK, 0 for general error, -1 if bad user, -2 if cannot get the security context, -3 if bad password, -4 if application not enabled, -100 if error in the init procedure, -5 if password must be changed, -6 if password format is bad
// if bSSO flag is set, the user is ever authenticated via SSO
int NvSession::Login(const char *UserName, const char *UserPassword, const char *AppliName, const char *Type, const char *NewPassword, const char *NewPasswordConfirm, bool bWithNewPassword, bool bWithNewPasswordConfirm, int TimeOut, const char *Hash, const char *Language, std::map<std::string, std::string> *mParameters, NvCommand *SenderCommand, bool bSSO, const char *SSOUser, const char *SSODomain, NvIdentToken *IdentToken)
{
	NV_LOCKOBJECT(this);		// Only one command at a time is allowed on a session
	if(!Initialized)
	{
		NV_UNLOCKOBJECT(this);
		return 0;
	}

	// Set the creator ip address
	if(SenderCommand != NULL)
		m_sCreatorClientAddress = SenderCommand->m_sClientAddress;
	else
		m_sCreatorClientAddress = "";

	string AppliNameHere = NV_APPLICATION_DEFAULT_NAME;
	if(AppliName != NULL)
		AppliNameHere = AppliName;
	if(!AppliNameHere.length())
		AppliNameHere = NV_APPLICATION_DEFAULT_NAME;
	string UserNameHere = NV_SESSION_DEFAULT_USER;
	if(UserName != NULL)
		UserNameHere = UserName;
	if(!UserNameHere.length())
		UserNameHere = NV_SESSION_DEFAULT_USER;
	// If ident token, user is taken from it
	if(IdentToken != NULL && IdentToken->PrivateToken.length())
	{
		if(IdentToken->Application != AppliNameHere)
			return -2;	// App doesn't match
		if(m_sCreatorClientAddress.length() && (IdentToken->IpOrigin != m_sCreatorClientAddress))
			return -2;	// Ip doesn't match
		// The user name is taken from Ident token
		UserNameHere = IdentToken->User;
	}

	string UserHereForCheck = UserNameHere;
	CleanString(UserHereForCheck, true);
	MakeLower(UserHereForCheck);

	if((SenderCommand != NULL) && ((SenderCommand->m_iCommandSource == NV_SOURCE_CLIENT) || (SenderCommand->m_iCommandSource == NV_SOURCE_BROWSER)))
	{
		if((UserHereForCheck == "nvadmin") && (!GetMain()->GetRemoteAdmin()))
		{
			// If sender is not local machine we reject the login as bad user
			string HostHere = SenderCommand->m_sHost;
			CleanString(HostHere);
			MakeUpper(HostHere);
			string::size_type index = HostHere.find(':');
			if(index != string::npos)
				HostHere = HostHere.substr(0, index);
			if((HostHere != "LOCALHOST") && (HostHere != "127.0.0.1") && (HostHere != "::1") && (m_sCreatorClientAddress != "127.0.0.1") && (m_sCreatorClientAddress != "::1"))
			{
				// We must check the ip address
				// if the sender ip adress is empty we do not accept
				if(!m_sCreatorClientAddress.length())
					return -1;
				
				vector<string> hosts;
				MyGetHostByName(GetMain()->GetMachineName().c_str(), hosts);
				bool Match = false;
				int i = 0;
				for(i=0;i<(int)(hosts.size());i++)
				{
					string MachineIpAdress = hosts[i];
					CleanString(MachineIpAdress);
					if(m_sCreatorClientAddress == MachineIpAdress)
					{
						Match = true;
						break;
					}
				}

/*				struct hostent *hp;
				hp = gethostbyname(GetMain()->GetMachineName().c_str());
				if(hp == NULL)
					return -1;
				struct in_addr addr;
				bool Match = false;
				int i = 0;
				while (hp->h_addr_list[i] != 0)
				{
					addr.s_addr = *(u_long *) hp->h_addr_list[i++];
					string MachineIpAdress = inet_ntoa(addr);
					CleanString(MachineIpAdress);
					if(m_sCreatorClientAddress == MachineIpAdress)
					{
						Match = true;
						break;
					}
				}
*/
				if(!Match)
					return -1;
			}
		}
	}
	
	// First log out if ever logged
	if(Application != NULL)
		Logout();

	// Set session type
	m_sSessionType = Type;

	// Set default language
	if(Language != NULL)
		m_sLanguage = Language;

	Application = NULL;
	SetAppName("");
	SetUserName(NV_SESSION_DEFAULT_USER);
	SetHash("");
	m_sPassword = "";
	SetActualCommand("");
	SetOriginalCommand("");
	SetNumCommands();

	// Try to get the application from the global application list
	Application = GetMain()->Applications.GetApplication(AppliName);

	// The application doesn't exist
	if(Application == NULL)
	{
		NV_UNLOCKOBJECT(this);
		return -4;
	}

	bool bControlPassword = true;
	if(m_bFromTransaction)
		bControlPassword = false;

	if(bSSO)
	{
		// If we got an SSO authentication, we don't have to check the paswword
		bControlPassword = false;

		// Set the Nirva user as SSO user if required
		if(Application->GetNirvaIsSsoUser())
			UserNameHere = SSOUser;
	}

	string UserFullName = "";
	int RetVal = Application->Login(m_sIdent.c_str(), UserNameHere.c_str(), UserPassword, NewPassword, NewPasswordConfirm, bWithNewPassword, bWithNewPasswordConfirm, &m_oPermissions, UserFullName, bControlPassword, IdentToken);
	if(RetVal != 1)
	{
		GetMain()->Applications.FreeApplication(Application->GetName().c_str());
		Application = NULL;
		NV_UNLOCKOBJECT(this);
		return RetVal;
	}

	SetAppName(Application->GetName().c_str());
	m_sVariableIdent = Application->GetVariableIdent();
	
	SetUserName(UserNameHere.c_str());
	SetUserFullName(UserFullName.c_str());
	if(UserHereForCheck == "nvadmin")
		m_bAdministrator = true; // Administrator user
	if(SSOUser != NULL)
		SetSSOUser(SSOUser);
	if(SSODomain != NULL)
		SetSSODomain(SSODomain);
	// Keep the password for use with requests or session:create
	// but only if this is not a connection by token (in this case we don't have the password)
	if((IdentToken == NULL) || !(IdentToken->PrivateToken.length()))
	{
		if(UserPassword != NULL)
		{
			m_sPassword = UserPassword;
			if(bWithNewPassword)
				m_sPassword = NewPassword;
		}
	}
	if(Hash != NULL)
		SetHash(Hash);

	if(TimeOut != 0)
		SetTimeOut(TimeOut);
	else
		SetTimeOut(Application->GetDefaultTimeOut());

	
	NV_UNLOCKOBJECT(this);

	// We call here the the session open procedure if there is one
	if(m_sOpenProc.length())
	{
//		string ProcFileName = GetProcedureFileName(m_sOpenProc);
//		if((ProcFileName.length()) && FileExist(ProcFileName.c_str()))
		if(IsSessionAttachConsoleCommand(SenderCommand))
		{
			SessionAttachConsole(SenderCommand);
		}
		NvCommand CommandForInit;
		CommandForInit.m_iCommandSource = NV_SOURCE_SERVER;
		CommandForInit.m_sService = "SYSTEM";
		CommandForInit.m_sClass = "MISC";
		CommandForInit.m_sCommand = "NOP";
		CommandForInit.m_sInContainer = NV_CONTAINER_DEFAULT_NAME;
		CommandForInit.m_sOutContainer = NV_CONTAINER_DEFAULT_NAME;
		if(SenderCommand != NULL)
			CommandForInit.m_sClientAddress = SenderCommand->m_sClientAddress;
		else
			CommandForInit.m_sClientAddress = "";
		if(SenderCommand != NULL)
			CommandForInit.m_sHost = SenderCommand->m_sHost;
		else
			CommandForInit.m_sHost = "";
		if(mParameters != NULL)
		{
			// Set the parameters from original command
			map<string,string>::iterator pos;
			for(pos = mParameters->begin(); pos != mParameters->end(); ++pos)
			{
				CommandForInit.Parameters.SetValue(pos->first, pos->second);
#ifdef _DEBUG
				string ParamName = pos->first;
				string ParamValue = pos->second;
				;
#endif
			}
		}
		CommandForInit.Parameters.SetValue("NV_PROC", m_sOpenProc);
		CommandForInit.Parameters.SetValue("NV_POST_PROC", "");
		CommandForInit.m_sLanguage = m_sLanguage;
		if(SenderCommand != NULL)
			CommandForInit.Parameters.Add(&(SenderCommand->HttpInHeaders));

		bInOpenProc = true;
		mOpenParameters = mParameters;
		bool CommandRetVal = true;
		try
		{
			CommandRetVal = Command(&CommandForInit);
		}
		catch(...)
		{
			GetMain()->IncNumExceptions();
			string Message = CommandForInit.m_sService;
			Message += ":";
			Message += CommandForInit.m_sClass;
			Message += ":";
			Message += CommandForInit.m_sCommand;
			CommandForInit.Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_EXCEPTION, Message);
			CommandRetVal = false;
			GetMain()->Logs.Write("Exceptions", "Exception in init session procedure", Message.c_str(), NV_LOG_ERROR);
		}
		mOpenParameters = NULL;
		bInOpenProc = false;
		if(!CommandRetVal)
		{
			// If error in the open procedure, we cannot login

			//In any case we must send a NV_EXIT_SESSION to connected services here (bug redmine #79)
			int i;
			NV_LOCKLOCK(&ProtectInUse);	// The access to m_oServices is protected by the InUse Lock for the session reporting
			NvObjStringList m_oServicesBis;		// We work with a copy becacuse of possible deadlock problem between service lock and protectinuse lock
			m_oServicesBis.CopyFrom(&m_oServices);
			m_oServices.RemoveAll();
			NV_UNLOCKLOCK(&ProtectInUse);
			for(i=0; i<m_oServicesBis.GetSize(); i++)
			{
				string ServiceInList = m_oServicesBis.GetValue(i);
				NvService *Service = GetMain()->Services.GetService(ServiceInList.c_str(), true, true);
				if(Service == NULL)
					continue;
				string ServiceNameHere = Service->GetName();
				NvCommand RealCommand;
				RealCommand.m_pSession = this;
				MyNvServiceCommand Command;
				if(Service->m_iLanguage == LANGUAGE_JAVA)
					Command.i_Encoding = ENCODING_UTF8;
				else if(Service->m_iLanguage == LANGUAGE_DOTNET)
					Command.i_Encoding = ENCODING_UTF8;
				else
					Command.i_Encoding = Service->m_iEncoding;
				Command.m_pCommand = &RealCommand;
				RealCommand.m_iCommandSource = NV_SOURCE_SERVER;
				RealCommand.m_sService = ServiceInList;
				RealCommand.m_sClass = "SYSTEM";
				RealCommand.m_sCommand = "NV_EXIT_SESSION";
				RealCommand.m_sLanguage = m_sLanguage;
				Service->Command(&Command);
				GetMain()->Services.FreeService(ServiceNameHere.c_str());
			}
			m_oServicesBis.RemoveAll();

			Application->Logout(m_sIdent.c_str());
			GetMain()->Applications.FreeApplication(GetAppName().c_str());
			Application = NULL;
			m_sPassword = "";
			SetHash("");
			m_bAdministrator = false;
			int RetVal = -100;
			if(SenderCommand == NULL)
			{
				if((CommandForInit.Error.m_sErrorService == "SYSTEM") && (CommandForInit.Error.m_sErrorClass == "SECURITY"))
				{
					if(CommandForInit.Error.m_iErrorCode == NV_SECURITY_BAD_USER)
						RetVal = -1;
					if(CommandForInit.Error.m_iErrorCode == NV_SECURITY_BAD_PASSWORD)
						RetVal = -3;
					if(CommandForInit.Error.m_iErrorCode == NV_SECURITY_CHANGE_PASSWORD)
						RetVal = -5;
					if(CommandForInit.Error.m_iErrorCode == NV_SECURITY_BAD_PASSWORD_SYNTAX)
						RetVal = -6;
				}
				else
					RetVal = -100;
			}
			else
			{
				if(CommandForInit.Error.m_iErrorCode)
					SenderCommand->Error.SetError(CommandForInit.Error.m_sErrorService, CommandForInit.Error.m_sErrorClass, CommandForInit.Error.m_iErrorCode, CommandForInit.Error.m_sErrorInfo);
				else
				{
					string Info = GetUserName();
					Info += " - ";
					Info += GetAppName();
					SenderCommand->Error.SetError("SYSTEM", "SESSION", NV_SESSION_OPENPROC_ERROR, Info);
				}
				RetVal = -100;
			}
			SetAppName("");
			SetUserName(NV_SESSION_DEFAULT_USER);
			return RetVal;
		}
	}

	if(!bHidden)
	{
		string Extra = GetAppName().c_str();
		Extra += ":";
		Extra += GetUserName();
		string Message = "Open ";
		Message += m_sSessionType;
		Message += " session";
		GetMain()->Logs.Write("Sessions", Message.c_str(), Extra.c_str(), NV_LOG_INFO, 1, m_sIdent.c_str(), "system");
	}

	return 1;
}
	

// Called internally from cleanup so no lock
void NvSession::Logout()
{

	if(!Initialized)
		return;


	if(!bHidden)
	{
		string Extra = GetAppName().c_str();
		Extra += ":";
		Extra += GetUserName();
		string Message = "Close ";
		Message += m_sSessionType;
		Message += " session";
		GetMain()->Logs.Write("Sessions", Message.c_str(), Extra.c_str(), NV_LOG_INFO, 1, m_sIdent.c_str(), "system");
	}

	if(Application != NULL)
	{
		// Unlock all eventual application locked objects by this session
		Application->Logout(m_sIdent.c_str());
		GetMain()->Applications.FreeApplication(GetAppName().c_str());
	}

	Application = NULL;
	SetAppName("");
	SetUserName(NV_SESSION_DEFAULT_USER);
	m_sPassword = "";
	SetHash("");
	m_bAdministrator = false;
}
	

bool NvSession::IsLogin()
{
	NV_LOCKOBJECT(this);		// Only one command at a time is allowed on a session

	if(!Initialized)
	{
		NV_UNLOCKOBJECT(this);
		return false;
	}

	if(Application == NULL)
	{
		NV_UNLOCKOBJECT(this);
		return false;
	}

	NV_UNLOCKOBJECT(this);
	return true;
}


bool NvSession::CheckPermission(NvCommand *CommandToExecute, const char *Service, const char *Permission, const char *Procedure)
{
	if(m_bAdministrator)
		return true;		// Nirva administrator can do everything


	string ServiceHere = Service;
	CleanString(ServiceHere);
	MakeUpper(ServiceHere);
	string PermissionHere = Permission;
	CleanString(PermissionHere);
	MakeUpper(PermissionHere);
	string PermissionToCheck = ServiceHere;
	if(PermissionToCheck.length())
		PermissionToCheck += ":";
	PermissionToCheck += PermissionHere;
	if(!m_oPermissions.KeyExist(PermissionToCheck))
	{
		if(CommandToExecute != NULL)
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_NOT_ALLOWED, PermissionToCheck);
		return false;
	}
	string ProcedureHere = "";
	if(Procedure != NULL)
		ProcedureHere = Procedure;
	if(CommandToExecute == NULL)
		ProcedureHere = "";
	if(ProcedureHere.length())
	{
		// First create or reset the boolean object NV_PERMISSION_OK in the input container
		InContainer->RemoveObject("NV_PERMISSION_OK");
		if(!InContainer->CreateObject("NV_PERMISSION_OK", NV_OBJTYPE_BOOLEAN))
		{
			if(CommandToExecute != NULL)
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "NV_PERMISSION_OK");
			return false;
		}
		NvObjBoolean *PermissionOk = (NvObjBoolean *)(InContainer->GetObject("NV_PERMISSION_OK"));
		PermissionOk->SetValue(true);

		if(!ExecuteProcedure(CommandToExecute, ProcedureHere))
			return false;
		
		PermissionOk = (NvObjBoolean *)(InContainer->GetObject("NV_PERMISSION_OK"));
		if(PermissionOk == NULL)
		{
			// The object has been removed by the procedure
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_NOT_ALLOWED, PermissionToCheck);
			return false;
		}
		if(!PermissionOk->GetValue())
		{
			// No permission
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_NOT_ALLOWED, PermissionToCheck);
			return false;
		}
		InContainer->RemoveObject("NV_PERMISSION_OK");
	}
	return true;
}



// Check that a list of strings doesn't contain a nirva command (attack)
bool NvSession::CheckCommandInjection(NvCommand *CommandToExecute, NvObjIndStringList *List)
{
	// Authorized if not external command
	if((CommandToExecute->m_iCommandSource != NV_SOURCE_BROWSER) && (CommandToExecute->m_iCommandSource != NV_SOURCE_CLIENT) && (CommandToExecute->m_iCommandSource != NV_SOURCE_SERVER))
		return true;
	// For system command, authorized except in open procedure
	if((CommandToExecute->m_iCommandSource == NV_SOURCE_SERVER) && !bInOpenProc)
		return true;

	// If check injection is disabled, we return immediatly
	if((Application != NULL) && (!Application->GetCheckInjection()))
		return true;

	map<string,string>::iterator pos,pos2;

	string Provistr;
	for(pos = List->m_mList.begin(); pos !=List->m_mList.end(); ++pos)
	{
		if(pos->first == "REFERER")	//special http header that contains the url
			continue;
		if(pos->second.find('=') == string::npos)
			continue;
		Provistr = pos->second;
		MakeUpper(Provistr);
		if(Provistr.find("NV_") == string::npos)
			continue;
		NvCommand SpecialCommand;	// used only to parse parameters
		SpecialCommand.ParseParameters(Provistr.c_str());
		if(!SpecialCommand.Parameters.GetSize())
			continue;
		for(pos2 = SpecialCommand.Parameters.m_mList.begin(); pos2 !=SpecialCommand.Parameters.m_mList.end(); ++pos2)
		{
			if(pos2->first.find("NV_CMD") != string::npos)
				return false;
			if(pos2->first.find("NV_COMMAND") != string::npos)
				return false;
			if(pos2->first.find("NV_CLASS") != string::npos)
				return false;
			if(pos2->first.find("NV_SERVICE") != string::npos)
				return false;
		}
	}
	return true;
}

bool NvSession::CheckOverMaxSession(NvCommand *CommandToExecute)
{
	if(!m_bOverMaxSession)
		return true;		// Not in over max session mode so it's OK
	
	if(!m_bAdministrator)
		return false;		// We must be logged as administrator

	SetTimeOut(180);
	SLEEP(1000);


	return true;
}


// Get a pointer to the requested security (system or current application)
// If the current application security points to another application security
// The corresponding application is considered to be in use and must be free
// by the FreeSecurity function. At this time, the AppName parameter contains the name of the
// security application
NvSecurity* NvSession::GetSecurity(NvCommand *CommandToExecute, string& SecuApp, bool& SystemSecurity)
{
	SecuApp = "";
	SystemSecurity = true;
	string System = CommandToExecute->Parameters.GetValue("SYSTEM");
	CleanString(System, true);
	MakeUpper(System);
	NvSecurity *CurrentSecurity = &(GetMain()->Security);
	if(System != "YES")
	{
		CurrentSecurity = Application->GetSecurity(SecuApp);
		SystemSecurity = false;
		if(!(Application->IsLocalSecurity()))
		{
			CurrentSecurity = NULL;
			string ActualSecuApp = Application->GetSecuApp();
			string::size_type index = ActualSecuApp.find('[');
			string ServerAdress = ActualSecuApp.substr(index+1);
			ActualSecuApp = ActualSecuApp.substr(0, index);
			CleanString(ActualSecuApp);
			index = ServerAdress.find(']');
			ServerAdress = ServerAdress.substr(0, index);
			CleanString(ServerAdress);
			bool bUseSsl = false;
			if(ServerAdress.length() >= 5)
			{
				string sUseSsl = ServerAdress.substr(ServerAdress.length()-5);
				MakeUpper(sUseSsl);
				if(sUseSsl == "(SSL)")
				{
					bUseSsl = true;
					ServerAdress = ServerAdress.substr(0, ServerAdress.length()-5);
				}
			}
			CleanString(ServerAdress);
			string Message = ActualSecuApp;
			if(!Message.length())
				Message = "NVDEF";
			Message += ": ";
			Message += ServerAdress;
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_OTHER_SERVER, Message);
			SystemSecurity = false;
			return NULL;
		}
		string SecurityService = Application->GetServiceSecurity();
		if(SecurityService.length())
		{
			// Security defined on a dedicated service
			CurrentSecurity = NULL;
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_DEDICATED_SERVICE, SecurityService);
			SystemSecurity = false;
			return NULL;
		}
	}

	if(CurrentSecurity == NULL)
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_BAD_APPLICATION, Application->GetSecuApp());
	
	return CurrentSecurity;
}

// Eventually free the corresponding application if AppName is not empty
void NvSession::FreeSecurity(string& SecuApp)
{
	if(SecuApp.length())
		GetMain()->Applications.FreeApplication(SecuApp.c_str());
}

// Get a session, service, application or system container
NvContainer* NvSession::GetContainer(string ContainerName, bool CreateIt, NvCommand *CommandToExecute)
{
	string::size_type index = ContainerName.find(':');
	if(index == string::npos)
	{
		if(CreateIt)
			return Containers.GetCreateContainer(ContainerName.c_str(), (void *)this);
		else
			return Containers.GetContainer(ContainerName.c_str(), (void *)this);
	}

	string sLevel = ContainerName.substr(0, index);
	ContainerName = ContainerName.substr(index+1);
	CleanString(sLevel, true);
	MakeUpper(sLevel);
	NvContainer* RetContainer = NULL;

	int Level = CONTAINER_LEVEL_APPLICATION;

	NvContainerList *ContainerList = NULL;
	if(Application != NULL)
		ContainerList = &(Application->Containers);
	if(sLevel == "SYSTEM")
		Level = CONTAINER_LEVEL_SYSTEM;
	else if(sLevel == "[NV_PARENT]")
		Level = CONTAINER_LEVEL_PARENT;
	else if(sLevel.length())
		Level = CONTAINER_LEVEL_SERVICE;

	if(Level == CONTAINER_LEVEL_SYSTEM)
		ContainerList = &(GetMain()->Containers);		
	if(Level == CONTAINER_LEVEL_SERVICE)
	{
		NvService *Service = GetMain()->Services.GetService(sLevel.c_str(), true, true);
		if(Service == NULL)
			return NULL;
		if(Service->GetProtectContainers())
		{
			// The servic econtainers must be accessed only by the service itself
			bool bAuthorize = false;
			if(CommandToExecute->m_iCommandSource == NV_SOURCE_SERVICE)
			{
				string ServiceNameHere = sLevel;
				NvService::CleanSrvName(ServiceNameHere);
				string ServiceNameSource = CommandToExecute->m_sSourceService;
				NvService::CleanSrvName(ServiceNameSource);
				if(ServiceNameHere == ServiceNameSource)
					bAuthorize = true;
			}
			if(!bAuthorize)
			{
				GetMain()->Services.FreeService(sLevel.c_str());
				return NULL;
			}
		}
		ContainerList = &(Service->Containers);		
	}
	if(Level == CONTAINER_LEVEL_PARENT)
	{
		// This is parent container level
		// This can occur when a named or normal session has been called by
		// a parent session. The called session can then directly access
		// the containers of its parent.
		// If there is no parent, the actual session container list is used
		NvSession* Parent = (NvSession*)(CommandToExecute->m_pParentSession);
		if(Parent == NULL)
			ContainerList = &Containers;		
		else
			ContainerList = &(Parent->Containers);
	}

	if(CreateIt)
		RetContainer = ContainerList->GetCreateContainer(ContainerName.c_str(), (void *)this);
	else
		RetContainer = ContainerList->GetContainer(ContainerName.c_str(), (void *)this);
	
	if(RetContainer == NULL)
		return NULL;

	return RetContainer;
}

// a container if it's an application, service or system container
void NvSession::FreeContainer(NvContainer *Container, string ContainerName, bool Force, NvCommand *CommandToExecute)
{
	if(Container == NULL)
		return;

	string::size_type index = ContainerName.find(':');
	if(index == string::npos)
		return;		// Nothing to do for a session container

	string sLevel = ContainerName.substr(0, index);
	ContainerName = ContainerName.substr(index+1);
	CleanString(sLevel, true);
	MakeUpper(sLevel);
	NvContainer* RetContainer = NULL;

	int Level = CONTAINER_LEVEL_APPLICATION;

	NvContainerList *ContainerList = NULL;
	if(Application != NULL)
		ContainerList = &(Application->Containers);
	if(sLevel == "SYSTEM")
		Level = CONTAINER_LEVEL_SYSTEM;
	else if(sLevel == "[NV_PARENT]")
		Level = CONTAINER_LEVEL_PARENT;
	else if(sLevel.length())
		Level = CONTAINER_LEVEL_SERVICE;

	NvService *Service = NULL;
	if(Level == CONTAINER_LEVEL_SYSTEM)
		ContainerList = &(GetMain()->Containers);		
	if(Level == CONTAINER_LEVEL_SERVICE)
	{
		Service = GetMain()->Services.GetService(sLevel.c_str(), false, true);	// No locking because ever locked
		if(Service == NULL)
			return;
		ContainerList = &(Service->Containers);		
	}
	if(Level == CONTAINER_LEVEL_PARENT)
	{
		// This is parent container level
		// This can occur when a named or normal session has been called by
		// a parent session. The called session can then directly access
		// the containers of its parent.
		// If there is no parent, the actual session container list is used
		NvSession* Parent = (NvSession*)(CommandToExecute->m_pParentSession);
		if(Parent == NULL)
			ContainerList = &Containers;		
		else
			ContainerList = &(Parent->Containers);
	}

	ContainerList->FreeContainer(ContainerName.c_str(), (void *)this, Force);
	if(Service != NULL)
		GetMain()->Services.FreeService(sLevel.c_str());	// We must free the service in this case
}

// Every Nirva client command goes here
bool NvSession::Command(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Check if this is an order to attach a console
	// And process it immediatly if yes
	// This command is executed outside the session command lock but it has its own lock
	if(IsSessionAttachConsoleCommand(CommandToExecute))
	{
		if(!SessionAttachConsole(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding))
			return false;
		return true;
	}

	// If it's a debug class and we are not in debug mode, we must
	// leave immediatly in order to save time
	CommandToExecute->bDebugCommand = false;
	if(CommandToExecute->m_sClass == "DEBUG")
	{
		if(!bDebug)
			return true;
		CommandToExecute->bDebugCommand = true;
		if((CommandToExecute->m_iCommandSource != NV_SOURCE_PROCEDURE) && (CommandToExecute->m_iCommandSource != NV_SOURCE_SERVICE) && (CommandToExecute->m_iCommandSource != NV_SOURCE_SERVER))
		{
			// Debug commands are only available from server, service or procedure
			string Message = "SYSTEM:DEBUG:";
			Message += CommandToExecute->m_sCommand;
			CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, Message);
			return false;
		}
	}

	// Check if the command has to be executed only in debug mode
	string sDebugOnly = CommandToExecute->Parameters.GetValue("NV_DEBUG_ONLY");
	CleanString(sDebugOnly);
	MakeUpper(sDebugOnly);
	if(sDebugOnly == "YES")
	{
		if(!bDebug)
			return true;
	}



	bool RetVal = true;

	if((Buffer != NULL) && BufferSize)
		Buffer[0] = '\0';

	// Used for recursive calls (because InContainer and OutContainer are defined at session level)
	NvContainer* LocalInContainer = NULL;
	NvContainer* LocalOutContainer = NULL;

	// First check if the required session is not another one
	// Changing session is possible from procedure or service
	// Eventually get some command parameters from an indexed string list object
	if(CommandToExecute->Parameters.KeyExist("NV_PARAM"))
	{
		string NvParam = CommandToExecute->Parameters.GetValue("NV_PARAM");
		if(NvParam.length())
		{
			// Saves the actual input container
			LocalInContainer = InContainer;
			InContainer = GetContainer(CommandToExecute->m_sInContainer.c_str(), true, CommandToExecute);
			if(CheckInputContainer(CommandToExecute))
			{
				NvObjIndStringList* Params = (NvObjIndStringList*)GetObjectCheckType(CommandToExecute, InContainer, NvParam.c_str(), NV_OBJTYPE_INDSTRINGLIST);
				if(Params != NULL)
				{
					map<string,string>::iterator pos;
					bool bHasVarIdent = false;
					string sVarIdent = m_sVariableIdent;
					int iVarSepLength = (int)(sVarIdent.length());
					for(pos = Params->m_mList.begin(); pos != Params->m_mList.end(); ++pos)
					{
						string ParaName = pos->first.c_str();
						CleanString(ParaName, true);
						if (!ParaName.length())
							continue;
						string ParaValue = pos->second.c_str();
						if(iVarSepLength && (ParaValue[0]==sVarIdent[0]) && (strstr(ParaValue.c_str(), sVarIdent.c_str()) == ParaValue))
							ParaValue = CommandToExecute->GetVariable(ParaValue, &m_oVariables, sVarIdent.c_str());
						CommandToExecute->Parameters.SetValue(ParaName, ParaValue);
						if(!bHasVarIdent)
						{
							if(CommandToExecute->Parameters.KeyExist("NV_VAR_IDENT"))
							{
								sVarIdent = CommandToExecute->Parameters.GetValue("NV_VAR_IDENT");
								CleanString(sVarIdent, true);
								iVarSepLength = (int)(sVarIdent.length());
								bHasVarIdent = true;
							}
						}
					}
				}
				FreeContainer(InContainer, CommandToExecute->m_sInContainer.c_str(), false, CommandToExecute);
				InContainer = LocalInContainer;
			}
		}
	}

	if(bDebug)
		DebugDisplayParameters(CommandToExecute);

	if(!CommandToExecute->bFastMode)
	{
		// PM Modif, the source SERVER has nothing to do here because a SERVER command never calls another session
//		if((CommandToExecute->m_iCommandSource == NV_SOURCE_PROCEDURE) || (CommandToExecute->m_iCommandSource == NV_SOURCE_SERVICE) || (CommandToExecute->m_iCommandSource == NV_SOURCE_SERVER))
		if((CommandToExecute->m_iCommandSource == NV_SOURCE_PROCEDURE) || (CommandToExecute->m_iCommandSource == NV_SOURCE_SERVICE) || (CommandToExecute->m_iCommandSource == NV_SOURCE_SERVER))
		{
			string SessionId = CommandToExecute->Parameters.GetValue("NV_SESSION_ID");
			CleanString(SessionId);
			if(SessionId.length() && (SessionId[0] == '.'))
				SessionId = "";
			MakeUpper(SessionId);
			if(SessionId.length())
			{
				// A specific session has been requested
				// Remove it otherwise we loop
				CommandToExecute->Parameters.Remove("NV_SESSION_ID");
				if(SessionId != m_sIdent)
				{
					// This is really an alternate session
					// let's check if it exists
					NvSession *Session = GetMain()->Sessions.GetSession(SessionId.c_str());
					if(Session == NULL)
					{
						// The session doesn't exist
						CommandToExecute->Error.SetError("SYSTEM", "SESSION", NV_SESSION_NOTEXIST, SessionId);
						return false;
					}
					else
					{
						// Yes, the session exist so we give it the control
						CommandToExecute->m_pParentSession = this;
						// We must lock if the thread doesn't already own the lock
						bool bDoLocking = false;
						Session->ProtectInUse.Lock();
						long ThreadHandle = Session->m_iThreadHandleOwner;
						Session->ProtectInUse.Unlock();
						if(ThreadHandle != NvGetCurrentThreadId())
							bDoLocking = true;
						if(bDoLocking)
						{
							// The owner is not this thread so we must lock
							NV_LOCKOBJECT(Session);	
							// Here we got the lock
							Session->ProtectInUse.Lock();
							Session->m_iThreadHandleOwner = NvGetCurrentThreadId();
							Session->ProtectInUse.Unlock();
						}
						NvCommand *PreviousOriginalCommand = Session->OriginalCommand;
						Session->OriginalCommand = OriginalCommand;
						string PreviousOriginalCommandTxt = Session->GetOriginalCommand();
						Session->SetOriginalCommand(GetOriginalCommand().c_str());
						// We must lock the session before because since the command comes from a procedure
						// this is not done in this in InitSessionCommand
						try
						{
							RetVal = Session->Command(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
						}
						catch(...)
						{
							GetMain()->IncNumExceptions();
							string Message = CommandToExecute->m_sService;
							Message += ":";
							Message += CommandToExecute->m_sClass;
							Message += ":";
							Message += CommandToExecute->m_sCommand;
							CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_EXCEPTION, Message);
							GetMain()->Logs.Write("Exceptions", "Exception in session command", Message.c_str(), NV_LOG_ERROR);
							RetVal = false;
						}
						Session->OriginalCommand = PreviousOriginalCommand;
						Session->SetOriginalCommand(PreviousOriginalCommandTxt.c_str());
						// Unlock the session
						if(bDoLocking)
						{
							Session->ProtectInUse.Lock();
							Session->m_iThreadHandleOwner = 0;
							Session->ProtectInUse.Unlock();
							NV_UNLOCKOBJECT(Session);
						}
						GetMain()->Sessions.FreeSession(Session->m_sIdent.c_str());
						CommandToExecute->m_pParentSession = NULL;
						return RetVal;
					}
				}
			}
			else
			{
				string SessionName = CommandToExecute->Parameters.GetValue("NV_SESSION_NAME");
				CleanString(SessionName);
				MakeLower(SessionName);
				if(SessionName.length())
				{
					// Remove it otherwise we loop
					CommandToExecute->Parameters.Remove("NV_SESSION_NAME");
					// try to see if it's a SESSION:FREE_NAMED command
					bool bFromFreeNamed = false;
					if((CommandToExecute->m_sCommand == "FREE_NAMED") && (CommandToExecute->m_sClass == "SESSION") && (CommandToExecute->m_sService == "SYSTEM"))
						bFromFreeNamed = true;

					// try to see if it's a SESSION:GET_NAMED command
					bool bFromGetNamed = false;
					if((CommandToExecute->m_sCommand == "GET_NAMED") && (CommandToExecute->m_sClass == "SESSION") && (CommandToExecute->m_sService == "SYSTEM"))
						bFromGetNamed = true;


					// This is really an alternate session
					// let's check if it exists
					NvSession *Session = GetMain()->Sessions.GetSessionByName(SessionName.c_str(), this, CommandToExecute);
					if(Session == NULL)
					{
						// The session doesn't exist
						CommandToExecute->Error.SetError("SYSTEM", "SESSION", NV_SESSION_NOTEXIST, SessionName);
						return false;
					}
					else
					{
						// Yes, the session exist,
						// Let's verify it's not our actual session
						// and give it the control
						if(Session != this)
						{
							try
							{
								RetVal = Session->Command(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
							}
							catch(...)
							{
								GetMain()->IncNumExceptions();
								string Message = CommandToExecute->m_sService;
								Message += ":";
								Message += CommandToExecute->m_sClass;
								Message += ":";
								Message += CommandToExecute->m_sCommand;
								CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_EXCEPTION, Message);
								GetMain()->Logs.Write("Exceptions", "Exception in session command", Message.c_str(), NV_LOG_ERROR);
								RetVal = false;
							}
							// Free the named session only if it's not a manual get to the named session
							if(!bFromGetNamed)
								GetMain()->Sessions.FreeNamedSession(SessionName, this, CommandToExecute);
							// Free again if it's a manual request to free
							if(bFromFreeNamed)
								GetMain()->Sessions.FreeNamedSession(SessionName, this, CommandToExecute);
							return RetVal;
						}
					}
				}
			}
		}
	}


	// The command is to be processed by this session

	CommandToExecute->m_pSession = this;
	ActualCommand = NULL;
	
	InitSessionCommand(CommandToExecute);

	SetActualCommand("");
	
	string OldErrorProcName = m_sErrorProcName;
	if(CommandToExecute->Parameters.KeyExist("NV_ERROR_PROC"))
		m_sErrorProcName = CommandToExecute->Parameters.GetValue("NV_ERROR_PROC");

	NvRequest *Request = NULL;
	if(!CommandToExecute->bFastMode)
	{
		string RequestName = CommandToExecute->Parameters.GetValue("NV_REQUEST");
		CleanString(RequestName);
		MakeUpper(RequestName);
		if(RequestName.length())
		{
			// Check if this is a close command
			bool bClose = false;
			if((CommandToExecute->m_sClass == "REQUEST") && (CommandToExecute->m_sCommand == "CLOSE"))
				bClose = true;

			if(!bClose)
			{
				// Distant server
				Request = Requests.GetRequest(RequestName.c_str());
				if(Request == NULL)
				{
					CommandToExecute->Error.SetError("SYSTEM", "REQUEST", NV_REQUEST_BAD_REQUEST_IDENTIFIER, RequestName);
					// Lets process error
					OnCommandError(CommandToExecute, OldErrorProcName);
					// Free session
					CleanupSessionCommand(CommandToExecute, false);	// Do not pop comamnd stack here because not yet pushed
					return false;
				}
				Request->Session = this;
				CommandToExecute->Parameters.Remove("NV_REQUEST");
			}
		}
	}

	// Add to the command stack
	PushCommandStack(CommandToExecute, Request);

	if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT) || (CommandToExecute->m_iCommandSource == NV_SOURCE_SERVER))
		ActualSystemLogLevel = GetMain()->Logs.GetLevel("System"); // Update that when this is an external command

	// Lets write the command to the system log
	bool IsLogWriteCommand = false;
	if((ActualSystemLogLevel>0) && !bHidden && !CommandToExecute->bDebugCommand)
	{
		if(CommandToExecute->m_sService == "SYSTEM")
			if(CommandToExecute->m_sClass == "LOG")
				if(CommandToExecute->m_sCommand == "WRITE")
					IsLogWriteCommand = true;
		
		if(!IsLogWriteCommand)
		{
			bool bIsOriginalCommand = false;
			if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT) || (CommandToExecute->m_iCommandSource == NV_SOURCE_SERVER))
				bIsOriginalCommand = true;
			if(bIsOriginalCommand || (ActualSystemLogLevel>1))
			{
				string Message = "";
				if(ActualSystemLogLevel > 2)
					Message += "start ";
				Message += "command ";
				if(Application != NULL)
				{
					Message += Application->GetName();
					Message += ":";
				}
				Message += CommandToExecute->m_sService;
				Message += ":";
				Message += CommandToExecute->m_sClass;
				Message += ":";
				Message += CommandToExecute->m_sCommand;
				if(CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT)
					Message += "(C)";
				if(CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER)
					Message += "(B)";
				if(CommandToExecute->m_iCommandSource == NV_SOURCE_PROCEDURE)
					Message += "(P)";
				if(CommandToExecute->m_iCommandSource == NV_SOURCE_SERVICE)
					Message += "(S)";
				if(CommandToExecute->m_iCommandSource == NV_SOURCE_SERVER)
					Message += "(V)";
				if(Request != NULL)
				{
					Message += " -> ";
					Message += Request->m_sServerAddress;
				}

				string Extra = "";

				// Write the command itself
				GetMain()->Logs.Write("System", Message.c_str(), Extra.c_str(), NV_LOG_INFO, 2, m_sIdent.c_str(), "system");

				int LogLevelToWrite = 1;
				if(ActualSystemLogLevel >= 4)
					LogLevelToWrite = 4;

				if(LogLevelToWrite == 4)
				{
					// We write all the parameters to the log file
					Message = "parameters of command ";
					if(Application != NULL)
					{
						Message += Application->GetName();
						Message += ":";
					}
					Message += CommandToExecute->m_sService;
					Message += ":";
					Message += CommandToExecute->m_sClass;
					Message += ":";
					Message += CommandToExecute->m_sCommand;
					map<string,string>::iterator pos;
					for(pos = CommandToExecute->Parameters.m_mList.begin(); pos != CommandToExecute->Parameters.m_mList.end(); ++pos)
					{
						string ParameterNameHere = pos->first;
						CleanString(ParameterNameHere);
						MakeUpper(ParameterNameHere);
						string ParameterValueHere = pos->second;
						CleanString(ParameterValueHere);
						string Provistr = ParameterNameHere;
						// We must eliminate passwords if there are ones
						bool IsPassword = false;
						if(ParameterNameHere.find("PASSWORD") != string::npos)
							IsPassword = true;
						else if(ParameterNameHere.find("PWD") != string::npos)
							IsPassword = true;

						// Do not display also the unique service id for generating licenses
						if(CommandToExecute->m_sClass == "LICENSE")
							if(CommandToExecute->m_sService == "SYSTEM")
								if((CommandToExecute->m_sCommand == "SET") || (CommandToExecute->m_sCommand == "GET"))
									if(ParameterNameHere == "SERVICE_ID")
										continue;
						
						if(Extra.length())
							Extra += " ";
						Extra += ParameterNameHere;
						Extra += "=\"";
						if(IsPassword)
							Extra += "********";
						else
							Extra += ParameterValueHere;
						Extra += "\"";
					}
					GetMain()->Logs.Write("System", Message.c_str(), Extra.c_str(), NV_LOG_INFO, LogLevelToWrite, m_sIdent.c_str(), "system");
				}
			}
		}

	}

	// Write start command to attached console if there is one
	if(HasSessionAttachedConsole() && (!IsLogWriteCommand) && !bHidden && !CommandToExecute->bDebugCommand)
		WriteCommandToAttachedConsole(CommandToExecute, 1, Request);

	if(!CommandToExecute->bFastMode)
	{
		if(!Initialized)
		{
			CommandToExecute->Error.SetError("SYSTEM", "SESSION", NV_SESSION_NOT_INITIALIZED);
			// Lets process error
			OnCommandError(CommandToExecute, OldErrorProcName);
			// Free session
			CleanupSessionCommand(CommandToExecute);
			return false;
		}
		
		// Check if we are log in
		if(Application == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "SESSION", NV_SESSION_NOT_INITIALIZED);
			// Lets process error
			OnCommandError(CommandToExecute, OldErrorProcName);
			// Free session
			CleanupSessionCommand(CommandToExecute);
			return false;
		}
	}

	// Verify security command filter
	if(!CheckAccessCommand(CommandToExecute))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_NOT_ALLOWED);
		// Lets process error
		OnCommandError(CommandToExecute, OldErrorProcName);
		// Free session
		CleanupSessionCommand(CommandToExecute);
		return false;
	}

	if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT) || (CommandToExecute->m_iCommandSource == NV_SOURCE_SERVER) || (CommandToExecute->m_bInitNamedSessionCommand))
	{
		// We reset the flag for init named session
		CommandToExecute->m_bInitNamedSessionCommand = false;

		// Set some original command parameters as variables
		string KeepVariables = CommandToExecute->Parameters.GetValue("NV_KEEP_VAR");
		CleanString(KeepVariables);
		MakeUpper(KeepVariables);
		if(KeepVariables != "YES")
			m_oVariables.RemoveAll();
		m_oVariables.SetValue("NV_NIRVADIR", GetMain()->GetNirvaDir());
		m_oVariables.SetValue("NV_HOME_LOGDIR", GetMain()->GetLogHomeDir());
		m_oVariables.SetValue("NV_HOME_REGDIR", GetMain()->GetRegistryHomeDir());
		m_oVariables.SetValue("NV_HOME_APPWORKDIR", GetMain()->GetAppWorkHomeDir());
		string sPlatform = "WIN32";
#ifdef WIN32
#ifdef X64
		sPlatform = "WIN64";
#endif
#endif
#ifdef AIX
		sPlatform = "AIX";
#endif
#ifdef LINUX
		sPlatform = "LINUX";
#ifdef X64
		sPlatform = "LINUX64";
#endif
#endif
#ifdef HPUX
		sPlatform = "HPUX";
#endif
#ifdef HPUXI
		sPlatform = "HPUXI";
#endif
#ifdef SOLARIS
		sPlatform = "SOLARIS";
#endif
		m_oVariables.SetValue("NV_PLATFORM", sPlatform);
		m_oVariables.Add(&(OriginalCommand->HttpInHeaders));
		m_oVariables.Remove("NV_PASSWORD");
		m_oVariables.Remove("NV_NEW_PASSWORD");
		m_oVariables.Remove("NV_NEW_PASSWORD_CONFIRM");
		m_oVariables.Add(&(OriginalCommand->Parameters));
		m_oVariables.SetValue("NV_APPLICATION", GetAppName());
		m_oVariables.SetValue("NV_USER", GetUserName());
		m_oVariables.SetValue("NV_SSO_USER", GetSSOUser());
		m_oVariables.SetValue("NV_SSO_DOMAIN", GetSSODomain());
		m_oVariables.SetValue("NV_SESSION_ID", m_sIdent);
		string ParentIdent = "";
		if(CommandToExecute->m_pParentSession != NULL)
		{
			NvSession *ParentSession = (NvSession*)(CommandToExecute->m_pParentSession);
			ParentIdent = ParentSession->m_sIdent;
		}
		m_oVariables.SetValue("NV_PARENT_SESSION_ID", ParentIdent);
		m_oVariables.SetValue("NV_SERVICE", OriginalCommand->m_sService);
		m_oVariables.SetValue("NV_CLASS", OriginalCommand->m_sClass);
		m_oVariables.SetValue("NV_COMMAND", OriginalCommand->m_sCommand);
		m_oVariables.SetValue("NV_CONTAINER", OriginalCommand->m_sInContainer);
		m_oVariables.SetValue("NV_IN_CONTAINER", OriginalCommand->m_sInContainer);
		m_oVariables.SetValue("NV_OUT_CONTAINER", OriginalCommand->m_sOutContainer);
		string MachineNameEncoded = GetMain()->GetMachineName();
		EncodeString(MachineNameEncoded, true, GetMain()->IsUnicode());
		m_oVariables.SetValue("NV_HOSTNAME", MachineNameEncoded);
		m_oVariables.SetValue("NV_LANGUAGE", CommandToExecute->m_sLanguage);
		if(GetMain()->IsUnicode())
			m_oVariables.SetValue("NV_UNICODE", "YES");
		else
			m_oVariables.SetValue("NV_UNICODE", "NO");
		if(bDebug)
			m_oVariables.SetValue("NV_DEBUG", "YES");
		else
			m_oVariables.SetValue("NV_DEBUG", "NO");
		m_oVariables.SetValue("NV_CLIENT_IP", CommandToExecute->m_sClientAddress);
		m_oVariables.SetValue("NV_CLIENT_CREATOR_IP", m_sCreatorClientAddress);
		m_oVariables.SetValue("NV_HTTP_HOST", CommandToExecute->m_sHost);
	
		// Do some more security checking
		if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT))
		{
			// Check if we have right to access this session
			if(!Application->GetAllowNotOwnedSession())
			{
				if(!CheckPermission(NULL, "SYSTEM", "ACCESS_NOT_OWNED_SESSIONS"))
				{
					// The user doesn't have authorisation to access not owned sessions
					// Verfiy if we give access or not
					bool bHasAccess = false;
					if(!m_sCreatorClientAddress.length())
						bHasAccess = true;		// Give access if the creator IP is not known
					if(!CommandToExecute->m_sClientAddress.length())
						bHasAccess = true;		// Give access if the client IP is not known
					if((CommandToExecute->m_sClientAddress == "127.0.0.1") || (CommandToExecute->m_sClientAddress == "::1"))
						bHasAccess = true;		// Give access if the command comes from a local client
					if(CommandToExecute->m_sClientAddress == m_sCreatorClientAddress)
						bHasAccess = true;		// Give access if the command and client addresses are identical
					if(!bHasAccess)
					{
						// No access to this session
						CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_ACCESS_NOT_OWNED_SESSION);
						// Lets process error
						OnCommandError(CommandToExecute, OldErrorProcName);
						// Free session
						CleanupSessionCommand(CommandToExecute);
						return false;
					}
				}
			}
		}
		// Check if variables values doesn't contain any Nirva commands
		if(!CheckCommandInjection(CommandToExecute, &m_oVariables))
		{
			// We have detected a command inside a value
			CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_COMMAND_IN_VALUE);
			// Lets process error
			OnCommandError(CommandToExecute, OldErrorProcName);
			// Free session
			CleanupSessionCommand(CommandToExecute);
			return false;
		}
	}

	// Check if we are in over max session mode (only few operations allowed in this mode)
	if(!CheckOverMaxSession(CommandToExecute))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_NOT_ALLOWED);
		// Lets process error
		OnCommandError(CommandToExecute, OldErrorProcName);
		// Free session
		CleanupSessionCommand(CommandToExecute);
		return false;
	}

	// Get eventual XML input to be transformed into NIRVA objects from the input container
	if((CommandToExecute->iXmlInputType) && ((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT)))
	{
		// Saves the actual input container
		LocalInContainer = InContainer;
		InContainer = GetContainer(CommandToExecute->m_sInContainer.c_str(), true, CommandToExecute);
		RetVal = SetXmlInput(CommandToExecute);
		FreeContainer(InContainer, CommandToExecute->m_sInContainer.c_str(), false, CommandToExecute);
		InContainer = LocalInContainer;
		if(!RetVal)
		{
			// Lets process error
			OnCommandError(CommandToExecute, OldErrorProcName);
			// Free session
			CleanupSessionCommand(CommandToExecute);
			return false;
		}
	}

	// Process procedures here
	// Procedures are given by the NVPROC parameter
	// several procedures can be chained separated by a semicolumn
	// A procedure is a file in the application procedure directory
	// if the procedure name is in parentheses, this is considered as
	// a system procedure that resides in the system procedure directory
	// A procedure file contains lines of commands. A command is on 1 line
	// A command can use session variables

	// In fact, we will create an NvCommand object for processing procedure commands
	string Procedures = CommandToExecute->Parameters.GetValue("NV_PROC");
	if(Procedures.length())
		ActualSystemLogLevel = GetMain()->Logs.GetLevel("System"); // Just give a chance to update that some time to time

	// Check if we have a trigger for external commands and add it to the list of procedures if yes
	if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT))
	{
		string Trigger = Application->GetTrigger();
		if(Trigger.length() && Application->GetUseTriggerProcedure())
		{
			if(Procedures.length())
				Trigger += ";";
			Trigger += Procedures;
			Procedures = Trigger;
		}
	}

	// Execute the procedures here but only if it's not a distant call
	if(Request == NULL)
		RetVal = ExecuteProcedure(CommandToExecute, Procedures, true, "", Buffer, BufferSize, DataSize, iEncoding);
	

	if(!RetVal)
	{
		// Lets process error
		OnCommandError(CommandToExecute, OldErrorProcName);
		// Free session
		CleanupSessionCommand(CommandToExecute);
		return false;
	}

	// Now process the real command

	LocalInContainer = GetContainer(CommandToExecute->m_sInContainer.c_str(), true, CommandToExecute);
	InContainer = LocalInContainer;
	LocalOutContainer = GetContainer(CommandToExecute->m_sOutContainer.c_str(), true, CommandToExecute);
	OutContainer = LocalOutContainer;

	if(!CommandToExecute->bFastMode)
	{
		if(!CheckInputContainer(CommandToExecute))
		{
			FreeContainer(LocalOutContainer, CommandToExecute->m_sOutContainer.c_str(), false, CommandToExecute);
			// Lets process error
			OnCommandError(CommandToExecute, OldErrorProcName);
			// Free session
			CleanupSessionCommand(CommandToExecute);
			return false;
		}
		if(!CheckOutputContainer(CommandToExecute))
		{
			FreeContainer(LocalInContainer, CommandToExecute->m_sInContainer.c_str(), false, CommandToExecute);
			// Lets process error
			OnCommandError(CommandToExecute, OldErrorProcName);
			// Free session
			CleanupSessionCommand(CommandToExecute);
			return false;
		}
		bool bClearInput = false;
		bool bClearOutput = false;
		if(CommandToExecute->Parameters.KeyExist("NV_IN_CONTAINER_CLEAR"))
		{
			string ClearContainer = CommandToExecute->Parameters.GetValue("NV_IN_CONTAINER_CLEAR");
			CleanString(ClearContainer);
			MakeUpper(ClearContainer);
			if(ClearContainer == "YES")
				bClearInput = true;
		}
		if(CommandToExecute->Parameters.KeyExist("NV_OUT_CONTAINER_CLEAR"))
		{
			string ClearContainer = CommandToExecute->Parameters.GetValue("NV_OUT_CONTAINER_CLEAR");
			CleanString(ClearContainer);
			MakeUpper(ClearContainer);
			if(ClearContainer == "YES")
				bClearOutput = true;
		}
		if(CommandToExecute->Parameters.KeyExist("NV_CONTAINER_CLEAR"))
		{
			string ClearContainer = CommandToExecute->Parameters.GetValue("NV_CONTAINER_CLEAR");
			CleanString(ClearContainer);
			MakeUpper(ClearContainer);
			if(ClearContainer == "YES")
			{
				bClearOutput = true;
				bClearInput = true;
			}
		}

		if(bClearInput)
			LocalInContainer->Clear();
		if(bClearOutput)
			LocalOutContainer->Clear();
	}
	
	string sActual = CommandToExecute->m_sService.c_str();
	sActual += ":";
	sActual += CommandToExecute->m_sClass.c_str();
	sActual += ":";
	sActual += CommandToExecute->m_sCommand.c_str();
	SetActualCommand(sActual.c_str());
	IncNumCommands();

	// Get eventual XML input to be transformed into NIRVA objects from the input container
	// PM Changed in v 2.6.009: called before executing procedures NV_PROC.
/*	if((CommandToExecute->iXmlInputType) && ((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT)))
	{
		if(!SetXmlInput(CommandToExecute))
		{
			FreeContainer(LocalInContainer, CommandToExecute->m_sInContainer.c_str(), false, CommandToExecute);
			FreeContainer(LocalOutContainer, CommandToExecute->m_sOutContainer.c_str(), false, CommandToExecute);

			// Lets process error
			OnCommandError(CommandToExecute, OldErrorProcName);
			// Free session
			CleanupSessionCommand(CommandToExecute);
			return false;
		}
	}
*/	
	// The ActualCommand member is used only in case of crash under UNIX
	ActualCommand = CommandToExecute;

#ifdef WIN32
	clock_t m_cStartProcessTime = 0;
	if(GetMain()->b_WithTime)
		m_cStartProcessTime = clock();
#else
	unsigned int m_cStartProcessTime = 0;
	if(GetMain()->b_WithTime)
		m_cStartProcessTime = GetMilliSeconds();
#endif


	try
	{
		if(Request == NULL)
		{
			if(CommandToExecute->m_sService == "SYSTEM")
			{
				// System command
				RetVal = ProcessSystemCommand(CommandToExecute,  Buffer, BufferSize, DataSize, iEncoding);
			}
			else
			{
				RetVal = ProcessServiceCommand(CommandToExecute,  Buffer, BufferSize, DataSize, iEncoding);
			}
		}
		else
		{
			RetVal = Request->RequestCommand(CommandToExecute,  Buffer, BufferSize, DataSize);
		}

	}
	catch(...)
	{
		// We must free the service if it's a service command
		GetMain()->IncNumExceptions();
		if(Request == NULL)
			if(CommandToExecute->m_sService != "SYSTEM")
				GetMain()->Services.FreeService(CommandToExecute->m_sService.c_str());
		string sMessageInfo = "*** Exception *** ";
		sMessageInfo += CommandToExecute->m_sService;
		sMessageInfo += ":";
		sMessageInfo += CommandToExecute->m_sClass;
		sMessageInfo += ":";
		sMessageInfo += CommandToExecute->m_sCommand;
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_EXCEPTION, sMessageInfo);
		GetMain()->Logs.Write("Exceptions", "Exception in session command", sMessageInfo.c_str(), NV_LOG_ERROR);
		RetVal = false;
	}
	ActualCommand = NULL;
	SetActualCommand("");

	if(RetVal)
	{
		// Execute post procedures if they exist
		Procedures = CommandToExecute->Parameters.GetValue("NV_POST_PROC");
		if(Procedures.length())
			ActualSystemLogLevel = GetMain()->Logs.GetLevel("System"); // Just give a change to update that some time to time
		if(Request == NULL)
			RetVal = ExecuteProcedure(CommandToExecute, Procedures, true, "", Buffer, BufferSize, DataSize, iEncoding);
	}

	bool bSetXmlOutput = false;
	if(CommandToExecute->iOutputType == OUTPUT_XML)
	{
		if(CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER)
		{
			// Set output type as xml except for REST connector
			bSetXmlOutput = true;
			if((CommandToExecute->m_sCommand == "REST") && (CommandToExecute->m_sClass == "MISC") && (CommandToExecute->m_sService == "SYSTEM"))
				bSetXmlOutput = false;
		}
		if(CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT)
			if(CommandToExecute->iXmlInputType)
				bSetXmlOutput = true;
	}
	if(!RetVal)
		bSetXmlOutput = false;

	bool bForceFree = false;
	if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT) || (CommandToExecute->m_iCommandSource == NV_SOURCE_SERVER))
		bForceFree = true;
	// Reset the in use of the container for this session if command comes from external
	FreeContainer(LocalInContainer, CommandToExecute->m_sInContainer.c_str(), bForceFree, CommandToExecute);

	// Prepare for setting XML output for debug only
	bool bDebugOnly = false;
	if(bDebug && !bSetXmlOutput)
	{
		if(CommandToExecute->Parameters.KeyExist("NV_XML_DEBUG"))
		{
			bSetXmlOutput = true;
			bDebugOnly = true;
		}
	}

	if(bSetXmlOutput)
	{
		// If command comes from browser, we have to eventually set an XML output
		// except if the command blocks it

		// We must get back the ouput container because it may have been nulled
		if(OutContainer == NULL)
			OutContainer = GetContainer(CommandToExecute->m_sOutContainer.c_str(), true, CommandToExecute);
		if(!SetXmlOutput(CommandToExecute, bDebugOnly))
		{
			if(!bDebugOnly)
				RetVal = false;
		}
	}
	FreeContainer(LocalOutContainer, CommandToExecute->m_sOutContainer.c_str(), bForceFree, CommandToExecute);

	InContainer = NULL;
	OutContainer = NULL;
	NV_LOCKLOCK(&ProtectInUse);
	tLastUseTime = time(NULL);
	NV_UNLOCKLOCK(&ProtectInUse);

	// reset the error to be sure to not display an error in case the return status is ok but the error information
	// contains error
	if(RetVal)
		CommandToExecute->Error.SetError();

	// We write the command to the console
	bool ConsoleDisplayIt = false;
	if((GetMain()->b_ConsoleMode) && (!IsLogWriteCommand) && !bHidden && !CommandToExecute->bDebugCommand)
	{
		ConsoleDisplayIt = true;
		if(!GetMain()->b_Verbose)
		{
			ConsoleDisplayIt = false;
			if((CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT) || (CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER))
				ConsoleDisplayIt = true;
			if((CommandToExecute->m_sService == "SYSTEM") && (CommandToExecute->m_sClass == "MISC") && (CommandToExecute->m_sCommand == "NOP"))
				ConsoleDisplayIt = false;
			if(CommandToExecute->Error.m_iErrorCode)
			{
				ConsoleDisplayIt = true;
				string sNoError = CommandToExecute->Parameters.GetValue("NV_NO_ERROR");
				CleanString(sNoError);
				MakeUpper(sNoError);
				if(sNoError == "YES")
					ConsoleDisplayIt = false;
			}
		}
	}
	if(CommandToExecute->bDebugCommand)
		ConsoleDisplayIt = false;
	if(ConsoleDisplayIt)
	{
		NV_LOCKLOCK(&ProtectConsoleLog);
		time_t tTime = time(NULL);
		struct tm tCurrentTime;
		localtime_r(&tTime, &tCurrentTime);
		printf("%02d:%02d:%02d",
						tCurrentTime.tm_hour,
						tCurrentTime.tm_min,
						tCurrentTime.tm_sec); 
		printf(" ");
		printf("%s", m_sIdent.c_str());
		printf(" ");
		printf("%10s",Application->GetName().c_str());
		printf(" ");
		printf("%s", CommandToExecute->m_sService.c_str());
		printf(":");
		printf("%s", CommandToExecute->m_sClass.c_str());
		printf(":");
		printf("%s", CommandToExecute->m_sCommand.c_str());
		if(Request != NULL)
		{
			printf(" -> ");
			printf("%s", Request->m_sServerAddress.c_str());
		}
		if(GetMain()->b_WithTime)
		{
#ifdef WIN32
			clock_t m_cProcessTime = clock();
			m_cProcessTime = m_cProcessTime - m_cStartProcessTime;
			int RealProcessTimeInMilliseconds = (int)((m_cProcessTime * 1000)/CLOCKS_PER_SEC);
#else
			unsigned int m_cProcessTime = GetMilliSeconds();
			m_cProcessTime = m_cProcessTime - m_cStartProcessTime;
			int RealProcessTimeInMilliseconds = m_cProcessTime;
#endif
			if(RealProcessTimeInMilliseconds > 9)
				RealProcessTimeInMilliseconds -= 9; // arondi
			printf(" (%.2f s)", ((float)RealProcessTimeInMilliseconds)/1000);
		}
/*		if(CommandToExecute->Error.m_iErrorCode)
		{
			string sNoError = CommandToExecute->Parameters.GetValue("NV_NO_ERROR");
			CleanString(sNoError);
			MakeUpper(sNoError);
			if(sNoError != "YES")
			{
				printf("\n*** Error %s:%s:%d:%s (%s)", CommandToExecute->Error.m_sErrorService.c_str(), CommandToExecute->Error.m_sErrorClass.c_str(), CommandToExecute->Error.m_iErrorCode, CommandToExecute->Error.GetErrorDesc().c_str(), CommandToExecute->Error.m_sErrorInfo.c_str());
			}
		}*/
		printf("\n");
		NV_UNLOCKLOCK(&ProtectConsoleLog);
	}

	
	// Lets process error
	OnCommandError(CommandToExecute, OldErrorProcName);

	// Write end command to attached console if there is one
	if(HasSessionAttachedConsole() && (!IsLogWriteCommand) && !bHidden && !CommandToExecute->bDebugCommand)
		WriteCommandToAttachedConsole(CommandToExecute, 2, Request);

	// Free session
	CleanupSessionCommand(CommandToExecute);

	// Lets write the end command to the system log if log level is at least 3
	if(!bHidden && (ActualSystemLogLevel >= 3) && (!IsLogWriteCommand) && !CommandToExecute->bDebugCommand)
	{
		string Message = "End command ";
		if(Application != NULL)
		{
			Message += Application->GetName();
			Message += ":";
		}
		Message += CommandToExecute->m_sService;
		Message += ":";
		Message += CommandToExecute->m_sClass;
		Message += ":";
		Message += CommandToExecute->m_sCommand;
		GetMain()->Logs.Write("System", Message.c_str(), NULL, NV_LOG_INFO, 3, m_sIdent.c_str(), "system");
	}

	return RetVal;
}

void NvSession::InitSessionCommand(NvCommand *CommandToExecute)
{
	// Only one command at a time is allowed on a session
	// Since this function is a recursive function
	// We must lock the session only the first time
	// In any case the locking occurs only for external commands
	// There is also a locking at named session level
	if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT) || (CommandToExecute->m_iCommandSource == NV_SOURCE_SERVER))
	{
		CommandToExecute->m_bHasLockedSession = false;
		if(CommandToExecute->bLockSession)
		{
			// If we have a parent session, the lock has already be done because we can have a parent
			// only if the command comes from procedure, service or server
			if(CommandToExecute->m_pParentSession == NULL)
			{
				// The owner is not this thread so we must lock
				NV_LOCKOBJECT(this);	
				// Here we got the lock
				ProtectInUse.Lock();
				m_iThreadHandleOwner = NvGetCurrentThreadId();
				ProtectInUse.Unlock();
				CommandToExecute->m_bHasLockedSession = true;
			}
			else
			{
				// The command comes from another session
				// In this case, we must lock only if the
				// actual owner of the session is from a different thread
				ProtectInUse.Lock();
				long ThreadHandle = m_iThreadHandleOwner;
				ProtectInUse.Unlock();
				if(ThreadHandle != NvGetCurrentThreadId())
				{
					// The owner is not this thread so we must lock
					NV_LOCKOBJECT(this);	
					// Here we got the lock
					ProtectInUse.Lock();
					m_iThreadHandleOwner = NvGetCurrentThreadId();
					ProtectInUse.Unlock();
					CommandToExecute->m_bHasLockedSession = true;
				}
			}
		}
		// Clear the command stack
		ClearCommandStack();

		OriginalCommand = CommandToExecute;
		string sOriginal = OriginalCommand->m_sService.c_str();
		sOriginal += ":";
		sOriginal += OriginalCommand->m_sClass.c_str();
		sOriginal += ":";
		sOriginal += OriginalCommand->m_sCommand.c_str();
		SetOriginalCommand(sOriginal.c_str());
		// Post parse the command parameters
		CommandToExecute->PostParseParameters(&m_oVariables, m_sVariableIdent.c_str());
	}
}

void NvSession::CleanupSessionCommand(NvCommand *CommandToExecute, bool PopStack)
{
	// Remove from the command stack
	if(PopStack)
		PopCommandStack();
	if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) || (CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT) || (CommandToExecute->m_iCommandSource == NV_SOURCE_SERVER))
	{
		OriginalCommand = NULL;
		SetOriginalCommand("");
		if(CommandToExecute->m_bHasLockedSession)
		{
			// We unlock only if we are the command that locked the session
			ProtectInUse.Lock();
			m_iThreadHandleOwner = 0;
			ProtectInUse.Unlock();
			CommandToExecute->m_bHasLockedSession = false;
			NV_UNLOCKOBJECT(this);
		}
	}
}

// Called when a command produced an error
void NvSession::OnCommandError(NvCommand *CommandToExecute, string OldErrorProcName)
{
	// Write log error information
	SetErrorLog(CommandToExecute);

	if(CommandToExecute->Error.m_iErrorCode && (!m_bInErrorProc))
	{
		// Get the no Error flag
		string sNoError = CommandToExecute->Parameters.GetValue("NV_NO_ERROR");
		CleanString(sNoError);
		MakeUpper(sNoError);

		// Diplay error information
		// We write the command to the console
		bool ConsoleDisplayIt = false;
		if((GetMain()->b_ConsoleMode) && !bHidden && !CommandToExecute->bDebugCommand)
		{
			ConsoleDisplayIt = true;
			if(CommandToExecute->Error.m_iErrorCode)
			{
				if(sNoError == "YES")
					ConsoleDisplayIt = false;
			}
		}
		if(CommandToExecute->bDebugCommand)
			ConsoleDisplayIt = false;
		if(ConsoleDisplayIt)
		{
			// We must get the error desc outside the console log otherwise
			// we can have a deadlock with service locking
			string ErrorDesc = CommandToExecute->Error.GetErrorDesc().c_str();
			string AppNameHere = "";
			if(Application != NULL)
				AppNameHere = Application->GetName();
			NV_LOCKLOCK(&ProtectConsoleLog);
			printf("*** Error %s:%s:%s:%s:%s - %s:%s:%d:%s (%s)", 
					m_sIdent.c_str(),
					AppNameHere.c_str(),
					CommandToExecute->m_sService.c_str(),
					CommandToExecute->m_sClass.c_str(),
					CommandToExecute->m_sCommand.c_str(),
					CommandToExecute->Error.m_sErrorService.c_str(),
					CommandToExecute->Error.m_sErrorClass.c_str(),
					CommandToExecute->Error.m_iErrorCode,
					ErrorDesc.c_str(),
					CommandToExecute->Error.m_sErrorInfo.c_str());
			printf("\n");
			NV_UNLOCKLOCK(&ProtectConsoleLog);
		}
		if(HasSessionAttachedConsole() && !bHidden && !CommandToExecute->bDebugCommand)
		{
			string ErrorDesc = CommandToExecute->Error.GetErrorDesc().c_str();
			string Message = "*** Error ";
			Message += CommandToExecute->Error.m_sErrorService;
			Message += ":";
			Message += CommandToExecute->Error.m_sErrorClass;
			Message += ":";
			char Buf[20];
			sprintf(Buf, "%d", CommandToExecute->Error.m_iErrorCode);
			Message += Buf;
			Message += ":";
			Message += ErrorDesc;
			Message += " (";
			Message += CommandToExecute->Error.m_sErrorInfo;
			Message += ")\n";
			WriteAttachedConsole(Message.c_str());
		}

		// Execute the ErrorProcedure if there is one and if the No errror flag is not set
		if(sNoError != "YES")
		{
			string ErrorProcNameHere = m_sErrorProcName;
			if(m_sDefErrorProcName.length() && (!ErrorProcNameHere.length()))
				ErrorProcNameHere = m_sDefErrorProcName;

			if(ErrorProcNameHere.length())
			{
				// Save error information
				string ErrorService = CommandToExecute->Error.m_sErrorService;
				string ErrorClass = CommandToExecute->Error.m_sErrorClass;
				int ErrorCode = CommandToExecute->Error.m_iErrorCode;
				string ErrorInfo = CommandToExecute->Error.m_sErrorInfo;
					
				m_bInErrorProc = true;
				m_sErrorProcedure = m_sProcName;
				ExecuteProcedure(CommandToExecute, ErrorProcNameHere, false); 
				m_bInErrorProc = false;

				// restore error information
				CommandToExecute->Error.SetError(ErrorService, ErrorClass, ErrorCode, ErrorInfo);
			}
		}
	}
	m_sErrorProcName = OldErrorProcName;
}


// Check command security filters
bool NvSession::CheckAccessCommand(NvCommand *CommandToExecute)
{
	// FIlter only client and web commands
	if((CommandToExecute->m_iCommandSource != NV_SOURCE_BROWSER) && (CommandToExecute->m_iCommandSource != NV_SOURCE_CLIENT))
		return true;

	bool Retval = true;
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER)
	{
		if(Application->GetCommandWebFilterMode() == NV_COMMAND_FILTER_NEVER)
			return true; // Do not use filter (allow all)
		bool DoFilter = false;
		if(Application->GetCommandWebFilterMode() == NV_COMMAND_FILTER_ALWAYS)
			DoFilter = true; // Always use filter
		if(Application->GetCommandWebFilterMode() == NV_COMMAND_FILTER_BYSECU)
		{
			// Use filter except if user has authorisation to bypasss
			if(!CheckPermission(CommandToExecute, "SYSTEM", "BYPASS_COMMAND_WEB_FILTER"))
				DoFilter = true;
		}

		// If local or if user is administrator, do not filter
		if(m_bAdministrator)
			DoFilter = false;
		if((CommandToExecute->m_sClientAddress == "127.0.0.1") || (CommandToExecute->m_sClientAddress == "::1"))
			DoFilter = false;

		if(!DoFilter)
			return true;

		Retval = Application->CommandWebFilter.Allow(CommandToExecute);
	}
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT)
	{
		if(Application->GetCommandClientFilterMode() == NV_COMMAND_FILTER_NEVER)
			return true; // Do not use filter (allow all)
		bool DoFilter = false;
		if(Application->GetCommandClientFilterMode() == NV_COMMAND_FILTER_ALWAYS)
			DoFilter = true; // Always use filter (except local and administrator)
		if(Application->GetCommandClientFilterMode() == NV_COMMAND_FILTER_BYSECU)
		{
			// Use filter except if user has authorisation to bypasss
			if(!CheckPermission(CommandToExecute, "SYSTEM", "BYPASS_COMMAND_CLIENT_FILTER"))
				DoFilter = true;
		}

		// If local or if user is administrator, do not filter
		if(m_bAdministrator)
			DoFilter = false;
		if((CommandToExecute->m_sClientAddress == "127.0.0.1") || (CommandToExecute->m_sClientAddress == "::1"))
			DoFilter = false;

		if(!DoFilter)
			return true;

		// Apply filter
		Retval = Application->CommandClientFilter.Allow(CommandToExecute);
	}
	return Retval;
}


void NvSession::PushCommandStack(NvCommand *CommandToExecute, NvRequest *Request)
{
	// Available only in debug mode
	if(!bDebug)
		return;
	string Message = "Command: ";
	Message += CommandToExecute->m_sService;
	Message += ":";
	Message += CommandToExecute->m_sClass;
	Message += ":";
	Message += CommandToExecute->m_sCommand;
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT)
		Message += "(C)";
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER)
		Message += "(B)";
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_PROCEDURE)
		Message += "(P)";
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_SERVICE)
		Message += "(S)";
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_SERVER)
		Message += "(V)";
	if(Request != NULL)
	{
		Message += " -> ";
		Message += Request->m_sServerAddress;
	}
	PushCommandStack(Message);
}
void NvSession::PushCommandStack(std::string Message)
{
	// Available only in debug mode
	if(!bDebug)
		return;
	NV_LOCKLOCK(&ProtectInUse);
	m_vStack.push_back(Message);
	NV_UNLOCKLOCK(&ProtectInUse);
}
void NvSession::PopCommandStack()
{
	// Available only in debug mode
	if(!bDebug)
		return;
	NV_LOCKLOCK(&ProtectInUse);
	if(m_vStack.size())
		m_vStack.pop_back();
	NV_UNLOCKLOCK(&ProtectInUse);
}
void NvSession::ClearCommandStack()
{
	// Available only in debug mode
	if(!bDebug)
		return;
	NV_LOCKLOCK(&ProtectInUse);
	m_vStack.clear();
	NV_UNLOCKLOCK(&ProtectInUse);
}
void NvSession::GetCommandStack(std::vector<std::string> &Stack)
{
	NV_LOCKLOCK(&ProtectInUse);
	Stack = m_vStack;
	NV_UNLOCKLOCK(&ProtectInUse);
}


void NvSession::SetErrorLog(NvCommand *CommandToExecute)
{
	// Lets write the eventual error in system log
	if(ActualSystemLogLevel && !bHidden && !CommandToExecute->bDebugCommand)
	{
		if(CommandToExecute->Error.m_iErrorCode)
		{
			string sNoError = CommandToExecute->Parameters.GetValue("NV_NO_ERROR");
			CleanString(sNoError);
			MakeUpper(sNoError);
			if(sNoError != "YES")
			{
				string Message = "*** Error in command ";
				if(Application != NULL)
				{
					Message += Application->GetName();
					Message += ":";
				}
				Message += CommandToExecute->m_sService;
				Message += ":";
				Message += CommandToExecute->m_sClass;
				Message += ":";
				Message += CommandToExecute->m_sCommand;
				string Extra = CommandToExecute->Error.m_sErrorService.c_str();
				Extra += " - ";
				Extra += CommandToExecute->Error.m_sErrorClass.c_str();
				char sBuf[20];
				sprintf(sBuf, "%d", CommandToExecute->Error.m_iErrorCode);
				Extra += " - ";
				Extra += sBuf;
				Extra += " - ";
				Extra += CommandToExecute->Error.GetErrorDesc(CommandToExecute->m_sLanguage.c_str()).c_str();
				Extra += " - ";
				Extra += CommandToExecute->Error.m_sErrorInfo.c_str();
				GetMain()->Logs.Write("System", Message.c_str(), Extra.c_str(), NV_LOG_ERROR, 1, m_sIdent.c_str(), "system");
			}
		}
	}
}
	

// Execute the procedures given in string
bool NvSession::ExecuteProcedure(NvCommand *CommandToExecute, string Procedures, bool ErrorIfNoExist, string ProcDir, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!Procedures.length())
		return true;
	
	bool RetVal = true;
	// Save original containers
	NvContainer *LocalInContainer = InContainer;
	NvContainer *LocalOutContainer = OutContainer;

	CleanString(Procedures);
	NvCommand *NativeProcedureCommand = NVNEW NvCommand;	// Used only for native procedures
	NvCommand ProcedureCommand;								// Used to set procedure specific parameters
	ProcedureCommand.m_pSession = CommandToExecute->m_pSession;
	ProcedureCommand.m_iCommandSource = CommandToExecute->m_iCommandSource;
	ProcedureCommand.m_sService = CommandToExecute->m_sService;
	ProcedureCommand.m_sClass = CommandToExecute->m_sClass;
	ProcedureCommand.m_sCommand = CommandToExecute->m_sCommand;
	ProcedureCommand.m_sInContainer = CommandToExecute->m_sInContainer;
	ProcedureCommand.m_sOutContainer = CommandToExecute->m_sOutContainer;
	ProcedureCommand.m_sLanguage = CommandToExecute->m_sLanguage;
	ProcedureCommand.m_sHost = CommandToExecute->m_sHost;
	ProcedureCommand.m_sSourceService = CommandToExecute->m_sSourceService;
	ProcedureCommand.m_sMachineName = CommandToExecute->m_sMachineName;
	ProcedureCommand.m_sMachineUser = CommandToExecute->m_sMachineUser;
	ProcedureCommand.bLockSession = CommandToExecute->bLockSession;
	ProcedureCommand.Error.SetError(CommandToExecute->Error.m_sErrorService, CommandToExecute->Error.m_sErrorClass, CommandToExecute->Error.m_iErrorCode, CommandToExecute->Error.m_sErrorInfo);
	ProcedureCommand.m_pParentSession = CommandToExecute->m_pParentSession;
	m_sProcName = "";
	NvObjIndStringList SpecificParameters;

	string DotNetDomainType = DOMAIN_TYPE_APPLICATION;
	string DotNetDomainName = "NVDEF";
	string JavaEnvType = JAVAENV_TYPE_APPLICATION;
	string JavaEnvName = "NVDEF";
	string DotNetClass = "";
	if(Application != NULL)
	{
		DotNetDomainName = Application->GetName();
		JavaEnvName = Application->GetName();
	}

	while(RetVal && (Procedures.length()))
	{
		SpecificParameters.RemoveAll();

		// Several procedures can be chained
		// They are separated by a semicolumn
		string Procedure = Procedures;
		string::size_type index = Procedures.find(';');
		if(index != string::npos)
		{
			Procedure = Procedures.substr(0, index);
			Procedures = Procedures.substr(index+1);
		}
		else
			Procedures = "";

		// A procedure name has been found
		CleanString(Procedure);
		
		// It should not be empty
		if(!Procedure.length())
			continue;

		m_sProcName = Procedure;

		// By default, we have an application procedure
		string ProcedureFile = "";


		ProcedureFile = Application->GetProcDir();
		if(ProcDir.length())
		{
			// A specific procedure directory is requested (from web service)
			ProcedureFile = ProcDir;
			AddDirSuffix(ProcedureFile);
		}
	
		// Check if we have procedure parameters. Procedure parameters are enclosed in braquets ([]) and are pairs paraname='paravalue' separated by space 
		index = Procedure.find('[');
		if(index != string::npos)
		{
			string::size_type index2 = Procedure.rfind(']');
			if(index2 == string::npos)
			{
				CommandToExecute->Error.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_INVALID_NAME, Procedure);
				RetVal = false;
				break;
			}
			if(index2 < index)
			{
				CommandToExecute->Error.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_INVALID_NAME, Procedure);
				RetVal = false;
				break;
			}
			// We have parameters		

			string Before = Procedure.substr(0, index);
			string After = Procedure.substr(index2+1);
			string ProcParameters = Procedure.substr(index+1, index2-index -1);
			Procedure = Before;
			Procedure += After;
			
			NvCommand SpecialCommand;	// used only to parse parameters
			SpecialCommand.ParseParameters(ProcParameters.c_str(), &m_oVariables, m_sVariableIdent.c_str());
			if(!CheckCommandInjection(CommandToExecute, &(SpecialCommand.Parameters)))
			{
				// We have detected a command inside a value
				CommandToExecute->Error.SetError("SYSTEM", "SECURITY", NV_SECURITY_COMMAND_IN_VALUE, Procedure);
				RetVal = false;
				break;
			}
			SpecificParameters.CopyFrom(&(SpecialCommand.Parameters));
		}

		if(m_bInErrorProc && CommandToExecute->Error.m_iErrorCode)
		{
			// If this is an error procedure
			// We add the error information as parameter
			SpecificParameters.SetValue("NV_ERROR_SERVICE", CommandToExecute->Error.m_sErrorService);
			SpecificParameters.SetValue("NV_ERROR_CLASS", CommandToExecute->Error.m_sErrorClass);
			char Buf[20];
			sprintf(Buf, "%d", CommandToExecute->Error.m_iErrorCode);
			SpecificParameters.SetValue("NV_ERROR_CODE", Buf);
			SpecificParameters.SetValue("NV_ERROR_INFO", CommandToExecute->Error.m_sErrorInfo);
			// We add also the error description
			SpecificParameters.SetValue("NV_ERROR_DESC", CommandToExecute->Error.GetErrorDesc(CommandToExecute->m_sLanguage.c_str()));
			// We add also the current procedure name
			SpecificParameters.SetValue("NV_CURRENT_PROCEDURE", m_sErrorProcedure);
		}
			
		index = Procedure.find('(');
		if((index != string::npos) && (!ProcDir.length()))
		{
			if(Procedure[Procedure.length() - 1] != ')')
			{
				CommandToExecute->Error.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_INVALID_NAME, Procedure);
				RetVal = false;
				break;
			}

			string ServiceName = "";
			// this is a system or service procedure
			if(index != 0)
			{
				// it's probably a service procedure
				// we must get the service name
				ServiceName = Procedure.substr(0, index);
				CleanString(ServiceName);
				MakeUpper(ServiceName);
				Procedure = Procedure.substr(index);
			}

			// We must remove the parentheses
			Procedure = Procedure.substr(1, Procedure.length() - 2);
			
			// And get the procedure file from system or service procedure directory
			// And get the xsl file from system or service Files directory
			if(ServiceName.length())
			{
				// We must check if this is a web service procedure
				index = ServiceName.find('{');
				if(index != string::npos)
				{
					if(ServiceName[ServiceName.length() - 1] != '}')
					{
						CommandToExecute->Error.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_INVALID_NAME, Procedure);
						RetVal = false;
						break;
					}
					// This is a web service procedure
					ServiceName = ServiceName.substr(1);
					ServiceName = ServiceName.substr(0, ServiceName.length() - 1);
					CleanString(ServiceName);
					MakeUpper(ServiceName);
					if(!ServiceName.length())
					{
						CommandToExecute->Error.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_INVALID_NAME, Procedure);
						RetVal = false;
						break;
					}

					if(!GetMain()->Webservices.WebserviceExist(ServiceName.c_str()))
					{
						CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_UNKNOWN, ServiceName);
						RetVal = false;
						break;
					}


					if(!GetMain()->Webservices.IsWebserviceEnable(ServiceName.c_str()))
					{
						CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_NOT_AVAILABLE, ServiceName);
						RetVal = false;
						break;
					}

					NvWebs* WebService = GetMain()->Webservices.GetWebservice(ServiceName.c_str());
					if(WebService == NULL)
					{
						CommandToExecute->Error.SetError("SYSTEM", "WEBSERVICE", NV_WEBSERVICE_CANNOT_GET, ServiceName);
						RetVal = false;
						break;
					}
					
					ProcedureFile = WebService->GetProcDir();

					// Free the web service
					GetMain()->Webservices.FreeWebservice(ServiceName.c_str());
					
					AddDirSuffix(ProcedureFile);
					DotNetDomainType = DOMAIN_TYPE_WEBSERVICE;
					DotNetDomainName = ServiceName;
					JavaEnvType = JAVAENV_TYPE_WEBSERVICE;
					JavaEnvName = ServiceName;
				}
				else
				{
					// This is a service procedure
					NvService *Service = GetMain()->Services.GetService(ServiceName.c_str(), true, true);
					if(Service == NULL)
					{
						// Maybe the service is not mounted (service install command)
						string SrvDirName = GetMain()->GetNirvaDir();
						SrvDirName += "Services";
						AddDirSuffix(SrvDirName);
						SrvDirName += ServiceName;
						AddDirSuffix(SrvDirName);
						SrvDirName += "Procs";
						if(!DirExist(SrvDirName.c_str()))
						{
							CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, ServiceName);
							RetVal = false;
							break;
						}
						ProcedureFile = SrvDirName;
					}
					else
					{
						ProcedureFile = Service->GetProcDir();
						GetMain()->Services.FreeService(ServiceName.c_str());
					}
					AddDirSuffix(ProcedureFile);
					DotNetDomainType = DOMAIN_TYPE_SERVICE;
					DotNetDomainName = ServiceName;
					JavaEnvType = JAVAENV_TYPE_SERVICE;
					JavaEnvName = ServiceName;
				}
			}
			else
			{
				// This is a system procedure
				ProcedureFile = GetMain()->GetNirvaDir();
				AddDirSuffix(ProcedureFile);
				ProcedureFile += "Procs";
				AddDirSuffix(ProcedureFile);
				DotNetDomainType = DOMAIN_TYPE_SYSTEM;
				DotNetDomainName = "SYSTEM";
				JavaEnvType = JAVAENV_TYPE_SYSTEM;
				JavaEnvName = "SYSTEM";
			}
		}

		bool IsPerlProc = false;
		bool IsJavaProc = false;
		bool IsDotnetProc = false;
		index = Procedure.find(":");
		if(index != string::npos)
		{
			string LanguageIdent = Procedure.substr(0, index);
			CleanString(LanguageIdent);
			MakeUpper(LanguageIdent);
			if(LanguageIdent == "PERL")
			{
				Procedure = Procedure.substr(index+1);
				CleanString(Procedure);
				IsPerlProc = true;
			}
			if(LanguageIdent == "JAVA")
			{
				Procedure = Procedure.substr(index+1);
				CleanString(Procedure);
				IsJavaProc = true;
			}
			if(LanguageIdent == "DOTNET")
			{
				Procedure = Procedure.substr(index+1);
				CleanString(Procedure);
				index = Procedure.rfind(":");
				if(index != string::npos)
				{
					DotNetClass = Procedure.substr(index+1);
					Procedure = Procedure.substr(0, index);
				}
				IsDotnetProc = true;
			}
		}


		// if no extension, we give the .nvp or .pl or .dll extension but not for a java procedure
		index = Procedure.rfind(".");
		if(!IsJavaProc)
		{
			if(index == string::npos)
			{
				if(IsPerlProc)
					Procedure += ".pl";
				else if(IsDotnetProc)
					Procedure += ".dll";
				else
					Procedure += ".nvp";
			}
		}


		ProcedureFile += Procedure;
		if(ProcedureFile.find("..") != string::npos)
		{
			CommandToExecute->Error.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_CANNOT_GET_FILE, ProcedureFile);
			RetVal = false;
			break;
		}

		if(!IsJavaProc)
		{
			if(!GetNoCaseRealFileName(ProcedureFile))
			{
				if(ErrorIfNoExist)
				{
					CommandToExecute->Error.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_CANNOT_GET_FILE, ProcedureFile);
					RetVal = false;
				}
				break;
			}
		}

		
		NativeProcedureCommand->Clear();
		ProcedureCommand.Parameters.CopyFrom(&(CommandToExecute->Parameters));
		ProcedureCommand.Parameters.Add(&SpecificParameters);	// We add the specific procedure parameters
	
		// Reset the last output buffer
		ProcedureCommand.m_sLastOutputBuffer = "";
		ProcedureCommand.m_sLastOutputBufferEncoded = "";
		NativeProcedureCommand->m_sLastOutputBuffer = "";
		NativeProcedureCommand->m_sLastOutputBufferEncoded = "";
		CommandToExecute->m_sLastOutputBuffer = "";
		CommandToExecute->m_sLastOutputBufferEncoded = "";

#ifdef WIN32
		clock_t m_cStartProcessTime = 0;
		if(GetMain()->b_ConsoleMode && !bHidden && !CommandToExecute->bDebugCommand)
			m_cStartProcessTime = clock();
#else
		unsigned int m_cStartProcessTime = 0;
		if(GetMain()->b_ConsoleMode && !bHidden && !CommandToExecute->bDebugCommand)
			m_cStartProcessTime = GetMilliSeconds();
#endif
		if(IsPerlProc)
		{
			if(GetMain()->b_ConsoleMode && !bHidden && !CommandToExecute->bDebugCommand)
			{
				NV_LOCKLOCK(&ProtectConsoleLog);
				time_t tTime = time(NULL);
				struct tm tCurrentTime;
				localtime_r(&tTime, &tCurrentTime);
				printf("%02d:%02d:%02d",
								tCurrentTime.tm_hour,
								tCurrentTime.tm_min,
								tCurrentTime.tm_sec); 
				printf(" ");
				printf("%s", m_sIdent.c_str());
				printf(" ");
				printf("%10s",Application->GetName().c_str());
				printf(" --- Start procedure: ");
				printf("%s", m_sProcName.c_str());
				printf("\n");
				NV_UNLOCKLOCK(&ProtectConsoleLog);
			}
			if(ActualSystemLogLevel && !bHidden && !CommandToExecute->bDebugCommand)
			{
				string Message = " --- Start procedure: ";
				if(Application != NULL)
				{
					Message += Application->GetName();
					Message += ":";
				}
				Message += m_sProcName;
				GetMain()->Logs.Write("System", Message.c_str(), "", NV_LOG_INFO, 1, m_sIdent.c_str(), "system");
			}
			if(HasSessionAttachedConsole() && !bHidden && !CommandToExecute->bDebugCommand)
			{
				char Buff[50];
				time_t tTime = time(NULL);
				struct tm tCurrentTime;
				localtime_r(&tTime, &tCurrentTime);
				sprintf(Buff, "%02d:%02d:%02d",
								tCurrentTime.tm_hour,
								tCurrentTime.tm_min,
								tCurrentTime.tm_sec);
				string Message = Buff;
				Message += " --- Start procedure: ";
				Message += m_sProcName.c_str();
				Message += "\n";
				WriteAttachedConsole(Message.c_str());
			}
			string StackMessage = "Procedure:";
			StackMessage += m_sProcName;
			PushCommandStack(StackMessage);
			string OldProcName = m_sProcName;

//			SystemDebugState(CommandToExecute);
//#ifdef _DEBUG
//int AllocatedMemory = GetProcessAllocatedMemory();
//if(GetMain()->b_ConsoleMode && !bHidden && !bDebugCommand)
//{
//	NV_LOCKLOCK(&ProtectConsoleLog);
//	printf("memory before %s %d", m_sProcName.c_str(), AllocatedMemory);
//	printf("\n");
//	NV_UNLOCKLOCK(&ProtectConsoleLog);
//}
//#endif
			if(!PerlRunScript(ProcedureFile.c_str(), &ProcedureCommand))
			{
				string Info = ProcedureFile;
				Info += " - Error ";
				Info += ProcedureCommand.Error.m_sErrorService;
				Info += ":";
				Info += ProcedureCommand.Error.m_sErrorClass;
				Info += ":";
				char Buf[50];
				sprintf(Buf, "%d", ProcedureCommand.Error.m_iErrorCode);
				Info += Buf;
				Info += " (";
				Info += ProcedureCommand.Error.GetErrorDesc(ProcedureCommand.m_sLanguage.c_str());
				Info += ")";
				if(ProcedureCommand.Error.m_sErrorInfo.length())
				{
					Info += " : ";
					Info += ProcedureCommand.Error.m_sErrorInfo;
				}
				if(ProcedureCommand.Error.m_iErrorCode)
					CommandToExecute->Error.SetError(ProcedureCommand.Error.m_sErrorService, ProcedureCommand.Error.m_sErrorClass, ProcedureCommand.Error.m_iErrorCode, ProcedureCommand.Error.m_sErrorInfo);
				else	// Generic error
					CommandToExecute->Error.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_PERL_ERROR, ProcedureFile);
				RetVal = false;
			}
			else
			{
				// Get the last output buffer
				CommandToExecute->m_sLastOutputBuffer = ProcedureCommand.m_sLastOutputBuffer;
				CommandToExecute->m_sLastOutputBufferEncoded = ProcedureCommand.m_sLastOutputBufferEncoded;
				GetLastOutputBuffer(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
				// Reset the last output buffer
				CommandToExecute->m_sLastOutputBuffer = "";
				CommandToExecute->m_sLastOutputBufferEncoded = "";
			}
			m_sProcName = OldProcName;
		}
		else if(IsJavaProc)
		{
			MakeUpper(JavaEnvType);
			if(GetMain()->b_ConsoleMode && !bHidden && !CommandToExecute->bDebugCommand)
			{
				NV_LOCKLOCK(&ProtectConsoleLog);
				time_t tTime = time(NULL);
				struct tm tCurrentTime;
				localtime_r(&tTime, &tCurrentTime);
				printf("%02d:%02d:%02d",
								tCurrentTime.tm_hour,
								tCurrentTime.tm_min,
								tCurrentTime.tm_sec); 
				printf(" ");
				printf("%s", m_sIdent.c_str());
				printf(" ");
				printf("%10s",Application->GetName().c_str());
				printf(" --- Start procedure: ");
				printf("%s", m_sProcName.c_str());
				printf("\n");
				NV_UNLOCKLOCK(&ProtectConsoleLog);
			}
			if(ActualSystemLogLevel && !bHidden && !CommandToExecute->bDebugCommand)
			{
				string Message = " --- Start procedure: ";
				if(Application != NULL)
				{
					Message += Application->GetName();
					Message += ":";
				}
				Message += m_sProcName;
				GetMain()->Logs.Write("System", Message.c_str(), "", NV_LOG_INFO, 1, m_sIdent.c_str(), "system");
			}
			if(HasSessionAttachedConsole() && !bHidden && !CommandToExecute->bDebugCommand)
			{
				char Buff[50];
				time_t tTime = time(NULL);
				struct tm tCurrentTime;
				localtime_r(&tTime, &tCurrentTime);
				sprintf(Buff, "%02d:%02d:%02d",
								tCurrentTime.tm_hour,
								tCurrentTime.tm_min,
								tCurrentTime.tm_sec);
				string Message = Buff;
				Message += " --- Start procedure: ";
				Message += m_sProcName.c_str();
				Message += "\n";
				WriteAttachedConsole(Message.c_str());
			}
			string StackMessage = "Procedure:";
			StackMessage += m_sProcName;
			PushCommandStack(StackMessage);
			string OldProcName = m_sProcName;
//			SystemDebugState(CommandToExecute);
			string RealMethod = JavaEnvType;
			RealMethod += "::";
			RealMethod += JavaEnvName;
			RealMethod += "::";
			RealMethod += ProcedureFile;
			if(!JavaRunMethod(RealMethod.c_str(), &ProcedureCommand))
			{
				string Info = ProcedureFile;
				Info += " - Error ";
				Info += ProcedureCommand.Error.m_sErrorService;
				Info += ":";
				Info += ProcedureCommand.Error.m_sErrorClass;
				Info += ":";
				char Buf[50];
				sprintf(Buf, "%d", ProcedureCommand.Error.m_iErrorCode);
				Info += Buf;
				Info += " (";
				Info += ProcedureCommand.Error.GetErrorDesc(ProcedureCommand.m_sLanguage.c_str());
				Info += ")";
				if(ProcedureCommand.Error.m_sErrorInfo.length())
				{
					Info += " : ";
					Info += ProcedureCommand.Error.m_sErrorInfo;
				}
				if(ProcedureCommand.Error.m_iErrorCode)
					CommandToExecute->Error.SetError(ProcedureCommand.Error.m_sErrorService, ProcedureCommand.Error.m_sErrorClass, ProcedureCommand.Error.m_iErrorCode, ProcedureCommand.Error.m_sErrorInfo);
				else
					CommandToExecute->Error.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_JAVA_ERROR, ProcedureFile);
//				OriginalCommand->Error.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_JAVA_ERROR, Info);
				RetVal = false;
			}
			else
			{
				// Get the last output buffer
				CommandToExecute->m_sLastOutputBuffer = ProcedureCommand.m_sLastOutputBuffer;
				CommandToExecute->m_sLastOutputBufferEncoded = ProcedureCommand.m_sLastOutputBufferEncoded;
				GetLastOutputBuffer(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
				// Reset the last output buffer (PM do not reset because otherwise the GET_LAST_OUTPUT_BUFFER is n ot working)
//				CommandToExecute->m_sLastOutputBuffer = "";
//				CommandToExecute->m_sLastOutputBufferEncoded = "";
			}
//			SystemDebugDump(CommandToExecute);
			m_sProcName = OldProcName;
		}
		else if(IsDotnetProc)
		{
			MakeUpper(DotNetDomainName);
			if(GetMain()->b_ConsoleMode && !bHidden && !CommandToExecute->bDebugCommand)
			{
				NV_LOCKLOCK(&ProtectConsoleLog);
				time_t tTime = time(NULL);
				struct tm tCurrentTime;
				localtime_r(&tTime, &tCurrentTime);
				printf("%02d:%02d:%02d",
								tCurrentTime.tm_hour,
								tCurrentTime.tm_min,
								tCurrentTime.tm_sec); 
				printf(" ");
				printf("%s", m_sIdent.c_str());
				printf(" ");
				printf("%10s",Application->GetName().c_str());
				printf(" --- Start procedure: ");
				printf("%s", m_sProcName.c_str());
				printf("\n");
				NV_UNLOCKLOCK(&ProtectConsoleLog);
			}
			if(ActualSystemLogLevel && !bHidden && !CommandToExecute->bDebugCommand)
			{
				string Message = " --- Start procedure: ";
				if(Application != NULL)
				{
					Message += Application->GetName();
					Message += ":";
				}
				Message += m_sProcName;
				GetMain()->Logs.Write("System", Message.c_str(), "", NV_LOG_INFO, 1, m_sIdent.c_str(), "system");
			}
			if(HasSessionAttachedConsole() && !bHidden && !CommandToExecute->bDebugCommand)
			{
				char Buff[50];
				time_t tTime = time(NULL);
				struct tm tCurrentTime;
				localtime_r(&tTime, &tCurrentTime);
				sprintf(Buff, "%02d:%02d:%02d",
								tCurrentTime.tm_hour,
								tCurrentTime.tm_min,
								tCurrentTime.tm_sec);
				string Message = Buff;
				Message += " --- Start procedure: ";
				Message += m_sProcName.c_str();
				Message += "\n";
				WriteAttachedConsole(Message.c_str());
			}
			string StackMessage = "Procedure:";
			StackMessage += m_sProcName;
			PushCommandStack(StackMessage);
			string OldProcName = m_sProcName;
			string RealMethod = DotNetDomainType;
			RealMethod += "::";
			RealMethod += DotNetDomainName;
			RealMethod += "::";
			RealMethod += ProcedureFile;
			if(DotNetClass.length())
			{
				RealMethod += ":";
				RealMethod += DotNetClass;
			}
			if(!DotnetRunMethod(RealMethod.c_str(), &ProcedureCommand))
			{
				string Info = ProcedureFile;
				Info += " - Error ";
				Info += ProcedureCommand.Error.m_sErrorService;
				Info += ":";
				Info += ProcedureCommand.Error.m_sErrorClass;
				Info += ":";
				char Buf[50];
				sprintf(Buf, "%d", ProcedureCommand.Error.m_iErrorCode);
				Info += Buf;
				Info += " (";
				Info += ProcedureCommand.Error.GetErrorDesc(ProcedureCommand.m_sLanguage.c_str());
				Info += ")";
				if(ProcedureCommand.Error.m_sErrorInfo.length())
				{
					Info += " : ";
					Info += ProcedureCommand.Error.m_sErrorInfo;
				}
				if(ProcedureCommand.Error.m_iErrorCode)
					CommandToExecute->Error.SetError(ProcedureCommand.Error.m_sErrorService, ProcedureCommand.Error.m_sErrorClass, ProcedureCommand.Error.m_iErrorCode, ProcedureCommand.Error.m_sErrorInfo);
				else
					CommandToExecute->Error.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_DOTNET_ERROR, ProcedureFile);
//				OriginalCommand->Error.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_DOTNET_ERROR, Info);
				RetVal = false;
			}
			else
			{
				// Get the last output buffer
				CommandToExecute->m_sLastOutputBuffer = ProcedureCommand.m_sLastOutputBuffer;
				CommandToExecute->m_sLastOutputBufferEncoded = ProcedureCommand.m_sLastOutputBufferEncoded;
				GetLastOutputBuffer(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
				// Reset the last output buffer (PM do not reset because otherwise the GET_LAST_OUTPUT_BUFFER is n ot working)
//				CommandToExecute->m_sLastOutputBuffer = "";
//				CommandToExecute->m_sLastOutputBufferEncoded = "";
			}
//			SystemDebugDump(CommandToExecute);
			m_sProcName = OldProcName;
		}
		else
		{
			if(GetMain()->b_ConsoleMode && !bHidden && !CommandToExecute->bDebugCommand)
			{
				NV_LOCKLOCK(&ProtectConsoleLog);
				time_t tTime = time(NULL);
				struct tm tCurrentTime;
				localtime_r(&tTime, &tCurrentTime);
				printf("%02d:%02d:%02d",
								tCurrentTime.tm_hour,
								tCurrentTime.tm_min,
								tCurrentTime.tm_sec); 
				printf(" ");
				printf("%s", m_sIdent.c_str());
				printf(" ");
				printf("%10s",Application->GetName().c_str());
				printf(" --- Start procedure: ");
				printf("%s", m_sProcName.c_str());
				printf("\n");
				NV_UNLOCKLOCK(&ProtectConsoleLog);
			}
			if(ActualSystemLogLevel && !bHidden && !CommandToExecute->bDebugCommand)
			{
				string Message = " --- Start procedure: ";
				if(Application != NULL)
				{
					Message += Application->GetName();
					Message += ":";
				}
				Message += m_sProcName;
				GetMain()->Logs.Write("System", Message.c_str(), "", NV_LOG_INFO, 1, m_sIdent.c_str(), "system");
			}
			if(HasSessionAttachedConsole() && !bHidden && !CommandToExecute->bDebugCommand)
			{
				char Buff[50];
				time_t tTime = time(NULL);
				struct tm tCurrentTime;
				localtime_r(&tTime, &tCurrentTime);
				sprintf(Buff, "%02d:%02d:%02d",
								tCurrentTime.tm_hour,
								tCurrentTime.tm_min,
								tCurrentTime.tm_sec);
				string Message = Buff;
				Message += " --- Start procedure: ";
				Message += m_sProcName.c_str();
				Message += "\n";
				WriteAttachedConsole(Message.c_str());
			}
			string StackMessage = "Procedure: ";
			StackMessage += m_sProcName;
			PushCommandStack(StackMessage);
			NativeProcedureCommand->m_pSession = ProcedureCommand.m_pSession;
			NativeProcedureCommand->m_iCommandSource = NV_SOURCE_PROCEDURE;

			// By default, the procedure containers are the same than the original command containers
			if(OriginalCommand == NULL)
			{
				NativeProcedureCommand->m_sHost = ProcedureCommand.m_sHost;
				NativeProcedureCommand->m_sMachineName = ProcedureCommand.m_sMachineName;
				NativeProcedureCommand->m_sMachineUser = ProcedureCommand.m_sMachineUser;
			}
			else
			{
				NativeProcedureCommand->m_sHost = OriginalCommand->m_sHost;
				NativeProcedureCommand->m_sMachineName = OriginalCommand->m_sMachineName;
				NativeProcedureCommand->m_sMachineUser = OriginalCommand->m_sMachineUser;
			}
			// By default, the procedure containers are the same than the actual command containers
			NativeProcedureCommand->m_sInContainer = ProcedureCommand.m_sInContainer;
			NativeProcedureCommand->m_sOutContainer = ProcedureCommand.m_sOutContainer;
			// The procedure inherits of the parent session if there is one
			NativeProcedureCommand->m_pParentSession = ProcedureCommand.m_pParentSession;

			// Check if we have to revert the containers
			bool bReverseContainers = false;
			string sReverseContainers = NativeProcedureCommand->Parameters.GetValue("NV_REVERSE_CONTAINER");
			CleanString(sReverseContainers);
			MakeUpper(sReverseContainers);
			if(sReverseContainers == "YES")
			{
				// Revert the containers
				NativeProcedureCommand->m_sInContainer = ProcedureCommand.m_sOutContainer;
				NativeProcedureCommand->m_sOutContainer = ProcedureCommand.m_sInContainer;
			}

			// The language is also the same
			if(OriginalCommand == NULL)
			{
				NativeProcedureCommand->m_sLanguage = ProcedureCommand.m_sLanguage;
			}
			else
			{
				NativeProcedureCommand->m_sLanguage = OriginalCommand->m_sLanguage;
				NativeProcedureCommand->HttpInHeaders.CopyFrom(&(OriginalCommand->HttpInHeaders));
			}

			NvObjFile ObjProcedureFile;
			ObjProcedureFile.SetPersistTime();
			ObjProcedureFile.SetFileName(ProcedureFile);
			if(!ObjProcedureFile.Open("r"))
			{
				CommandToExecute->Error.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_CANNOT_OPEN_FILE, ProcedureFile);
				RetVal = false;
			}
			else
			{
				string Line;
				int LineNum = 0;
				while(RetVal && (ObjProcedureFile.ReadLine(Line)))
				{
					LineNum++;
					RemoveFirstAndEndBlanks(Line);
					if(!Line.length())
						continue;
					if(Line[0] == ';')
						continue;	// Comment line

					NativeProcedureCommand->FreeData();
					NativeProcedureCommand->Error.SetError();

					// By default, the procedure containers are the same than the actual command containers
					NativeProcedureCommand->m_sInContainer = ProcedureCommand.m_sInContainer;
					NativeProcedureCommand->m_sOutContainer = ProcedureCommand.m_sOutContainer;

					// The procedure inherits of the parent session if there is one
					NativeProcedureCommand->m_pParentSession = ProcedureCommand.m_pParentSession;
		
					NativeProcedureCommand->ParseParameters(Line.c_str(), &m_oVariables, m_sVariableIdent.c_str());

					// Check if we have to revert the containers
					bool bReverseContainers = false;
					string sReverseContainers = NativeProcedureCommand->Parameters.GetValue("NV_REVERSE_CONTAINER");
					CleanString(sReverseContainers);
					MakeUpper(sReverseContainers);
					if(sReverseContainers == "YES")
					{
						// Revert the containers
						NativeProcedureCommand->m_sInContainer = ProcedureCommand.m_sOutContainer;
						NativeProcedureCommand->m_sOutContainer = ProcedureCommand.m_sInContainer;
					}


					NativeProcedureCommand->m_sService = "";
					NativeProcedureCommand->m_sClass = "";
					NativeProcedureCommand->m_sCommand = "";
					NativeProcedureCommand->m_sCommand = NativeProcedureCommand->Parameters.GetValue("NV_CMD");
					if(NativeProcedureCommand->m_sCommand.length())
					{
						string::size_type index = NativeProcedureCommand->m_sCommand.rfind(':');
						if(index != string::npos)
						{
							NativeProcedureCommand->m_sClass = NativeProcedureCommand->m_sCommand.substr(0, index);
							NativeProcedureCommand->m_sCommand = NativeProcedureCommand->m_sCommand.substr(index+1);
							index = NativeProcedureCommand->m_sClass.rfind(':');
							if(index != string::npos)
							{
								NativeProcedureCommand->m_sService = NativeProcedureCommand->m_sClass.substr(0, index);
								NativeProcedureCommand->m_sClass = NativeProcedureCommand->m_sClass.substr(index+1);
							}
						}
					}
					else
					{
						NativeProcedureCommand->m_sService = NativeProcedureCommand->Parameters.GetValue("NV_SERVICE");
						NativeProcedureCommand->m_sClass = NativeProcedureCommand->Parameters.GetValue("NV_CLASS");
						NativeProcedureCommand->m_sCommand = NativeProcedureCommand->Parameters.GetValue("NV_COMMAND");
					}
					if(!CommandToExecute->bFastMode)
					{
						CleanString(NativeProcedureCommand->m_sService, true);
						CleanString(NativeProcedureCommand->m_sClass, true);
						CleanString(NativeProcedureCommand->m_sCommand, true);
					}
					if((!NativeProcedureCommand->m_sService.length()) && (!NativeProcedureCommand->m_sClass.length()) && (!NativeProcedureCommand->m_sCommand.length()))
					{
						NativeProcedureCommand->m_sService = "SYSTEM";
						NativeProcedureCommand->m_sClass = "MISC";
						NativeProcedureCommand->m_sCommand = "NOP";
					}
				
					if(!NativeProcedureCommand->m_sService.length())
						NativeProcedureCommand->m_sService = "SYSTEM";
					if(!NativeProcedureCommand->m_sClass.length())
						NativeProcedureCommand->m_sClass = NativeProcedureCommand->m_sService;


					if(NativeProcedureCommand->Parameters.KeyExist("NV_CONTAINER"))
					{
						NativeProcedureCommand->m_sInContainer = NativeProcedureCommand->Parameters.GetValue("NV_CONTAINER");
						NativeProcedureCommand->m_sOutContainer = NativeProcedureCommand->Parameters.GetValue("NV_CONTAINER");
					}
					if(NativeProcedureCommand->Parameters.KeyExist("NV_IN_CONTAINER"))
						NativeProcedureCommand->m_sInContainer = NativeProcedureCommand->Parameters.GetValue("NV_IN_CONTAINER");
					if(NativeProcedureCommand->Parameters.KeyExist("NV_OUT_CONTAINER"))
						NativeProcedureCommand->m_sOutContainer = NativeProcedureCommand->Parameters.GetValue("NV_OUT_CONTAINER");

					if(!CommandToExecute->bFastMode)
					{
						CleanString(NativeProcedureCommand->m_sInContainer);
						CleanString(NativeProcedureCommand->m_sOutContainer);
						CleanString(NativeProcedureCommand->m_sLanguage);
					}
					MakeUpper(NativeProcedureCommand->m_sService);				// Service is always uppercase
					MakeUpper(NativeProcedureCommand->m_sClass);				// Class is always uppercase
					MakeUpper(NativeProcedureCommand->m_sCommand);				// Command is always uppercase
					MakeLower(NativeProcedureCommand->m_sInContainer);			// In container is always lowercase
					MakeLower(NativeProcedureCommand->m_sOutContainer);			// Out container is always lowercase
	//				if(!NativeProcedureCommand->m_sOutContainer.length())
	//					NativeProcedureCommand->m_sOutContainer = NativeProcedureCommand->m_sInContainer;
					MakeUpper(NativeProcedureCommand->m_sLanguage);				// language is always uppercase


					// Remove reserved parameters from command
					NativeProcedureCommand->Parameters.Remove("NV_SERVICE");
					NativeProcedureCommand->Parameters.Remove("NV_CLASS");
					NativeProcedureCommand->Parameters.Remove("NV_COMMAND");
					NativeProcedureCommand->Parameters.Remove("NV_CMD");
					NativeProcedureCommand->Parameters.Remove("NV_IN_CONTAINER");
					NativeProcedureCommand->Parameters.Remove("NV_OUT_CONTAINER");

					if(!NativeProcedureCommand->m_sService.length())
					{
						string sMessageInfo = " : NV_SERVICE in procedure file ";
						sMessageInfo += ProcedureFile;
						sMessageInfo += " - Line ";
						char Buf[50];
						sprintf(Buf, "%d", LineNum);
						sMessageInfo += Buf;
						CommandToExecute->Error.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_BAD_PARAMETER, sMessageInfo);
						RetVal = false;
						break;
					}

					if(!NativeProcedureCommand->m_sCommand.length())
					{
						string sMessageInfo = " : NV_COMMAND in procedure file ";
						sMessageInfo += ProcedureFile;
						sMessageInfo += " - Line ";
						char Buf[50];
						sprintf(Buf, "%d", LineNum);
						sMessageInfo += Buf;
						CommandToExecute->Error.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_BAD_PARAMETER, sMessageInfo);
						RetVal = false;
						break;
					}

					// This is the recursivity that executes the actual procedure command
					// Get the no error flag
					string NoError = NativeProcedureCommand->Parameters.GetValue("NV_NO_ERROR");
					MakeUpper(NoError);
					CleanString(NoError);
					bool bNoError = false;
					if(NoError == "YES")
						bNoError = true;

					string OldProcName = m_sProcName;
					bool bGetParamCommand = false;
					if(NativeProcedureCommand->m_sService == "SYSTEM")
						if(NativeProcedureCommand->m_sClass == "COMMAND")
							if(NativeProcedureCommand->m_sCommand == "GET_PARAMETER")
								bGetParamCommand = true;
					if(bGetParamCommand)
					{
						// The only command managed here (specific to native procedures)
						string sOriginal = NativeProcedureCommand->Parameters.GetValue("ORIGIN");
						MakeUpper(sOriginal);
						CleanString(sOriginal);
						bool bOriginal = false;
						if(sOriginal == "YES")
							bOriginal = true;
						if(OriginalCommand == NULL)
							bOriginal = false;
						string sName = NativeProcedureCommand->Parameters.GetValue("NAME");
						CleanString(sName);
						string sValue = "";
						if(sName.length())
						{
							if(bOriginal)
								sValue = OriginalCommand->Parameters.GetValue(sName);
							else
								sValue = ProcedureCommand.Parameters.GetValue(sName);
						}
						string VarName = NativeProcedureCommand->Parameters.GetValue("NV_VAR");
						if(!VarName.length())
							VarName = "NV_RESULT";
						m_oVariables.SetValue(VarName, sValue);
					}
					else
					{
						bool CommandRetVal = true;
						try
						{
							CommandRetVal = Command(NativeProcedureCommand);
						}
						catch(...)
						{
							GetMain()->IncNumExceptions();
							string Message = NativeProcedureCommand->m_sService;
							Message += ":";
							Message += NativeProcedureCommand->m_sClass;
							Message += ":";
							Message += NativeProcedureCommand->m_sCommand;
							NativeProcedureCommand->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_EXCEPTION, Message);
							GetMain()->Logs.Write("Exceptions", "Exception in native procedure command", Message.c_str(), NV_LOG_ERROR);
							CommandRetVal = false;
						}
						if((!CommandRetVal) && (!bNoError))
						{
							string sMessageInfo = NativeProcedureCommand->Error.m_sErrorInfo;
							sMessageInfo += " (Procedure file ";
							sMessageInfo += ProcedureFile;
							sMessageInfo += " - Line ";
							char Buf[50];
							sprintf(Buf, "%d", LineNum);
							sMessageInfo += Buf;
							sMessageInfo += ")";
							CommandToExecute->Error.SetError(NativeProcedureCommand->Error.m_sErrorService, NativeProcedureCommand->Error.m_sErrorClass, NativeProcedureCommand->Error.m_iErrorCode, sMessageInfo);
							RetVal = false;
							m_sProcName = OldProcName;
							break;
						}
					}
					m_sProcName = OldProcName;
				}
				ObjProcedureFile.Close();
				if(RetVal)
				{
					// Get the last output buffer
					CommandToExecute->m_sLastOutputBuffer = NativeProcedureCommand->m_sLastOutputBuffer;
					CommandToExecute->m_sLastOutputBufferEncoded = NativeProcedureCommand->m_sLastOutputBufferEncoded;
					GetLastOutputBuffer(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
					// Reset the last output buffer (PM do not reset because otherwise the GET_LAST_OUTPUT_BUFFER is n ot working)
//					CommandToExecute->m_sLastOutputBuffer = "";
//					CommandToExecute->m_sLastOutputBufferEncoded = "";
				}
			}
		}	// End else native proc

		PopCommandStack();
		if(GetMain()->b_ConsoleMode && !bHidden && !CommandToExecute->bDebugCommand)
		{
			NV_LOCKLOCK(&ProtectConsoleLog);
			time_t tTime = time(NULL);
			struct tm tCurrentTime;
			localtime_r(&tTime, &tCurrentTime);
			printf("%02d:%02d:%02d",
							tCurrentTime.tm_hour,
							tCurrentTime.tm_min,
							tCurrentTime.tm_sec); 
			printf(" ");
			printf("%s", m_sIdent.c_str());
			printf(" ");
			printf("%10s",Application->GetName().c_str());
			printf(" --- End ");
/*			if(IsPerlProc)
				printf("perl ");
			else if(IsJavaProc)
				printf("java ");
			else if(IsDotnetProc)
				printf("dotnet ");
			else
				printf("native ");*/
			printf("procedure: ");
			printf("%s", m_sProcName.c_str());
			if(GetMain()->b_WithTime)
			{
#ifdef WIN32
				clock_t m_cProcessTime = clock();
				m_cProcessTime = m_cProcessTime - m_cStartProcessTime;
				int RealProcessTimeInMilliseconds = (int)((m_cProcessTime * 1000)/CLOCKS_PER_SEC);
#else
				unsigned int m_cProcessTime = GetMilliSeconds();
				m_cProcessTime = m_cProcessTime - m_cStartProcessTime;
				int RealProcessTimeInMilliseconds = m_cProcessTime;
#endif
				if(RealProcessTimeInMilliseconds > 9)
					RealProcessTimeInMilliseconds -= 9; // arondi
				printf(" (%.2f s)", ((float)RealProcessTimeInMilliseconds)/1000);
			}
			printf("\n");
			NV_UNLOCKLOCK(&ProtectConsoleLog);
		}
		if(ActualSystemLogLevel && !bHidden && !CommandToExecute->bDebugCommand)
		{
			string Message = " --- End ";
/*			if(IsPerlProc)
				Message += "perl ";
			else if(IsJavaProc)
				Message += "java ";
			else if(IsDotnetProc)
				Message += "dotnet ";
			else
				Message += "native ";*/
			Message += "procedure: ";
			if(Application != NULL)
			{
				Message += Application->GetName();
				Message += ":";
			}
			Message += m_sProcName;
			GetMain()->Logs.Write("System", Message.c_str(), "", NV_LOG_INFO, 1, m_sIdent.c_str(), "system");
		}
		if(HasSessionAttachedConsole() && !bHidden && !CommandToExecute->bDebugCommand)
		{
			char Buff[50];
			time_t tTime = time(NULL);
			struct tm tCurrentTime;
			localtime_r(&tTime, &tCurrentTime);
			sprintf(Buff, "%02d:%02d:%02d",
							tCurrentTime.tm_hour,
							tCurrentTime.tm_min,
							tCurrentTime.tm_sec);
			string Message = Buff;
			Message += " --- End ";
/*			if(IsPerlProc)
				Message += "perl ";
			else if(IsJavaProc)
				Message += "java ";
			else if(IsDotnetProc)
				Message += "dotnet ";
			else
				Message += "native ";*/
			Message += "procedure: ";
			Message += m_sProcName;
			Message += "\n";
			WriteAttachedConsole(Message.c_str());
		}
	} // End while

	delete NativeProcedureCommand;
	m_sProcName = "";

	// Retrieve the original containers
	InContainer = LocalInContainer;
	OutContainer = LocalOutContainer;

	return RetVal;
}


// Get procedure file name
// return the complete path of the procedure file
// doesn't check if it exists
string NvSession::GetProcedureFileName(string Procedure, string ProcDir)
{
	string RetVal = "";
	CleanString(Procedure);
	if(!Procedure.length())
		return RetVal;
	

	// By default, we have an application procedure
	string ProcedureFile = "";
	ProcedureFile = Application->GetProcDir();
	if(ProcDir.length())
	{
		// A specific procedure directory is requested (from web service)
		ProcedureFile = ProcDir;
		AddDirSuffix(ProcedureFile);
	}
	
	// Check if we have procedure parameters. Procedure parameters are enclosed in braquets ([]) and are pairs paraname='paravalue' separated by space 
	string::size_type index = Procedure.find('[');
	if(index != string::npos)
	{
		string::size_type index2 = Procedure.rfind(']');
		if((index2 != string::npos) && (index2 > index))
		{
			// We have parameters		

			string Before = Procedure.substr(0, index);
			string After = Procedure.substr(index2+1);
			Procedure = Before;
			Procedure += After;
		}
	}
			
	index = Procedure.find('(');
	if((index != string::npos) && (!ProcDir.length()))
	{
		if(Procedure[Procedure.length() - 1] == ')')
		{
			string ServiceName = "";
			// this is a system or service procedure
			if(index != 0)
			{
				// it's probably a service procedure
				// we must get the service name
				ServiceName = Procedure.substr(0, index);
				CleanString(ServiceName);
				MakeUpper(ServiceName);
				Procedure = Procedure.substr(index);
			}

			// We must remove the parentheses
			Procedure = Procedure.substr(1, Procedure.length() - 2);
		
			// And get the procedure file from system or service procedure directory
			if(ServiceName.length())
			{
				NvService *Service = GetMain()->Services.GetService(ServiceName.c_str(), true, true);
				if(Service == NULL)
				{
					// Maybe the service is not mounted (service install command)
					string SrvDirName = GetMain()->GetNirvaDir();
					SrvDirName += "Services";
					AddDirSuffix(SrvDirName);
					SrvDirName += ServiceName;
					AddDirSuffix(SrvDirName);
					SrvDirName += "Procs";
					ProcedureFile = SrvDirName;
				}
				else
				{
					ProcedureFile = Service->GetProcDir();
					GetMain()->Services.FreeService(ServiceName.c_str());
				}
				AddDirSuffix(ProcedureFile);
			}
			else
			{
				ProcedureFile = GetMain()->GetNirvaDir();
				AddDirSuffix(ProcedureFile);
				ProcedureFile += "Procs";
				AddDirSuffix(ProcedureFile);
			}
		}
	}

	bool IsPerlProc = false;
	bool IsJavaProc = false;
	bool IsDotnetProc = false;
	index = Procedure.find(":");
	if(index != string::npos)
	{
		string LanguageIdent = Procedure.substr(0, index);
		CleanString(LanguageIdent);
		MakeUpper(LanguageIdent);
		if(LanguageIdent == "PERL")
		{
			Procedure = Procedure.substr(index+1);
			CleanString(Procedure);
			IsPerlProc = true;
		}
		if(LanguageIdent == "JAVA")
		{
			Procedure = Procedure.substr(index+1);
			CleanString(Procedure);
			IsJavaProc = true;
		}
		if(LanguageIdent == "DOTNET")
		{
			Procedure = Procedure.substr(index+1);
			CleanString(Procedure);
			IsDotnetProc = true;
		}
	}


	// if no extension, we give the .nvp or .pl or .dll extension but not for a java procedure
	index = Procedure.rfind(".");
	if(index == string::npos)
	{
		if(IsPerlProc)
			Procedure += ".pl";
		else if(IsJavaProc)
			Procedure += ".class";
		else if(IsDotnetProc)
			Procedure += ".dll";
		else
			Procedure += ".nvp";
	}


	ProcedureFile += Procedure;
	RetVal = ProcedureFile;


	return RetVal;
}



bool NvSession::SetXmlOutput(NvCommand *CommandToExecute, bool DebugOnly, NvContainer *Model, string MessageName, string WebsName)
{
	if(DebugOnly && (!bDebug))
		return true;	// nothing to do in this case

	// If Model != NULL, the call comes from the WEBSERVICE EXECUTE command

	// We block if necessary but never in case of web service connector output
	// The SetXmlOutput is then called directly from the WEBSERVICE EXECUTE command
	if(Model != NULL)
		if(CommandToExecute->iOutputType != OUTPUT_XML)
			return true;

	NvContainer *XmlContainer = OutContainer;
	string sXmlContainer = "";
	bool bIsOutputContainer = true;
	if(!CommandToExecute->bWebs)
	{
		if(CommandToExecute->Parameters.KeyExist("NV_XML_CONTAINER"))
		{
			sXmlContainer = CommandToExecute->Parameters.GetValue("NV_XML_CONTAINER");
			XmlContainer = GetContainer(sXmlContainer.c_str(), true, CommandToExecute);
			bIsOutputContainer = false;
		}
	}
	if(XmlContainer == NULL)
	{
		// The container must exist
		string sContainerName = sXmlContainer;
		sContainerName = Containers.CleanContainerName(sContainerName);
		if(!sContainerName.length())
			sContainerName = NV_CONTAINER_DEFAULT_NAME;
		CommandToExecute->Error.SetError("SYSTEM", "CONTAINER", NV_CONTAINER_CANNOT_GET, sContainerName);
		return false;
	}

	// Get object names that may be in parameters
	string ObjNames = CommandToExecute->Parameters.GetValue("NV_XML_OBJECTS");
	CleanString(ObjNames);


	// Get the NV_XML_OUTPUT option
	string OutputType = CommandToExecute->Parameters.GetValue("NV_XML_OUTPUT");
	string OutputTypeHere = OutputType; 
	CleanString(OutputTypeHere);
	MakeUpper(OutputTypeHere);
	bool bHtmlOutput = true;	// Default is html output
	if(OutputTypeHere.length())		// Except if output type is explicitly given
		bHtmlOutput = false;
	if(CommandToExecute->iXmlInputType)
		bHtmlOutput = false;		// Except if the input was xml
	if(OutputTypeHere == "HTML")
		bHtmlOutput = true;
	if(CommandToExecute->bWebs)
		bHtmlOutput = false;

	// If debug only, the ouput is always XML
	if(DebugOnly)
		bHtmlOutput = false;

	// Get subcontainer option
	bool bWithSubContainers = false;
	if(CommandToExecute->iXmlInputType)
		bWithSubContainers = true;		// default is true for XML and SOAP connector
	// If debug only, the default is to have subcontainers
	if(DebugOnly)
		bWithSubContainers = true;
	string sWithSubContainers = CommandToExecute->Parameters.GetValue("NV_XML_SUBCONTAINERS");
	CleanString(sWithSubContainers);
	MakeUpper(sWithSubContainers);
	if(sWithSubContainers == "YES")
		bWithSubContainers = true;
	if(sWithSubContainers == "NO")
		bWithSubContainers = false;
	if(CommandToExecute->bWebs)
		bWithSubContainers = true;	// For web service , always use subcontainers



	// Get session option info
	bool bSessionInfo = true;
	if(CommandToExecute->iXmlInputType)
		bSessionInfo = false;		// default is false for XML and SOAP connectors
	string sSessionInfo = CommandToExecute->Parameters.GetValue("NV_XML_SESSION_INFO");
	CleanString(sSessionInfo);
	MakeUpper(sSessionInfo);
	if(sSessionInfo == "YES")
		bSessionInfo = true;
	if(sSessionInfo == "NO")
		bSessionInfo = false;

	// Get http headers option
	bool bWithHttpHeaders = true;
	if(CommandToExecute->iXmlInputType)
		bWithHttpHeaders = false;		// default is false for XML and SOAP connectors
	string sWithHttpHeaders = CommandToExecute->Parameters.GetValue("NV_XML_HTTP_HEADERS");
	CleanString(sWithHttpHeaders);
	MakeUpper(sWithHttpHeaders);
	if(sWithHttpHeaders == "YES")
		bWithHttpHeaders = true;
	if(sWithHttpHeaders == "NO")
		bWithHttpHeaders = false;
	if(CommandToExecute->bWebs)
		bWithHttpHeaders = false;	// For web service , no http headers

	// If debug only, we never have http headers
	if(DebugOnly)
		bWithHttpHeaders = false;

	// Get variables option
	bool bWithVariables = true;
	if(CommandToExecute->iXmlInputType)
		bWithVariables = false;		// default is false for XML and SOAP connectors
	// If debug only, the default is to not have variables
	if(DebugOnly)
		bWithVariables = false;
	string sWithVariables = CommandToExecute->Parameters.GetValue("NV_XML_VARIABLES");
	CleanString(sWithVariables);
	MakeUpper(sWithVariables);
	if(sWithVariables == "YES")
		bWithVariables = true;
	if(sWithVariables == "NO")
		bWithVariables = false;

	// Get with data option
	bool bWithData = true;
	bool bWithFileData = false;
	if(CommandToExecute->bWebs)
		bWithFileData = true;
	string sWithData = CommandToExecute->Parameters.GetValue("NV_XML_WITH_DATA");
	CleanString(sWithData);
	MakeUpper(sWithData);
	if(sWithData == "NO")
		bWithData = false;
	if(sWithData == "ALL")
	{
		bWithData = true;
		bWithFileData = true;
	}
	if(CommandToExecute->bWebs)
		bWithFileData = true;
	if(CommandToExecute->bWebs)
		bWithData = true;

	// Get the encoding charset
	// By default it's the one of the input XML if there is one
	// (Comes from the Content-Type HTTP header)
	// This value may be changed by the NV_XML_ENCODING parameter
	string sEncoding = "";
	if(CommandToExecute->iXmlInputType)
		sEncoding = CommandToExecute->m_sInputCharset;
	if(CommandToExecute->Parameters.KeyExist("NV_XML_ENCODING"))
		sEncoding = CommandToExecute->Parameters.GetValue("NV_XML_ENCODING");
	CleanString(sEncoding);
	MakeUpper(sEncoding);
	if((sEncoding.length()) && (sEncoding != "ISO-8859-1") && (sEncoding != "UTF-8"))
	{
		CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_UNSUPPORTED_ENCODING, sEncoding);
		if(!bIsOutputContainer)
			FreeContainer(XmlContainer, sXmlContainer.c_str(), false, CommandToExecute);
		return false;
	}
	bool bUnicode = false;
	if(!sEncoding.length())
	{
		if(GetMain()->IsUnicode())
			sEncoding = "UTF-8";
		else
			sEncoding = "ISO-8859-1";
	}
	if(sEncoding == "UTF-8")
		bUnicode = true;
	NvXml XmlObjectForCoding;
	if(CommandToExecute->Parameters.KeyExist("NV_XML_STRICT"))
	{
		string sStrict = CommandToExecute->Parameters.GetValue("NV_XML_STRICT");
		CleanString(sStrict, true);
		MakeUpper(sStrict);
		if(sStrict == "YES")
			XmlObjectForCoding.bStrict = true;
	}

	string::size_type index;

	int i;
	NvObjStringList ObjectsToGet;
	if(Model != NULL)
	{
		map<string,void *>::iterator pos;
		for(pos = Model->m_mObjects.begin(); pos != Model->m_mObjects.end(); ++pos)
		{
			NvObj *Object = (NvObj*)(pos->second);
			ObjectsToGet.Add(Object->GetName());
		}
	}
	else
	{
		if(CommandToExecute->Parameters.KeyExist("NV_XML_OBJECTS"))
		{
			while(ObjNames.length())
			{
				// Several objects can be in the list
				// They are separated by a semicolumn
				string ObjectName = ObjNames;
				index = ObjectName.find(';');
				if(index != string::npos)
				{
					ObjectName = ObjNames.substr(0, index);
					ObjNames = ObjNames.substr(index+1);
				}
				else
					ObjNames = "";

				// An object name has been found
				CleanString(ObjectName);
				
				// It should not be empty
				if(!ObjectName.length())
					continue;

				ObjectsToGet.Add(ObjectName);
			}
		}
		else
		{
			map<string,void *>::iterator pos;
			for(pos = XmlContainer->m_mObjects.begin(); pos != XmlContainer->m_mObjects.end(); ++pos)
			{
				NvObj *Object = (NvObj*)(pos->second);
				ObjectsToGet.Add(Object->GetName());
			}
		}
	}
	

	NvInt64 SizeData = 0;
	if(bWithData)
	{
		for(i=0; i<ObjectsToGet.GetSize(); i++)
		{
			string sObjectInList = ObjectsToGet.GetValue(i);
			XmlContainer->GetXmlObjectDataSize(ObjectsToGet.GetValue(i).c_str(), SizeData, false, bWithFileData, Model);
		}

		if(bWithSubContainers)
			XmlContainer->GetXmlObjectDataSize(NULL, SizeData, true, bWithFileData, Model);
	}

	// External renderer
	// Get the renderer name
	string Renderer = CommandToExecute->Parameters.GetValue("NV_HREND");
	CleanString(Renderer);
	MakeUpper(Renderer);
	if(!Renderer.length())
		Renderer = "XSLT";

	bool bFileOutput = false;
	if((Renderer == "XSLT") && (SizeData > 100000))
		bFileOutput = true;	// Xslt renderer can process file output

	// If debug only, we always use file output
	if(DebugOnly)
		bFileOutput = true;

	// If use external xslt processor, we always use file output
	if((Renderer == "XSLT") && (!GetMain()->GetInternalXsl()))
		bFileOutput = true;

	NvObjFile ObjectResult;
	string StringResult;
	if(bFileOutput)
	{
		if(!ObjectResult.SetAutoFileName(Application->GetWorkDir(), "xml", Application->GetAppNameForFiles()))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE_TEMP_FILE, Application->GetWorkDir());
			if(!bIsOutputContainer)
				FreeContainer(XmlContainer, sXmlContainer.c_str(), false, CommandToExecute);
			return false;
		}

		if(!ObjectResult.Open("w"))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE_OPEN_FILE, ObjectResult.GetFilePathName());
			if(!bIsOutputContainer)
				FreeContainer(XmlContainer, sXmlContainer.c_str(), false, CommandToExecute);
			return false;
		}
	}

	int iXmlCoding = CommandToExecute->iXmlCoding;

	if(DebugOnly)
	{
		// For debug only the default is normal model
		iXmlCoding = XML_MODEL_NORMAL;
		string sModel = CommandToExecute->Parameters.GetValue("NV_XML_MODEL");
		CleanString(sModel, true);
		MakeUpper(sModel);
		string sSimpleModel = CommandToExecute->Parameters.GetValue("NV_XML_SIMPLE");
		CleanString(sSimpleModel, true);
		MakeUpper(sSimpleModel);
		if(sModel.length())
		{
			if(sModel == "SIMPLE")
				iXmlCoding = XML_MODEL_SIMPLE;
			if(sModel == "NORMAL")
				iXmlCoding = XML_MODEL_NORMAL;
			if(sModel == "COMPACT")
				iXmlCoding = XML_MODEL_COMPACT;
			if(sModel == "TINY")
				iXmlCoding = XML_MODEL_TINY;
		}
		else if(sSimpleModel == "YES")
			iXmlCoding = XML_MODEL_SIMPLE;
	}

	int NumTabs = 0;
	if(CommandToExecute->iXmlInputType == XML_IN_SOAP)
		NumTabs = 2;
	string Tabs = "";
	for(i=0; i<NumTabs; i++)
		Tabs += '\t';
	if(iXmlCoding == XML_MODEL_NORMAL)
		NumTabs++;

	string XmlData = "<?xml version=\"1.0\"";
	ReserveString(XmlData, 1000);
	if(sEncoding.length())
	{
		XmlData +=  " encoding=\"";
		XmlData += sEncoding;
		XmlData += "\"";
	}
	XmlData += " ?>\n";

	if(CommandToExecute->iXmlInputType == XML_IN_SOAP)
	{
		// SOAP envelope
		XmlData += "<env:Envelope xmlns:env=\"http://schemas.xmlsoap.org/soap/envelope/\">\n";
		XmlData += "\t<env:Body>\n";
	}

	XmlData += Tabs;
	XmlData += "<";

	string NsPrefix = "";
	string NsHref = "";
	string NameSpaceHere = CommandToExecute->m_sXmlNameSpace;
	if(CommandToExecute->iXmlInputType == XML_IN_SOAP)
	{
		if(CommandToExecute->bWebs)
		{
			// For a web service, the namespace must be the same than the wsdl

			string AppNameHere = Application->GetAppNameForFiles();
			MakeLower(AppNameHere);
			string Host = GetMain()->GetHostWebs();
			vector<string> Protocols;
			vector<string> Hosts;
			
			NvWebs::ParseHostName(Host, Protocols, Hosts);
			MakeUpper(WebsName);
			string NsPrefixInWebs = "";
			if(WebsName.length())
			{
				NvWebs* WebService = GetMain()->Webservices.GetWebservice(WebsName.c_str());
				if(WebService != NULL)
				{
					WebsName = WebService->GetName();
					NsPrefixInWebs = WebService->GetNsPrefix();
					GetMain()->Webservices.FreeWebservice(WebsName.c_str());
				}
			}

			MakeLower(WebsName);
			NameSpaceHere = "nvs=http://";
			if(NsPrefixInWebs.length())
				NameSpaceHere += NsPrefixInWebs;
			else if (Hosts.size())
				NameSpaceHere += Hosts[0];
			else
				NameSpaceHere += Host;
			NameSpaceHere += "/nirva/";
			NameSpaceHere += AppNameHere;
			NameSpaceHere += "/";
			NameSpaceHere += WebsName;
		}
	}
	if(NameSpaceHere.length())
	{
		index = NameSpaceHere.find('=');
		if(index != string::npos)
		{
			if(index)
				NsPrefix = NameSpaceHere.substr(0, index);
			if(index < (NameSpaceHere.length()-1))
				NsHref = NameSpaceHere.substr(index+1);
		}
	}

	if(NsHref.length() && NsPrefix.length())
	{
		XmlData += NsPrefix;
		XmlData += ":";
	}
	if(!MessageName.length())
	{
		MessageName = "NIRVA";
		if((iXmlCoding == XML_MODEL_SIMPLE) || (iXmlCoding == XML_MODEL_COMPACT) || (iXmlCoding == XML_MODEL_TINY)) 
			MessageName = "nirva";
	}
	XmlData += MessageName;
	if(NsHref.length())
	{
		XmlData += " xmlns";
		if(NsPrefix.length())
		{
			XmlData += ":";
			XmlData += NsPrefix;
		}
		XmlData += "=\"";
		XmlData += XmlObjectForCoding.EncodeOutputString(NsHref, bUnicode);
		XmlData += "\"";
	}
	if(NsPrefix.length())
		NsPrefix += ":";
	
	if(CommandToExecute->bWebs)
	{
		// Add the nvt namaspace
		XmlData += " xmlns:nvt=\"http://www.nirva-systems.com/2004-03-29/nirva-basetypes\"";
	}

	if(bSessionInfo)
	{
		XmlData += " session=\"";
		XmlData += XmlObjectForCoding.EncodeOutputString(m_sIdent, bUnicode);
		XmlData += "\" application=\"";
		XmlData += XmlObjectForCoding.EncodeOutputString(GetAppName(), bUnicode);
		XmlData += "\" user=\"";
		XmlData += XmlObjectForCoding.EncodeOutputString(GetUserName(), bUnicode);
		XmlData += "\" source=\"";
		if(CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT)
			XmlData += "CLIENT";
		if(CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER)
			XmlData += "BROWSER";
		if(CommandToExecute->m_iCommandSource == NV_SOURCE_PROCEDURE)
			XmlData += "PROCEDURE";
		if(CommandToExecute->m_iCommandSource == NV_SOURCE_SERVICE)
			XmlData += "SERVICE";
		XmlData += "\" host=\"";
		XmlData += XmlObjectForCoding.EncodeOutputString(CommandToExecute->m_sHost, bUnicode);
		XmlData += "\" local=\"";
		string HostHere = CommandToExecute->m_sHost;
		CleanString(HostHere);
		MakeUpper(HostHere);
		index = HostHere.find(':');
		if(index != string::npos)
			HostHere = HostHere.substr(0, index);
		if((HostHere == "LOCALHOST") || (HostHere == "127.0.0.1") || (HostHere == "::1"))
			XmlData += "YES";
		else
			XmlData += "NO";
		XmlData += "\"";
	}
	XmlData += ">\n";
	if(bFileOutput)
	{
		if(fwrite(XmlData.c_str(), 1, XmlData.length(), ObjectResult.GetFileDescriptor()) != XmlData.length())
		{
			CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, XmlContainer->GetName());
			ObjectResult.Close();
			if(!bIsOutputContainer)
				FreeContainer(XmlContainer, sXmlContainer.c_str(), false, CommandToExecute);
			return false;
		}
	}
	else
	{
		StringResult += XmlData;
		ReserveString(StringResult);
	}


	if(iXmlCoding == XML_MODEL_COMPACT)
		bWithHttpHeaders = false; // No headers in compact mode
	if(iXmlCoding == XML_MODEL_TINY)
		bWithHttpHeaders = false; // No headers in tiny mode
	if(bWithHttpHeaders)
	{
		map<string,string>::iterator pos;
		for(pos = CommandToExecute->HttpInHeaders.m_mList.begin(); pos != CommandToExecute->HttpInHeaders.m_mList.end(); ++pos)
		{
			XmlData = Tabs;
			XmlData += "\t<";
			XmlData += NsPrefix;
			if(iXmlCoding == XML_MODEL_SIMPLE)
				XmlData += "httpheader";
			else
				XmlData += "NVHTTPHEADER";
			XmlData += " key=\"";
			XmlData += XmlObjectForCoding.EncodeOutputString(pos->first, bUnicode);
			XmlData += "\">";
			XmlData += XmlObjectForCoding.EncodeOutputString(pos->second, bUnicode);
			XmlData += "</";
			XmlData += NsPrefix;
			if(iXmlCoding == XML_MODEL_SIMPLE)
				XmlData += "httpheader";
			else
				XmlData += "NVHTTPHEADER";
			XmlData += ">\n";
			if(bFileOutput)
			{
				if(fwrite(XmlData.c_str(), 1, XmlData.length(), ObjectResult.GetFileDescriptor()) != XmlData.length())
				{
					CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, XmlContainer->GetName());
					ObjectResult.Close();
					if(!bIsOutputContainer)
						FreeContainer(XmlContainer, sXmlContainer.c_str(), false, CommandToExecute);
					return false;
				}
			}
			else
			{
				StringResult += XmlData;
				ReserveString(StringResult);
			}
		}
		XmlData = "\n";
		if(bFileOutput)
		{
			if(fwrite(XmlData.c_str(), 1, XmlData.length(), ObjectResult.GetFileDescriptor()) != XmlData.length())
			{
				CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, XmlContainer->GetName());
				ObjectResult.Close();
				if(!bIsOutputContainer)
					FreeContainer(XmlContainer, sXmlContainer.c_str(), false, CommandToExecute);
				return false;
			}
		}
		else
		{
			StringResult += XmlData;
			ReserveString(StringResult);
		}
	}

	if(bWithVariables)
	{
		map<string,string>::iterator pos;
		for(pos = m_oVariables.m_mList.begin(); pos != m_oVariables.m_mList.end(); ++pos)
		{
			XmlData = Tabs;
			XmlData += "\t<";
			XmlData += NsPrefix;
			if(iXmlCoding == XML_MODEL_SIMPLE)
				XmlData += "variable";
			else if(iXmlCoding == XML_MODEL_COMPACT)
				XmlData += "variable";
			else if(iXmlCoding == XML_MODEL_TINY)
				XmlData += "variable";
			else
				XmlData += "NVSESSIONVAR";
			XmlData +=" key=\"";
			XmlData += XmlObjectForCoding.EncodeOutputString(pos->first, bUnicode);
			XmlData += "\">";
			XmlData += XmlObjectForCoding.EncodeOutputString(pos->second, bUnicode);
			XmlData += "</";
			XmlData += NsPrefix;
			if(iXmlCoding == XML_MODEL_SIMPLE)
				XmlData += "variable";
			else if(iXmlCoding == XML_MODEL_COMPACT)
				XmlData += "variable";
			else if(iXmlCoding == XML_MODEL_TINY)
				XmlData += "variable";
			else
				XmlData += "NVSESSIONVAR";
			XmlData += ">\n";
			if(bFileOutput)
			{
				if(fwrite(XmlData.c_str(), 1, XmlData.length(), ObjectResult.GetFileDescriptor()) != XmlData.length())
				{
					CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, XmlContainer->GetName());
					ObjectResult.Close();
					if(!bIsOutputContainer)
						FreeContainer(XmlContainer, sXmlContainer.c_str(), false, CommandToExecute);
					return false;
				}
			}
			else
			{
				StringResult += XmlData;
				ReserveString(StringResult);
			}
		}
		XmlData = "\n";
	
		if(bFileOutput)
		{
			if(fwrite(XmlData.c_str(), 1, XmlData.length(), ObjectResult.GetFileDescriptor()) != XmlData.length())
			{
				CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, XmlContainer->GetName());
				ObjectResult.Close();
				if(!bIsOutputContainer)
					FreeContainer(XmlContainer, sXmlContainer.c_str(), false, CommandToExecute);
				return false;
			}
		}
		else
		{
			StringResult += XmlData;
			ReserveString(StringResult);
		}
	}


	if(iXmlCoding == XML_MODEL_NORMAL)
	{
		XmlData = Tabs;
		XmlData += "\t<";
		XmlData += NsPrefix;
		XmlData += "NVCONTAINER name=\"";
		XmlData += XmlObjectForCoding.EncodeOutputString(XmlContainer->GetName(), bUnicode);
		XmlData += "\">\n";
		if(bFileOutput)
		{
			if(fwrite(XmlData.c_str(), 1, XmlData.length(), ObjectResult.GetFileDescriptor()) != XmlData.length())
			{
				CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, XmlContainer->GetName());
				ObjectResult.Close();
				if(!bIsOutputContainer)
					FreeContainer(XmlContainer, sXmlContainer.c_str(), false, CommandToExecute);
				return false;
			}
		}
		else
		{
			StringResult += XmlData;
			ReserveString(StringResult);
		}
	}

	for(i=0; i<ObjectsToGet.GetSize(); i++)
	{
		string sObjectInList = ObjectsToGet.GetValue(i);
		// First get requested objects
		if(bFileOutput)
		{
			if(!(XmlContainer->GetXmlObjectDataToStream(ObjectsToGet.GetValue(i).c_str(), ObjectResult.GetFileDescriptor(), NumTabs, false, bWithData, bWithFileData, NsPrefix, iXmlCoding, Model, &XmlObjectForCoding, bUnicode, CommandToExecute->bWebs)))
			{
				CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, ObjectsToGet.GetValue(i));
				ObjectResult.Close();
				if(!bIsOutputContainer)
					FreeContainer(XmlContainer, sXmlContainer.c_str(), false, CommandToExecute);
				return false;
			}
		}
		else
		{
			if(!(XmlContainer->GetXmlObjectDataToString(ObjectsToGet.GetValue(i).c_str(), StringResult, NumTabs, false, bWithData, bWithFileData, NsPrefix, iXmlCoding, Model, &XmlObjectForCoding, bUnicode, CommandToExecute->bWebs)))
			{
				CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, ObjectsToGet.GetValue(i));
				if(!bIsOutputContainer)
					FreeContainer(XmlContainer, sXmlContainer.c_str(), false, CommandToExecute);
				return false;
			}
			ReserveString(StringResult);
		}
	}

	// Then get enventual subcontainers
	if(bWithSubContainers)
	{
		if(bFileOutput)
		{
			if(!(XmlContainer->GetXmlObjectDataToStream(NULL, ObjectResult.GetFileDescriptor(), NumTabs+1, bWithSubContainers, bWithData, bWithFileData, NsPrefix, iXmlCoding, Model, &XmlObjectForCoding, bUnicode, CommandToExecute->bWebs)))
			{
				CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, XmlContainer->GetName());
				ObjectResult.Close();
				if(!bIsOutputContainer)
					FreeContainer(XmlContainer, sXmlContainer.c_str(), false, CommandToExecute);
				return false;
			}
		}
		else
		{
			if(!(XmlContainer->GetXmlObjectDataToString(NULL, StringResult, NumTabs+1, bWithSubContainers, bWithData, bWithFileData, NsPrefix, iXmlCoding, Model, &XmlObjectForCoding, bUnicode, CommandToExecute->bWebs)))
			{
				CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, XmlContainer->GetName());
				if(!bIsOutputContainer)
					FreeContainer(XmlContainer, sXmlContainer.c_str(), false, CommandToExecute);
				return false;
			}
			ReserveString(StringResult);
		}
	}

	XmlData = "";
	if(iXmlCoding == XML_MODEL_NORMAL)
	{
		XmlData = Tabs;
		XmlData += "\t</";
		XmlData += NsPrefix;
		XmlData += "NVCONTAINER";
		XmlData += ">\n";
	}

	XmlData += Tabs;
	XmlData += "</";
	XmlData += NsPrefix;
	XmlData += MessageName;
	XmlData += ">";

	if(CommandToExecute->iXmlInputType == XML_IN_SOAP)
	{
		// End SOAP envelope
		XmlData += "\n\t</env:Body>\n";
		XmlData += "</env:Envelope>\n";
	}

	if(bFileOutput)
	{
		if(fwrite(XmlData.c_str(), 1, XmlData.length(), ObjectResult.GetFileDescriptor()) != XmlData.length())
		{
			CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, XmlContainer->GetName());
			ObjectResult.Close();
			if(!bIsOutputContainer)
				FreeContainer(XmlContainer, sXmlContainer.c_str(), false, CommandToExecute);
			return false;
		}
	}
	else
	{
		StringResult += XmlData;
		ReserveString(StringResult);
	}

	// If a special container has been requested (not the output container), we must free it
	// This is not necessary for the ouput container because this is done before calling the function
	if(!bIsOutputContainer)
		FreeContainer(XmlContainer, sXmlContainer.c_str(), false, CommandToExecute);


	char *OutputData = NULL;
	unsigned long OutputDataSize = (int)(StringResult.length());
	if(bFileOutput)
		ObjectResult.Close();
	else
	{
		OutputData = (char*)malloc(OutputDataSize);
		if(OutputData == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_OBJECT_DATA, XmlContainer->GetName());
			return false;
		}
		memcpy(OutputData, StringResult.c_str(), OutputDataSize);
	}

	if(!DebugOnly)
		CommandToExecute->FreeData();

	
	// Get the renderer page
	string RendererPage = CommandToExecute->Parameters.GetValue("NV_HREND_PAGE");
	CleanString(RendererPage);
	if(CommandToExecute->iXmlInputType == XML_IN_SOAP)
		RendererPage = "";	// No Renderer for the SOAP or WEBS connectors
	
	if(!RendererPage.length())
	{
		// If no renderer page, we check the NV_XML_XSL parameter for compatibility reason
		RendererPage = CommandToExecute->Parameters.GetValue("NV_XML_XSL");
		CleanString(RendererPage);
		if(CommandToExecute->iXmlInputType == XML_IN_SOAP)
			RendererPage = "";	// No XSL for the SOAP or WEBS connectors
	}
	string XmlFileForDebug = RendererPage;
	if(bDebug)
	{
		string XmlFileForDebugFromCommand = CommandToExecute->Parameters.GetValue("NV_XML_DEBUG");
		CleanString(XmlFileForDebugFromCommand,true);
		if(XmlFileForDebugFromCommand.length())
		{
			XmlFileForDebug = XmlFileForDebugFromCommand;
			index = XmlFileForDebug.rfind(".xml");
			if(index != string::npos)
				XmlFileForDebug.erase(index);
		}
		else
		{
//			index = XmlFileForDebug.rfind('/');
//			if(index != string::npos)
//				XmlFileForDebug.erase(0, index+1);
//			index = XmlFileForDebug.rfind('\\');
//			if( index != string::npos)
//				XmlFileForDebug.erase(0, index+1);
		}

		if(XmlFileForDebug.length())
			if(XmlFileForDebug[XmlFileForDebug.length()-1] == ')')
				XmlFileForDebug.erase(XmlFileForDebug.length()-1);
		index = XmlFileForDebug.find('(');
		if(index != string::npos)
		{
			// Service file
			string service = XmlFileForDebug.substr(0, index);
			XmlFileForDebug.erase(0, index+1);
			CleanString(service);
			if(!service.length())
				service = "system";
			MakeLower(service);
			string new_XmlFileForDebug = "services";
			AddDirSuffix(new_XmlFileForDebug);
			new_XmlFileForDebug += service;
			AddDirSuffix(new_XmlFileForDebug);
			new_XmlFileForDebug += XmlFileForDebug;
			XmlFileForDebug = new_XmlFileForDebug;
		}
		index = XmlFileForDebug.rfind(".xsl");
		if(index != string::npos)
			XmlFileForDebug.erase(index);
		index = XmlFileForDebug.rfind('.');
		if(index != string::npos)
			XmlFileForDebug.erase(index);

		MakeLower(XmlFileForDebug);

		if(XmlFileForDebug.length())
		{
			string DebugFile = Application->GetFileDir();
			AddDirSuffix(DebugFile);
			DebugFile += "Debug";
			CreateDir(DebugFile.c_str());
			AddDirSuffix(DebugFile);
			DebugFile += XmlFileForDebug;
			string DebugFileTmp = DebugFile;
			DebugFileTmp += ".tmp";
			DebugFile += ".xml";

			string DirName = DebugFile;
			SetDirName(DirName);
#ifdef WIN32
			index = DirName.rfind('\\');
#else
			index = DirName.rfind('/');
#endif
			if(index != string::npos)
				DirName = DirName.substr(0, index);
			if(CreateDir(DirName.c_str(), false))
			{
				NvObjFile DebugFileObj;
				DebugFileObj.SetPersistTime(-1);
				DebugFileObj.SetFileName(DebugFileTmp);
				if(bFileOutput)
					DebugFileObj.LoadFromFile(ObjectResult.GetFilePathName().c_str());
				else
					DebugFileObj.Populate(OutputData, OutputDataSize);
				unlink(DebugFile.c_str());
				rename(DebugFileTmp.c_str(), DebugFile.c_str());
			}
		}
	}

	if(DebugOnly)
	{
		// We leave here because we have nothing more to do
		if(OutputData != NULL)
			free(OutputData);
		return true;
	}

	if(RendererPage.length())
	{
		bool bExternalRenderer = false;
		NvObjFile ObjectFinalResult;
		if(bFileOutput)
		{
			if(!ObjectFinalResult.SetAutoFileName(Application->GetWorkDir(), "htm", Application->GetAppNameForFiles()))
			{
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE_TEMP_FILE, Application->GetWorkDir());
				if(OutputData != NULL)
					free(OutputData);
				return false;
			}
		}
		if(Renderer == "XSLT")
		{
			// Get the real xsl file path name
			if(!GetXslFile(CommandToExecute, RendererPage, Application->GetName()))
			{
				if(OutputData != NULL)
					free(OutputData);
				return false;
			}
			// Now we have the complete Xsl file path in RendererPage

			NvXml XmlParser;
			if(bFileOutput)
			{
				if(!ParseXml(CommandToExecute, ObjectResult.GetFilePathName().c_str(), ObjectFinalResult.GetFilePathName().c_str(), RendererPage.c_str()))
				{
					if(OutputData != NULL)
						free(OutputData);
					return false;
				}
			}
			else
			{
				if(!XmlParser.XsltTransform(&OutputData, &OutputDataSize, RendererPage.c_str()))
				{
					CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_XSL_PROCESSING_ERROR);
					if(OutputData != NULL)
						free(OutputData);
					return false;
				}
			}
		}
		else if(Renderer == "NVFM")
		{
			// Internal nirva framework
			if(!Application->GetUseFramework())
			{
				CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_DISABLE, Application->GetName());
				if(OutputData != NULL)
					free(OutputData);
				return false;
			}
			if(!Application->NidgetFramework.GetView(RendererPage.c_str(), m_sIdent.c_str(), CommandToExecute, &OutputData, &OutputDataSize, bDebug))
			{
				if(OutputData != NULL)
					free(OutputData);
				return false;
			}
			CommandToExecute->HttpOutHeaders.SetValue("Content-Type", "application/xhtml+xml");
		}
		else
		{
			//External renderer
			if(!RenderXml(CommandToExecute, &OutputData, &OutputDataSize, Renderer.c_str(), RendererPage.c_str(), sEncoding.c_str()))
			{
				if(OutputData != NULL)
					free(OutputData);
				return false;
			}
			bExternalRenderer = true;
			string RendererContentType = CommandToExecute->HttpOutHeaders.GetValue("Content-Type");
			if(RendererContentType != "text/html")
				bHtmlOutput = false;

		}
		if(bDebug && bHtmlOutput)
		{
			string DebugFile = Application->GetFileDir();
			AddDirSuffix(DebugFile);
			DebugFile += "Debug";
			CreateDir(DebugFile.c_str());
			AddDirSuffix(DebugFile);
			DebugFile += XmlFileForDebug;
			DebugFile += ".htm";
			NvObjFile DebugFileObj;
			DebugFileObj.SetPersistTime(-1);
			DebugFileObj.SetFileName(DebugFile);
			if(bFileOutput) 
				DebugFileObj.LoadFromFile(ObjectFinalResult.GetFilePathName().c_str());
			else
				DebugFileObj.Populate(OutputData, OutputDataSize);
		}

		bool bFileMessage = false;
		if(bFileOutput)
		{
			NvInt64 FileSize = ObjectFinalResult.GetSize();
			if(FileSize > (NvInt64)(GetMain()->GetHttpMaxMemBody()))	// for files grater than 1024 kbytes
				bFileMessage = true;
		}
		
		if(bFileMessage)
		{
			string ObjectFileName = ObjectFinalResult.GetFilePathName();

			// We must work with a copy of the object because the session can be closed
			// before sending the file object to the client
			string DirHere = GetMain()->GetNirvaDir();
			DirHere += "Work";
			string FileNameHere;
			if(!GetTemporaryFile(FileNameHere, DirHere, "tmp", "", ""))
			{
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET_DATA, ObjectFinalResult.GetFilePathName());
				if(OutputData != NULL)
					free(OutputData);
				return false;
			}
			if(!CopyFileEx(ObjectFileName.c_str(), FileNameHere.c_str()))
			{
				RemoveFile(FileNameHere.c_str());
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET_DATA, ObjectFinalResult.GetFilePathName());
				if(OutputData != NULL)
					free(OutputData);
				return false;
			}
			CommandToExecute->DataFile = FileNameHere;
			CommandToExecute->bDeleteDataFile = true;
		}
		else
		{
			// Not a too big file so we use memory data
			if(bFileOutput)
			{
				if(!ObjectFinalResult.GetData(&(CommandToExecute->Data), &(CommandToExecute->DataSize)))
				{
					CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET_DATA, ObjectFinalResult.GetFilePathName());
					if(OutputData != NULL)
						free(OutputData);
					return false;
				}
			}
			else
			{
				CommandToExecute->Data = OutputData;
				CommandToExecute->DataSize = OutputDataSize;
				OutputData = NULL;
				OutputDataSize = 0;
			}
		}

		// If we have xml input, we suppose that output is also xml
		if(!bExternalRenderer)
		{
			if(bHtmlOutput)
				CommandToExecute->HttpOutHeaders.SetValue("Content-Type", "text/html" );
			else
			{
				if(OutputType.length() && ((OutputTypeHere == "XML") || (OutputTypeHere == "SOAP")))
					CommandToExecute->HttpOutHeaders.SetValue("Content-Type", "text/xml" );
				else
					CommandToExecute->HttpOutHeaders.SetValue("Content-Type", OutputType.c_str());
			}
		}
	}
	else
	{
		bool bDoXml = true;
		if((CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER) && (CommandToExecute->iXmlInputType == XML_IN_NONE))
		{
			// Check the autorization here
			if(!Application->GetAllowWebXml())
				if(!CheckPermission(NULL, "SYSTEM", "DISPLAY_XML_BROWSER"))
					bDoXml = false;
		}

		if(bDoXml)
		{
			bool bFileMessage = false;
			if(bFileOutput)
			{
				NvInt64 FileSize = ObjectResult.GetSize();
				if(FileSize > (NvInt64)(GetMain()->GetHttpMaxMemBody()))	// for files greater than 1024 kbytes
					bFileMessage = true;
			}
			
			if(bFileMessage)
			{
				string ObjectFileName = ObjectResult.GetFilePathName();

				// We must work with a copy of the object because the session can be closed
				// before sending the file object to the client
				string DirHere = GetMain()->GetNirvaDir();
				DirHere += "Work";
				string FileNameHere;
				if(!GetTemporaryFile(FileNameHere, DirHere, "tmp", "", ""))
				{
					CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET_DATA, ObjectResult.GetFilePathName());
					if(OutputData != NULL)
						free(OutputData);
					return false;
				}
				if(!CopyFileEx(ObjectFileName.c_str(), FileNameHere.c_str()))
				{
					RemoveFile(FileNameHere.c_str());
					CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET_DATA, ObjectResult.GetFilePathName());
					if(OutputData != NULL)
						free(OutputData);
					return false;
				}
				CommandToExecute->DataFile = FileNameHere;
				CommandToExecute->bDeleteDataFile = true;
			}
			else
			{
				// Not a too big file so we use memory data
				if(bFileOutput)
				{
					if(!ObjectResult.GetData(&(CommandToExecute->Data), &(CommandToExecute->DataSize)))
					{
						CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET_DATA, ObjectResult.GetFilePathName());
						if(OutputData != NULL)
							free(OutputData);
						return false;
					}
				}
				else
				{
					CommandToExecute->Data = OutputData;
					CommandToExecute->DataSize = OutputDataSize;
					OutputData = NULL;
				}
			}
			CommandToExecute->HttpOutHeaders.SetValue("Content-Type", "text/xml" );
		}
		else
		{
			// Standard Html output
			TestBrowser(CommandToExecute);
		}
	}

	if(OutputData != NULL)
		free(OutputData);

	string HrendDisableCache = CommandToExecute->Parameters.GetValue("NV_HREND_DISABLE_CACHE");
	CleanString(HrendDisableCache,true);
	MakeUpper(HrendDisableCache);
	if(HrendDisableCache != "NO")
	{
		CommandToExecute->HttpOutHeaders.SetValue("Cache-Control", "no-cache");
		CommandToExecute->HttpOutHeaders.SetValue("Pragma", "no-cache");
		CommandToExecute->HttpOutHeaders.SetValue("Expires", "Wed, 26 Feb 1997 08:21:57 GMT"); // old date
	}
	return true;
}



bool NvSession::SetXmlInput(NvCommand *CommandToExecute)
{
	if(!CommandToExecute->iXmlInputType)
		return true;	// Nothing to do here

	// The function uses the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	NvXml XmlParser;
	string NameSpace;
	int XmlCoding = XML_MODEL_NORMAL;
	bool RetVal = XmlParser.SetInputData(InContainer, CommandToExecute, NameSpace, XmlCoding);
	if(XmlCoding != CommandToExecute->iXmlCoding)
	{
		// We must verify if we apply automatic selection of XML output model or not

		// We must verify if we apply automatic selection of XML output model or not
		string NvXmlModel = CommandToExecute->Parameters.GetValue("NV_XML_MODEL");
		CleanString(NvXmlModel, true);
		MakeUpper(NvXmlModel);
		if((NvXmlModel != "NORMAL") && (NvXmlModel != "SIMPLE") && (NvXmlModel != "COMPACT") && (NvXmlModel != "TINY"))
		{
			string sSimpleModel = CommandToExecute->Parameters.GetValue("NV_XML_SIMPLE");
			CleanString(sSimpleModel, true);
			MakeUpper(sSimpleModel);
			if((sSimpleModel != "NO") && (sSimpleModel != "YES"))
				CommandToExecute->iXmlCoding = XmlCoding;	// We apply automatic selection
		}
	}

	CommandToExecute->FreeData();
	
	return RetVal;
}


bool NvSession::XsltInputTransform(NvCommand *CommandToExecute, string XslParam, string AppName)
{
	if(!AppName.length())
		return true;		// nothing to do

	if(!CommandToExecute->iXmlInputType)
		return true;	// Nothing to do here

	// Get object names that may be in parameters
	string XslFile = XslParam;
	CleanString(XslFile);

	if(!XslFile.length())
		return true;		// nothing to do

	NvApplication *ApplicationHere = GetMain()->Applications.GetApplication(AppName.c_str());
	if(ApplicationHere == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_NOT_FOUND, AppName);
		return false;
	}
	string AppFileDir = ApplicationHere->GetFileDir();
	string AppWorkDir = ApplicationHere->GetWorkDir();
	string AppNameForFiles = ApplicationHere->GetAppNameForFiles();
	GetMain()->Applications.FreeApplication(AppName.c_str());

	if(bDebug)
	{
		string DebugFile = AppFileDir;
		AddDirSuffix(DebugFile);
		DebugFile += "Debug";
		CreateDir(DebugFile.c_str());
		AddDirSuffix(DebugFile);
		DebugFile += m_sIdent;
		DebugFile += ".xml";
		NvObjFile DebugFileObj;
		DebugFileObj.SetPersistTime(-1);
		DebugFileObj.SetFileName(DebugFile);
		if(CommandToExecute->DataFile.length())
			DebugFileObj.LoadFromFile(CommandToExecute->DataFile.c_str());
		else
			DebugFileObj.Populate(CommandToExecute->Data, CommandToExecute->DataSize);
	}

	// Get the real xsl file path name
	if(!GetXslFile(CommandToExecute, XslFile, AppName))
		return false;
	// Now we have the complete Xsl file path in XslFile

	// Check if we use internal or external XSLT processor
	if(GetMain()->GetInternalXsl())
	{
		NvXml XmlParser;
		if(CommandToExecute->DataFile.length())
		{
			NvObjFile DstFile;
			string sWorkDir = "";
			string sPrefix = "";
			sWorkDir = AppWorkDir;
			sPrefix = AppNameForFiles;
			if(!DstFile.SetAutoFileName(sWorkDir, "xml", sPrefix))
			{
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE_TEMP_FILE, AppWorkDir);
				return false;
			}
		
			if(!ParseXml(CommandToExecute, CommandToExecute->DataFile.c_str(), DstFile.GetFilePathName().c_str(), XslFile.c_str()))
				return false;
		
			DstFile.SaveToFile(CommandToExecute->DataFile.c_str());
		}
		else
		{
			if(!XmlParser.XsltTransform(&(CommandToExecute->Data), &(CommandToExecute->DataSize), XslFile.c_str()))
			{
				CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_XSL_PROCESSING_ERROR);
				return false;
			}
		}
	}
	else
	{
		// If using external XSLT processor, we can work only on files
		NvObjFile SrcFile;
		NvObjFile DstFile;
		string sWorkDir = "";
		string sPrefix = "";
		sWorkDir = AppWorkDir;
		sPrefix = AppNameForFiles;
		if(!SrcFile.SetAutoFileName(sWorkDir, "xml", sPrefix))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE_TEMP_FILE, AppWorkDir);
			return false;
		}
		if(!DstFile.SetAutoFileName(sWorkDir, "xml", sPrefix))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE_TEMP_FILE, AppWorkDir);
			return false;
		}

		if(CommandToExecute->DataFile.length())
			SrcFile.LoadFromFile(CommandToExecute->DataFile.c_str());
		else
			SrcFile.Populate(CommandToExecute->Data, CommandToExecute->DataSize);
		
		if(!ParseXml(CommandToExecute, SrcFile.GetFilePathName().c_str(), DstFile.GetFilePathName().c_str(), XslFile.c_str()))
			return false;

		if(CommandToExecute->DataFile.length())
		{
			DstFile.SaveToFile(CommandToExecute->DataFile.c_str());
		}
		else
		{
			CommandToExecute->FreeData();
			DstFile.GetData(&(CommandToExecute->Data), &(CommandToExecute->DataSize));
		}
	}
	
	if(bDebug)
	{
		string DebugFile = AppFileDir;
		AddDirSuffix(DebugFile);
		DebugFile += "Debug";
		CreateDir(DebugFile.c_str());
		AddDirSuffix(DebugFile);
		DebugFile += m_sIdent;
		DebugFile += ".xml";
		NvObjFile DebugFileObj;
		DebugFileObj.SetPersistTime(-1);
		DebugFileObj.SetFileName(DebugFile);
		if(CommandToExecute->DataFile.length())
			DebugFileObj.LoadFromFile(CommandToExecute->DataFile.c_str());
		else
			DebugFileObj.Populate(CommandToExecute->Data, CommandToExecute->DataSize);
	}


	return true;
}

bool NvSession::GetXslFile(NvCommand *CommandToExecute, string& XslParam, string AppName)
{
	CleanString(XslParam);

	// Try to get the Xsl file
	// By default, we have an application xsl file
	NvApplication *ApplicationHere = GetMain()->Applications.GetApplication(AppName.c_str());
	if(ApplicationHere == NULL)
	{
		if(CommandToExecute != NULL)
			CommandToExecute->Error.SetError("SYSTEM", "APPLICATION", NV_APPLICATION_NOT_FOUND, AppName);
		return false;
	}
	string AppFileDir = ApplicationHere->GetFileDir();
	GetMain()->Applications.FreeApplication(AppName.c_str());

	string XslFile = AppFileDir;
	
	// Check if this is a system or service Xsl (name is enclosed in parentheses preceded eventuall by the service name)
	// ex: Toto(zut) -> file zut.xsl for service toto
	string::size_type index = XslParam.find('(');
	if(index != string::npos)
	{
		if(XslParam[XslParam.length() - 1] != ')')
		{
			if(CommandToExecute != NULL)
				CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_INVALID_XSL_NAME, XslParam);
			return false;
		}

		string ServiceName = "";
		// this is a system or service xsl
		if(index != 0)
		{
			// it's probably a service xsl
			// we must get the service name
			ServiceName = XslParam.substr(0, index);
			CleanString(ServiceName);
			MakeUpper(ServiceName);
			XslParam = XslParam.substr(index);
		}

		// We must remove the parentheses
		XslParam = XslParam.substr(1, XslParam.length() - 2);
		
		// And get the xsl file from system or service Files directory
		if(ServiceName.length())
		{
			NvService *Service = GetMain()->Services.GetService(ServiceName.c_str(), true, true);
			if(Service == NULL)
			{
				if(CommandToExecute != NULL)
					CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, ServiceName);
				return false;
			}
			XslFile = Service->GetFileDir();
			GetMain()->Services.FreeService(ServiceName.c_str());
			AddDirSuffix(XslFile);
		}
		else
		{
			XslFile = GetMain()->GetNirvaDir();
			AddDirSuffix(XslFile);
			XslFile += "Files";
			AddDirSuffix(XslFile);
		}
	}

	// if no extension, we give the .xsl extension
	index = XslParam.rfind('.');
	if(index == string::npos)
		XslParam += ".xsl";


	XslFile += XslParam;
	if(XslFile.find("..") != string::npos)
	{
		if(CommandToExecute != NULL)
			CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_XSL_FILE, XslFile);
		return false;
	}
	if(!GetNoCaseRealFileName(XslFile))
	{
		if(CommandToExecute != NULL)
			CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_XSL_FILE, XslFile);
		return false;
	}

	XslParam = XslFile;
	
	return true;
}



bool NvSession::ParseXml(NvCommand *CommandToExecute, const char* XmlSrc, const char* XmlDest, const char* Xsl, NvObjIndStringList *Parameters, bool SkipDtd)
{
	string XmlSrcFile = XmlSrc;
	string XmlDestFile = XmlDest;
	string XslFile = Xsl;
	
	if(!GetNoCaseRealFileName(XslFile))
	{
		CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_CANNOT_GET_XSL_FILE, Xsl);
		return false;
	}

	// Check if we use internal or external XSLT processor
	if(GetMain()->GetInternalXsl())
	{
		// Internal one
		NvXml XmlParser;
		if(!XmlParser.XsltTransform(XmlSrcFile.c_str(), XmlDestFile.c_str(), XslFile.c_str(), Parameters, SkipDtd))
		{
			CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_XSL_PROCESSING_ERROR);
			return false;
		}
		return true;
	}

	// External one

	// Now we have the complete Xsl file path in XslFile
	string CommandLine = GetMain()->GetXslPath();
	CleanString(CommandLine);
	if(!CommandLine.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_NO_XSL_PROCESSOR, XslFile);
		return false;
	}

	string CommandLineForSearch = CommandLine;
	MakeUpper(CommandLineForSearch);
	string::size_type index = CommandLineForSearch.find("%INPUTFILE");
	if(index == string::npos)
	{
		CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_INVALID_XSL_PROCESSOR_COMMAND, CommandLine);
		return false;
	}
	CommandLineForSearch = CommandLine.substr(0, index);
	CommandLineForSearch += XmlSrcFile;
	CommandLineForSearch += CommandLine.substr(index+10);
	CommandLine = CommandLineForSearch;

	CommandLineForSearch = CommandLine;
	MakeUpper(CommandLineForSearch);
	index = CommandLineForSearch.find("%OUTPUTFILE");
	if(index == string::npos)
	{
		CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_INVALID_XSL_PROCESSOR_COMMAND, CommandLine);
		return false;
	}
	CommandLineForSearch = CommandLine.substr(0, index);
	CommandLineForSearch += XmlDestFile;
	CommandLineForSearch += CommandLine.substr(index+11);
	CommandLine = CommandLineForSearch;


	CommandLineForSearch = CommandLine;
	MakeUpper(CommandLineForSearch);
	index = CommandLineForSearch.find("%XSLFILE");
	if(index == string::npos)
	{
		CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_INVALID_XSL_PROCESSOR_COMMAND, CommandLine);
		return false;
	}
	CommandLineForSearch = CommandLine.substr(0, index);
	CommandLineForSearch += XslFile;
	CommandLineForSearch += CommandLine.substr(index+8);
	CommandLine = CommandLineForSearch;

	int RetCode;
	if(ExecPrgm(CommandLine.c_str(), &RetCode, -1) != 0)
	{
		CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_XSL_PROCESSING_ERROR, CommandLine);
		return false;
	}

	if(RetCode != 0)
	{
		CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_XSL_PROCESSING_ERROR, CommandLine);
		return false;
	}

	return true;
}

bool NvSession::RenderXml(NvCommand *CommandToExecute, char **Data, unsigned long *DataSize, const char *Renderer, const char *RendererPage, const char *Encoding)
{
	// Search for service here
	NvService *Service = GetMain()->Services.GetService(Renderer, true, true);
	if(Service == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, Renderer);
		return false;
	}

	string ServiceNameHere = Service->GetName();

	int i;
	bool bServiceEverInLocalList = false;
	NV_LOCKLOCK(&ProtectInUse);	// The access to m_oServices is protected by the InUse Lock for the session reporting
	for(i=0; i<m_oServices.GetSize(); i++)
	{
		if(m_oServices.GetValue(i) == ServiceNameHere)
		{
			bServiceEverInLocalList = true;
			break;
		}
	}
	NV_UNLOCKLOCK(&ProtectInUse);
	if(!bServiceEverInLocalList)
	{
		// This is the first call from this session to the service
		// So we must send a SYSTEM NV_INIT_SESSION command to the service
		NvCommand RealCommand;
		RealCommand.m_pSession = this;
		MyNvServiceCommand Command;
		if(Service->m_iLanguage == LANGUAGE_JAVA)
			Command.i_Encoding = ENCODING_UTF8;
		else if(Service->m_iLanguage == LANGUAGE_DOTNET)
			Command.i_Encoding = ENCODING_UTF8;
		else
			Command.i_Encoding = Service->m_iEncoding;
		Command.m_pCommand = &RealCommand;
		RealCommand.m_iCommandSource = NV_SOURCE_SERVER;
		RealCommand.m_sService = ServiceNameHere;
		RealCommand.m_sClass = "SYSTEM";
		RealCommand.m_sCommand = "NV_INIT_SESSION";
		RealCommand.m_sLanguage = CommandToExecute->m_sLanguage;
		if(!Service->Command(&Command))
		{
			CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_INIT_SESSION, Renderer);
			GetMain()->Services.FreeService(Service->GetName().c_str());
			return false;
		}
		NV_LOCKLOCK(&ProtectInUse);	// The access to m_oServices is protected by the InUse Lock for the session reporting
		m_oServices.Add(ServiceNameHere);
		NV_UNLOCKLOCK(&ProtectInUse);
	}


	MyNvRendererInt	RendererInt;
	RendererInt.m_pInputData = (char *)malloc(*DataSize);
	if(RendererInt.m_pInputData == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "SYSTEM", NV_SYSTEM_NOT_ENOUGH_MEMORY);
		GetMain()->Services.FreeService(Service->GetName().c_str());
		return false;
	}
	RendererInt.m_uInputDataSize = *DataSize;
	memcpy(RendererInt.m_pInputData, *Data, *DataSize);
	if(*Data != NULL)
		free(*Data);
	*Data = NULL;
	*DataSize = 0;

	if(Service->m_iLanguage == LANGUAGE_JAVA)
		RendererInt.i_Encoding = ENCODING_UTF8;
	else if(Service->m_iLanguage == LANGUAGE_DOTNET)
		RendererInt.i_Encoding = ENCODING_UTF8;
	else
		RendererInt.i_Encoding = Service->m_iEncoding;
	RendererInt.m_sPage = RendererPage;
	RendererInt.m_sInputEncoding = Encoding;
	EncodeCommandResult(RendererInt.i_Encoding, RendererInt.m_sPage);
	RendererInt.m_sSessionId = m_sIdent;
	EncodeCommandResult(RendererInt.i_Encoding, RendererInt.m_sSessionId);
	if(Application != NULL)
		RendererInt.m_sAppName = Application->GetName();
	EncodeCommandResult(RendererInt.i_Encoding, RendererInt.m_sAppName);

	bool RetVal = Service->Render(&RendererInt, this);
	
	EncodeCommandInputString(RendererInt.i_Encoding, RendererInt.m_sReason);
	EncodeCommandInputString(RendererInt.i_Encoding, RendererInt.m_sContentType);
	if(!RetVal)
	{
		// Renderer error
		string sInfo = Renderer;
		sInfo += " Renderer:";
		if(RendererInt.m_sReason.length())
			sInfo += RendererInt.m_sReason;
		else
			sInfo += "Unknown reason";

		CommandToExecute->Error.SetError("SYSTEM", "RENDERER", NV_RENDERER_PROCESSING_ERROR, sInfo);
	}

	GetMain()->Services.FreeService(ServiceNameHere.c_str());
	if(RendererInt.m_sContentType.length())
		CommandToExecute->HttpOutHeaders.SetValue("Content-Type", RendererInt.m_sContentType.c_str());
	else
		CommandToExecute->HttpOutHeaders.SetValue("Content-Type", "text/html");
	
	// We take the ownership of output data
	*Data = RendererInt.m_pOutputData;
	*DataSize = RendererInt.m_uOutputDataSize;
	
	RendererInt.m_pOutputData = NULL;
	RendererInt.m_uOutputDataSize = 0;

	CommandToExecute->m_sHttpRet = RendererInt.m_sHttpRet;

	return RetVal;
}


// Process the service commands
// Called only from Command function
bool NvSession::ProcessServiceCommand(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Search for service here
	NvService *Service = GetMain()->Services.GetService(CommandToExecute->m_sService.c_str(), true, true);
	if(Service == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_GET, CommandToExecute->m_sService);
		return false;
	}

	string ServiceNameHere = Service->GetName();

	int i;
	bool bServiceEverInLocalList = false;
	NV_LOCKLOCK(&ProtectInUse);	// The access to m_oServices is protected by the InUse Lock for the session reporting
	for(i=0; i<m_oServices.GetSize(); i++)
	{
		if(m_oServices.GetValue(i) == ServiceNameHere)
		{
			bServiceEverInLocalList = true;
			break;
		}
	}
	NV_UNLOCKLOCK(&ProtectInUse);
	if(!bServiceEverInLocalList)
	{
		// This is the first call from this session to the service
		// So we must send a SYSTEM NV_INIT_SESSION command to the service
		NvCommand RealCommand;
		RealCommand.m_pSession = this;
		MyNvServiceCommand Command;
		if(Service->m_iLanguage == LANGUAGE_JAVA)
			Command.i_Encoding = ENCODING_UTF8;
		else if(Service->m_iLanguage == LANGUAGE_DOTNET)
			Command.i_Encoding = ENCODING_UTF8;
		else
			Command.i_Encoding = Service->m_iEncoding;
		Command.m_pCommand = &RealCommand;
		RealCommand.m_iCommandSource = NV_SOURCE_SERVER;
		RealCommand.m_sService = ServiceNameHere;
		RealCommand.m_sClass = "SYSTEM";
		RealCommand.m_sCommand = "NV_INIT_SESSION";
		RealCommand.m_sLanguage = CommandToExecute->m_sLanguage;
		if(!Service->Command(&Command))
		{
			CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_CANNOT_INIT_SESSION, CommandToExecute->m_sService);
			GetMain()->Services.FreeService(Service->GetName().c_str());
			return false;
		}
		NV_LOCKLOCK(&ProtectInUse);	// The access to m_oServices is protected by the InUse Lock for the session reporting
		m_oServices.Add(ServiceNameHere);
		NV_UNLOCKLOCK(&ProtectInUse);
	}


	MyNvServiceCommand *ServiceCommand = NVNEW MyNvServiceCommand;
	if(Service->m_iLanguage == LANGUAGE_JAVA)
		ServiceCommand->i_Encoding = ENCODING_UTF8;
	else if(Service->m_iLanguage == LANGUAGE_DOTNET)
		ServiceCommand->i_Encoding = ENCODING_UTF8;
	else
		ServiceCommand->i_Encoding = Service->m_iEncoding;
	ServiceCommand->m_pCommand = CommandToExecute;

	bool RetVal = Service->Command(ServiceCommand);
	if(RetVal)
	{
		// Get the last output buffer
		GetLastOutputBuffer(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		// Reset the last output buffer (PM do not reset because otherwise the GET_LAST_OUTPUT_BUFFER is n ot working)
//		CommandToExecute->m_sLastOutputBuffer = "";
//		CommandToExecute->m_sLastOutputBufferEncoded = "";
	}	
	delete ServiceCommand;

	if(!RetVal && !CommandToExecute->Error.m_iErrorCode)
	{
		// The service didn't return an error code so we set a standard one
		CommandToExecute->Error.SetError("SYSTEM", "SERVICE", NV_SERVICE_UNKNOWN_ERROR, CommandToExecute->m_sService);
	}

	GetMain()->Services.FreeService(ServiceNameHere.c_str());

	return RetVal;
}


// Process the system commands
// Called only from Command function
bool NvSession::ProcessSystemCommand(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	bool RetVal = true;

#ifndef AIX
	string Signature = CommandToExecute->m_sClass + ":" + CommandToExecute->m_sCommand;
	map<string,NVS_SYS_COMMAND>::iterator pos=CommandTable.find(Signature);
	if(pos == CommandTable.end())
	{
		RetVal = false;
		string sInfoMessage = "SYSTEM:";
		sInfoMessage += CommandToExecute->m_sClass;
		sInfoMessage+= ":";
		sInfoMessage+= CommandToExecute->m_sCommand;
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE, sInfoMessage);
	}
	else
	{
		NVS_SYS_COMMAND MyCommand = pos->second;
		RetVal = (*this.*MyCommand)(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
	}

	return RetVal;
#else
	bool BadCommand = false;
	// This code is for AIX because xlc doesn't compile with the function pointers
	// in a hash table (patch necessary ?)
	if(CommandToExecute->m_sClass == "SYSTEM")
	{
		if(CommandToExecute->m_sCommand == "STOP")
			RetVal = StopServer(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_PARAM")
			RetVal = SetSystemParam(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_PARAM")
			RetVal = GetSystemParam(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "DEBUG_STATE")
			RetVal = SystemDebugState(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "DEBUG_DUMP")
			RetVal = SystemDebugDump(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_MIME_TYPES")
			RetVal = SystemGetMimeTypes(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_MIME_TYPE")
			RetVal = SystemSetMimeType(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_MIME_TYPE")
			RetVal = SystemGetMimeType(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE_MIME_TYPE")
			RetVal = SystemRemoveMimeType(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_ALIASES")
			RetVal = SystemGetAliases(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_ALIAS")
			RetVal = SystemSetAlias(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_ALIAS")
			RetVal = SystemGetAlias(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE_ALIAS")
			RetVal = SystemRemoveAlias(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_VHOSTS")
			RetVal = SystemGetVhosts(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "ADD_VHOST")
			RetVal = SystemAddVhost(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE_VHOST")
			RetVal = SystemRemoveVhost(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_VERSION")
			RetVal = SystemGetVersion(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_VERSION_TYPE")
			RetVal = SystemGetVersionType(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_NUM_PROCS")
			RetVal = SystemGetNumProcs(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_PLATFORM")
			RetVal = SystemGetPlatform(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_ENCODING")
			RetVal = SystemGetEncoding(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "THREAD_LIST")
			RetVal = SystemThreadList(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_NUM_EXCEPTIONS")
			RetVal = SystemGetNumExceptions(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "DEBUG")
	{
		if(CommandToExecute->m_sCommand == "DISPLAY_MESSAGE")
			RetVal = DebugDisplayMessage(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "DISPLAY_OBJECT")
			RetVal = DebugDisplayObject(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "DISPLAY_CONTAINER")
			RetVal = DebugDisplayContainer(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "DISPLAY_VARIABLES")
			RetVal = DebugDisplayVariables(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "VARIABLE")
	{
		if(CommandToExecute->m_sCommand == "SET")
			RetVal = SetVariable(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET")
			RetVal = GetVariable(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_MULTI")
			RetVal = GetMultiVariable(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE")
			RetVal = RemoveVariable(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "EXIST")
			RetVal = VariableExist(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "ENCODE")
			RetVal = EncodeVariable(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "COMMAND")
	{
		if(CommandToExecute->m_sCommand == "SET_PARAMETER")
			RetVal = CommandSetParameter(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		if(CommandToExecute->m_sCommand == "REMOVE_PARAMETER")
			RetVal = CommandRemoveParameter(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_HTTP_HEADER")
			RetVal = CommandSetHttpHeader(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_HTTP_RET_CODE")
			RetVal = CommandSetHttpRetCode(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_HTTP_HEADERS")
			RetVal = CommandGetHttpHeaders(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_OUTPUT_BUFFER")
			RetVal = CommandSetOutputBuffer(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "LOCK")
	{
		if(CommandToExecute->m_sCommand == "LOCK")
			RetVal = LockLock(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "UNLOCK")
			RetVal = LockUnlock(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "ISLOCKED")
			RetVal = LockIsLocked(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INFO")
			RetVal = LockInfo(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "SEMAPHORE")
	{
		if(CommandToExecute->m_sCommand == "LOCK")
			RetVal = SemaLock(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "UNLOCK")
			RetVal = SemaUnlock(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE")
			RetVal = SemaRemove(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INFO")
			RetVal = SemaInfo(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "OBJECT")
	{
		if(CommandToExecute->m_sCommand == "SEND")
			RetVal = SendObject(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET")
			RetVal = GetObject(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CREATE")
			RetVal = CreateObject(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_NUM")
			GetNumObjects(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_NAME")
			RetVal = GetObjectName(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_TYPE")
			RetVal = GetObjectType(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CLEAR_ALL")
			RetVal = ClearAllObjects(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "COPY")
			RetVal = CopyObject(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "MOVE")
			RetVal = MoveObject(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_NAME")
			RetVal = SetObjectName(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE")
			RetVal = RemoveObject(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "EXIST")
			IsObjectExist(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "ENCODE")
			RetVal = EncodeObject(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "BOOLEAN_SET_VALUE")
			RetVal = ObjBooleanSetValue(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "BOOLEAN_GET_VALUE")
			RetVal = ObjBooleanGetValue(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INTEGER_SET_VALUE")
			RetVal = ObjIntegerSetValue(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INTEGER_GET_VALUE")
			RetVal = ObjIntegerGetValue(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "STRING_SET_VALUE")
			RetVal = ObjStringSetValue(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "STRING_GET_VALUE")
			RetVal = ObjStringGetValue(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "STRINGLIST_SET_VALUE")
			RetVal = ObjStringListSetValue(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "STRINGLIST_GET_VALUE")
			RetVal = ObjStringListGetValue(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "STRINGLIST_GET_SIZE")
			RetVal = ObjStringListGetSize(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "STRINGLIST_REMOVE")
			RetVal = ObjStringListRemove(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "STRINGLIST_INSERT")
			RetVal = ObjStringListInsert(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "STRINGLIST_ADD_STRINGLIST")
			RetVal = ObjStringListAddStringList(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "STRINGLIST_SWAP")
			RetVal = ObjStringListSwap(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "STRINGLIST_SORT")
			RetVal = ObjStringListSort(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "STRINGLIST_GET_VALUES")
			RetVal = ObjStringListGetValues(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "STRINGLIST_SEARCH")
			RetVal = ObjStringListSearch(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INDSTRINGLIST_SET_VALUE")
			RetVal = ObjIndStringListSetValue(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INDSTRINGLIST_GET_VALUE")
			RetVal = ObjIndStringListGetValue(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INDSTRINGLIST_GET_VALUES")
			RetVal = ObjIndStringListGetValues(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INDSTRINGLIST_GET_SIZE")
			RetVal = ObjIndStringListGetSize(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INDSTRINGLIST_REMOVE")
			RetVal = ObjIndStringListRemove(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INDSTRINGLIST_ADD_INDSTRINGLIST")
			RetVal = ObjIndStringListAddIndStringList(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INDSTRINGLIST_KEY_EXIST")
			RetVal = ObjIndStringListKeyExist(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INDSTRINGLIST_GET_KEY")
			RetVal = ObjIndStringListGetKey(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_SET_DESCRIPTION")
			RetVal = ObjTableSetDescription(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_GET_DESCRIPTION")
			RetVal = ObjTableGetDescription(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_CLEAR")
			RetVal = ObjTableClear(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_CLEAR_DATA")
			RetVal = ObjTableClearData(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_SEARCH")
			RetVal = ObjTableSearch(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_SORT")
			RetVal = ObjTableSort(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_CREATE_FROM")
			RetVal = ObjTableCreateFrom(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_SAVE")
			RetVal = ObjTableSave(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_LOAD")
			RetVal = ObjTableLoad(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_IMPORT")
			RetVal = ObjTableImport(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_EXPORT")
			RetVal = ObjTableExport(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_SELECT_FROM")
			RetVal = ObjTableSelectFrom(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_JOIN")
			RetVal = ObjTableJoin(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_ADD_TABLE")
			RetVal = ObjTableAddTable(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_CLEAR_CELL")
			RetVal = ObjTableClearCell(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_SORT_CELL")
			RetVal = ObjTableSortCell(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_GET_CELL_NUM_LINES")
			RetVal = ObjTableGetCellNumLines(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_GET_CELL_LINE")
			RetVal = ObjTableGetCellLine(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_SET_CELL_LINE")
			RetVal = ObjTableSetCellLine(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_GET_CELL")
			RetVal = ObjTableGetCell(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_SET_CELL")
			RetVal = ObjTableSetCell(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_INSERT_CELL_LINE")
			RetVal = ObjTableInsertCellLine(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_REMOVE_CELL_LINE")
			RetVal = ObjTableRemoveCellLine(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_SWAP_CELL_LINES")
			RetVal = ObjTableSwapCellLines(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_GET_NUM_ROWS")
			RetVal = ObjTableGetNumRows(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_REMOVE_ROW")
			RetVal = ObjTableRemoveRow(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_REMOVE_ROWS")
			RetVal = ObjTableRemoveRows(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_CLEAR_ROW")
			RetVal = ObjTableClearRow(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_CLEAR_ROWS")
			RetVal = ObjTableClearRows(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_INSERT_ROWS")
			RetVal = ObjTableInsertRows(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_SWAP_ROWS")
			RetVal = ObjTableSwapRows(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_GET_ROWS")
			RetVal = ObjTableGetRows(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_GET_ROW")
			RetVal = ObjTableGetRow(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_GET_COLUMN")
			RetVal = ObjTableGetColumn(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_SET_COLUMN")
			RetVal = ObjTableSetColumn(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_GET_NUM_COLUMNS")
			RetVal = ObjTableGetNumColums(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_GET_COLUMN_NAME")
			RetVal = ObjTableGetColumnName(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_GET_COLUMN_DESCRIPTION")
			RetVal = ObjTableGetColumnDescription(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_IS_COLUMN_NUMERIC")
			RetVal = ObjTableIsColumnNumeric(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_GET_COLUMN_INDEX")
			RetVal = ObjTableGetColumnIndex(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_CLEAR_COLUMN")
			RetVal = ObjTableClearColumn(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_REMOVE_COLUMN")
			RetVal = ObjTableRemoveColumn(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_REMOVE_COLUMNS")
			RetVal = ObjTableRemoveColumns(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_INSERT_COLUMN")
			RetVal = ObjTableInsertColumn(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_ADD_COLUMNS")
			RetVal = ObjTableAddColumns(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_MODIFY_COLUMN")
			RetVal = ObjTableModifyColumn(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_SWAP_COLUMNS")
			RetVal = ObjTableSwapColumns(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_FILTER_COLUMNS")
			RetVal = ObjTableFilterColumns(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_SET_PRIMARY_COLUMN")
			RetVal = ObjTableSetPrimaryColumn(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_SET_COLUMN_NAME")
			RetVal = ObjTableSetColumnName(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_SET_COLUMN_DESCRIPTION")
			RetVal = ObjTableSetColumnDescription(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_SET_COLUMN_TYPE")
			RetVal = ObjTableSetColumnType(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_GET_PRIMARY_INFO")
			RetVal = ObjTableGetPrimaryInfo(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_GET_ROW_INDEX")
			RetVal = ObjTableGetRowIndex(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_SET_ROW")
			RetVal = ObjTableSetRow(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TABLE_ADD_ROWS")
			RetVal = ObjTableAddRows(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "FILE_SET_FILENAME")
			RetVal = ObjFileSetFileName(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "FILE_SET_PERSIST")
			RetVal = ObjFileSetPersist(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "FILE_GET_PATHNAME")
			RetVal = ObjFileGetPathName(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "FILE_GET_DIRNAME")
			RetVal = ObjFileGetDirName(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "FILE_GET_FILENAME")
			RetVal = ObjFileGetFileName(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "FILE_GET_EXTENSION")
			RetVal = ObjFileGetExtension(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "FILE_GET_SIZE")
			RetVal = ObjFileGetSize(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "FILE_REMOVE")
			RetVal = ObjFileRemove(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "FILE_CREATE")
			RetVal = ObjFileCreate(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "FILE_CLEAR")
			RetVal = ObjFileClear(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "FILE_APPEND")
			RetVal = ObjFileAppend(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "FILE_EXIST")
			RetVal = ObjFileExist(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "FILE_SET_ORIGIN")
			RetVal = ObjFileSetOrigin(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "FILE_GET_ORIGIN")
			RetVal = ObjFileGetOrigin(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "FILE_SET_ASCII")
			RetVal = ObjFileSetAscii(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "FILE_COMRPESS")
			RetVal = ObjFileCompress(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "FILE_DECOMPRESS")
			RetVal = ObjFileDecompress(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "BINARY_GET_SIZE")
			RetVal = ObjBinaryGetSize(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "BINARY_CLEAR")
			RetVal = ObjBinaryClear(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "BINARY_APPEND")
			RetVal = ObjBinaryAppend(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "BINARY_SAVE")
			RetVal = ObjBinarySave(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "BINARY_LOAD")
			RetVal = ObjBinaryLoad(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "TEST")
	{
		if(CommandToExecute->m_sCommand == "TEST_BROWSER")
			RetVal = TestBrowser(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CRASH")
			RetVal = Crash(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_LIST")
			RetVal = TestSetList(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "LIST")
			RetVal = TestList(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "MISC")
	{
		if(CommandToExecute->m_sCommand == "NOP")
			RetVal = Nop(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SLEEP")
			RetVal = SleepSession(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "EXEC")
			RetVal = ExecFilter(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INFOEX")
			RetVal = InfoEx(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_CRASH")
			RetVal = GetCrash(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "HASH")
			RetVal = MiscHash(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "UNIQ")
			RetVal = MiscUniq(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SEQUENCE")
			RetVal = MiscSequence(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REST")
			RetVal = MiscRest(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "WEBSERVICE")
	{
		if(CommandToExecute->m_sCommand == "EXECUTE")
			RetVal = WebserviceExecute(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE")
			RetVal = WebserviceRemove(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "START")
			RetVal = WebserviceStart(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "STOP")
			RetVal = WebserviceStop(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INFO")
			RetVal = WebserviceInfo(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INFOEX")
			RetVal = WebserviceInfoEx(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CREATE")
			RetVal = WebserviceCreate(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "PACKAGE")
			RetVal = WebservicePackage(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INSTALL")
			RetVal = WebserviceInstall(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_EDIT_MODE")
			RetVal = WebserviceSetEditMode(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "EDIT_UPDATE")
			RetVal = WebserviceEditUpdate(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "IS_EDIT")
			RetVal = WebserviceIsEdit(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CAN_EDIT")
			RetVal = WebserviceCanEdit(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "OPERATION_LIST")
			RetVal = WebserviceOperationList(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "OPERATION_ENABLE")
			RetVal = WebserviceOperationEnable(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "OPERATION_CREATE")
			RetVal = WebserviceOperationCreate(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "OPERATION_REMOVE")
			RetVal = WebserviceOperationRemove(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "OPERATION_SET_PARAM")
			RetVal = WebserviceOperationSetParam(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "OPERATION_SOAP")
			RetVal = WebserviceOperationSoap(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "MESSAGE_LIST")
			RetVal = WebserviceMessageList(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "MESSAGE_CREATE")
			RetVal = WebserviceMessageCreate(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "MESSAGE_REMOVE")
			RetVal = WebserviceMessageRemove(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "MESSAGE_SET_PARAM")
			RetVal = WebserviceMessageSetParam(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "MESSAGE_SET_CONTENT")
			RetVal = WebserviceMessageSetContent(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "MESSAGE_GET_CONTENT")
			RetVal = WebserviceMessageGetContent(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_HEADERS")
			RetVal = WebserviceGetHeaders(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_HEADERS")
			RetVal = WebserviceSetHeaders(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "WSDL")
			RetVal = WebserviceWsdl(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "TIME")
	{
		if(CommandToExecute->m_sCommand == "GET")
			RetVal = GetTime(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "MAIL")
	{
		if(CommandToExecute->m_sCommand == "SEND")
			RetVal = SendMail(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "SESSION")
	{
		if(CommandToExecute->m_sCommand == "GET_SESSION_ID")
			RetVal = GetSessionId(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_USER")
			RetVal = GetSessionUser(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_PASSWORD")
			RetVal = GetSessionPassword(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_NUM")
			RetVal = SessionGetNum(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SERVICES")
			RetVal = SessionServices(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "LIST")
			RetVal = SessionList(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CLOSE")
			RetVal = Nop(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);	// Do nothing. It's here for eventually executing procedures
		else if(CommandToExecute->m_sCommand == "CLOSE_OTHER")
			RetVal = SessionCloseOther(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CREATE")
			RetVal = SessionCreate(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_NAMED")
			RetVal = SessionGetNamed(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "FREE_NAMED")
			RetVal = SessionFreeNamed(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CHECK_CLOSE_REQUEST")
			RetVal = SessionCheckCloseRequest(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_DEFAULT_ERROR_PROC")
			RetVal = SessionSetDefaultErrorProc(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_DEFAULT_ERROR_XSL")
			RetVal = SessionSetDefaultErrorXsl(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_LANGUAGE")
			RetVal = SessionSetLanguage(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "REQUEST")
	{
		if(CommandToExecute->m_sCommand == "OPEN")
			RetVal = OpenRequest(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CLOSE")
			RetVal = CloseRequest(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "SERVICE")
	{
		if(CommandToExecute->m_sCommand == "START")
			RetVal = ServiceStart(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "STOP")
			RetVal = ServiceStop(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SESSIONS")
			RetVal = ServiceSessions(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "MOUNT")
			RetVal = ServiceMount(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "UNMOUNT")
			RetVal = ServiceUnmount(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SKELETON")
			RetVal = ServiceSkeleton(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INFO")
			RetVal = ServiceInfo(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INFOEX")
			RetVal = ServiceInfoEx(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CONFIG")
			RetVal = ServiceConfig(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "IDENT")
			RetVal = ServiceIdent(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "PACKAGE")
			RetVal = ServicePackage(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INSTALL")
			RetVal = ServiceInstall(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "PROTECT_CONTAINERS")
			RetVal = ServiceProtectContainers(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "LICENSE")
	{
		if(CommandToExecute->m_sCommand == "GET")
			RetVal = LicenseGet(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET")
			RetVal = LicenseSet(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE")
			RetVal = LicenseRemove(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INSTALL")
			RetVal = LicenseInstall(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "EXPORT")
			RetVal = LicenseExport(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INFO")
			RetVal = LicenseInfo(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "ISDEMO")
			RetVal = LicenseIsDemo(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CHECK_EXPIRE_SOON")
			RetVal = LicenseCheckExpireSoon(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_TYPE")
			RetVal = LicenseGetType(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "TRANSACTION")
	{
		if(CommandToExecute->m_sCommand == "START")
			RetVal = TransactionStart(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "RESTART")
			RetVal = TransactionReStart(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "ROLLBACK")
			RetVal = TransactionRollback(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "VALIDATE")
			RetVal = TransactionValidate(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE")
			RetVal = TransactionRemove(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INFO")
			RetVal = TransactionInfo(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INFOEX")
			RetVal = TransactionInfoEx(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_STATUS")
			RetVal = TransactionSetStatus(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_OPTIONS")
			RetVal = TransactionSetOptions(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_CONTAINER")
			RetVal = TransactionGetContainer(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_VARIABLES")
			RetVal = TransactionGetVariables(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "SCHEDULER")
	{
		if(CommandToExecute->m_sCommand == "CREATE_TASK")
			RetVal = SchedulerCreateTask(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE_TASK")
			RetVal = SchedulerRemoveTask(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "ENABLE_TASK")
			RetVal = SchedulerEnableTask(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "RUN_TASK")
			RetVal = SchedulerRunTask(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_TASK_PARAM")
			RetVal = SchedulerSetTaskParam(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_TASK_PARAM")
			RetVal = SchedulerGetTaskParam(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TASK_LIST")
			RetVal = SchedulerTaskList(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "LISTENER")
	{
		if(CommandToExecute->m_sCommand == "CREATE")
			RetVal = ListenerCreate(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE")
			RetVal = ListenerRemove(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "START")
			RetVal = ListenerStart(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "STOP")
			RetVal = ListenerStop(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_PARAM")
			RetVal = ListenerSetParam(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "LIST")
			RetVal = ListenerList(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "THREAD")
	{
		if(CommandToExecute->m_sCommand == "CREATE")
			RetVal = ThreadCreate(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "PACKAGE")
	{
		if(CommandToExecute->m_sCommand == "PACKAGE")
			RetVal = PackagePackage(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INSTALL")
			RetVal = PackageInstall(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INFO")
			RetVal = PackageInfo(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "LOG")
	{
		if(CommandToExecute->m_sCommand == "CREATE")
			RetVal = LogCreate(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE")
			RetVal = LogRemove(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INFO")
			RetVal = LogInfo(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_LEVEL")
			RetVal = LogGetLevel(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "WRITE")
			RetVal = LogWrite(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_OPTIONS")
			RetVal = LogSetOptions(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SEARCH")
			RetVal = LogSearch(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "ERASE")
			RetVal = LogErase(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "SECURITY")
	{
		if(CommandToExecute->m_sCommand == "INFO")
			RetVal = SecurityInfo(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		if(CommandToExecute->m_sCommand == "SET_PARAM")
			RetVal = SecuritySetParam(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		if(CommandToExecute->m_sCommand == "CHECK")
			RetVal = SecurityCheck(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "ADD_USER")
			RetVal = SecurityUserAdd(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "ENABLE_USER")
			RetVal = SecurityUserEnable(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE_USER")
			RetVal = SecurityUserRemove(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "USER_LIST")
			RetVal = SecurityUserList(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "USER_EXIST")
			RetVal = SecurityUserExist(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_USER_DESCRIPTION")
			RetVal = SecurityUserSetDescription(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_USER_FULLNAME")
			RetVal = SecurityUserSetFullname(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_USER_PARAM")
			RetVal = SecurityUserSetParam(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_USER_PASSWORD")
			RetVal = SecurityUserSetPassword(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_OTHER_USER_PASSWORD")
			RetVal = SecurityUserSetOtherPassword(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_USER_ROLES")
			RetVal = SecurityUserSetRoles(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE_USER_ROLES")
			RetVal = SecurityUserRemoveRoles(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "USER_PERMISSIONS")
			RetVal = SecurityUserPermissions(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SESSION_PERMISSIONS")
			RetVal = SecuritySessionPermissions(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "ADD_ROLE")
			RetVal = SecurityRoleAdd(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE_ROLE")
			RetVal = SecurityRoleRemove(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "ROLE_LIST")
			RetVal = SecurityRoleList(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "ROLE_EXIST")
			RetVal = SecurityRoleExist(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_ROLE_DESCRIPTION")
			RetVal = SecurityRoleSetDescription(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_ROLE_INHERITS")
			RetVal = SecurityRoleSetInherits(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE_ROLE_INHERITS")
			RetVal = SecurityRoleRemoveInherits(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_ROLE_PERMISSIONS")
			RetVal = SecurityRoleSetPermissions(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE_ROLE_PERMISSIONS")
			RetVal = SecurityRoleRemovePermissions(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "PERMISSION_LIST")
			RetVal = SecurityPermissionList(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "ADD_SESSION_PERMISSION")
			RetVal = SecurityAddSessionPermission(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE_SESSION_PERMISSION")
			RetVal = SecurityRemoveSessionPermission(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "LOAD_USER_CONTEXT")
			RetVal = SecurityLoadUserContext(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SAVE_USER_CONTEXT")
			RetVal = SecuritySaveUserContext(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "APPLICATION")
	{
		if(CommandToExecute->m_sCommand == "GET_NAME")
			RetVal = ApplicationGetName(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_DIR")
			RetVal = ApplicationGetDir(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_WORK_DIR")
			RetVal = ApplicationGetWorkDir(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_FILE_DIR")
			RetVal = ApplicationGetFileDir(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_WROOT_DIR")
			RetVal = ApplicationGetWrootDir(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE_DIR")
			RetVal = ApplicationRemoveDir(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CREATE_DIR")
			RetVal = ApplicationCreateDir(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "DIR")
			RetVal = ApplicationDir(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE")
			RetVal = ApplicationRemove(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "START")
			RetVal = ApplicationStart(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "STOP")
			RetVal = ApplicationStop(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SESSIONS")
			RetVal = ApplicationSessions(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INFO")
			RetVal = ApplicationInfo(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INFOEX")
			RetVal = ApplicationInfoEx(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CREATE")
			RetVal = ApplicationCreate(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CONFIG")
			RetVal = ApplicationConfig(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "PACKAGE")
			RetVal = ApplicationPackage(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "INSTALL")
			RetVal = ApplicationInstall(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REDIRECTION_ADD")
			RetVal = ApplicationRedirectionAdd(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REDIRECTION_REMOVE")
			RetVal = ApplicationRedirectionRemove(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REDIRECTION_ENABLE")
			RetVal = ApplicationRedirectionEnable(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REDIRECTION_INFO")
			RetVal = ApplicationRedirectionInfo(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "XML")
	{
		if(CommandToExecute->m_sCommand == "SET_XML")
			RetVal = XmlSetXml(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_XML")
			RetVal = XmlGetXml(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_FILE")		// For compatibility reason
			RetVal = XmlSetXml(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "TRANSFORM")
			RetVal = XmlTransform(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "LOAD_XSL")
			RetVal = XmlLoadXsl(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_XSL")
			RetVal = XmlSetXsl(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SEND")
			RetVal = XmlSend(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "VALIDATE")
			RetVal = XmlValidate(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CLEAR_CACHE")
			RetVal = XmlClearCache(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "CONTAINER")
	{
		if(CommandToExecute->m_sCommand == "REMOVE")
			RetVal = RemoveContainer(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "COPY")
			RetVal = CopyContainer(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "MOVE")
			RetVal = MoveContainer(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "LIST")
			RetVal = ListContainers(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_NUM_SUBCONTAINERS")
			RetVal = GetNumSubContainers(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_SUBCONTAINER_NAME")
			RetVal = GetSubContainerName(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET_PARENT_NAME")
			RetVal = GetContainerParentName(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "EXPORT")
			RetVal = ExportContainer(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "IMPORT")
			RetVal = ImportContainer(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "VIEW")
			RetVal = ViewContainer(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "ENCODE")
			RetVal = EncodeContainer(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "REGISTRY")
	{
		if(CommandToExecute->m_sCommand == "REMOVE")
			RetVal = RemoveRegistryKey(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "GET")
			RetVal = GetRegistryKey(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET")
			RetVal = SetRegistryKey(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CREATE")
			RetVal = CreateRegistryKey(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "EXIST")
			RetVal = RegistryKeyExist(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "EXPORT")
			RetVal = ExportRegistryKey(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "IMPORT")
			RetVal = ImportRegistryKey(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CLEAR_CACHE")
			RetVal = ClearRegistryCache(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "OPEN_REGISTRY")
			RetVal = OpenRegistry(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CLOSE_REGISTRY")
			RetVal = CloseRegistry(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "MQ")
	{
		if(CommandToExecute->m_sCommand == "ISAVAILABLE")
			RetVal = MqIsAvailable(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CREATE")
			RetVal = MqCreate(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SET_PARAM")
			RetVal = MqSetParam(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "REMOVE")
			RetVal = MqRemove(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "START")
			RetVal = MqStart(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "STOP")
			RetVal = MqStop(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "LIST")
			RetVal = MqList(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "EXIST")
			RetVal = MqExist(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else if(CommandToExecute->m_sClass == "FILEDB")
	{
		if(CommandToExecute->m_sCommand == "OPEN")
			RetVal = FileDbOpen(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "CLOSE")
			RetVal = FileDbClose(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else if(CommandToExecute->m_sCommand == "SQL")
			RetVal = FileDbSql(CommandToExecute, Buffer, BufferSize, DataSize, iEncoding);
		else
			BadCommand = true;
	}
	else
		BadCommand = true;

	if(BadCommand)
	{
		RetVal = false;
		string sInfoMessage = "SYSTEM:";
		sInfoMessage += CommandToExecute->m_sClass;
		sInfoMessage+= ":";
		sInfoMessage+= CommandToExecute->m_sCommand;
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE, sInfoMessage);
	}

	return RetVal;
#endif
}



// Internal function
void NvSession::GetStringValue(NvCommand *CommandToExecute, const char *String, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	CommandToExecute->m_sLastOutputBuffer = "";
	CommandToExecute->m_sLastOutputBufferEncoded = "";

	// All the local functions that return a value goes here
	if(String == NULL)
	{
		if(DataSize != NULL)
			*DataSize = 0;
		if(Buffer == NULL)
			return;
		if(BufferSize == 0)
		{
			// Buffer contains a variable name (is it really used ?)
			m_oVariables.SetValue(Buffer, "");
			return;
		}
		Buffer[0] = 0;
		return;
	}

	// The return value is always a string
	// Even if the result string has to be returned into a buffer, we also
	// always set it into a request variable. This variable is NV_RESULT by default
	// but can be changed by the value of the parameter NV_VAR
	string VarName = CommandToExecute->Parameters.GetValue("NV_VAR");
	if(!VarName.length())
		VarName = "NV_RESULT";
	m_oVariables.SetValue(VarName, String);

	if((CommandToExecute->m_iCommandSource != NV_SOURCE_CLIENT) && (CommandToExecute->m_iCommandSource != NV_SOURCE_SERVICE) && (Buffer == NULL) && (DataSize == NULL))
		return;	// Nothing to do in this case

	if((Buffer != NULL) && (BufferSize == 0))
	{
		// Buffer contains a variable name (is it really used ?)
		string Variable = Buffer;
		m_oVariables.SetValue(Buffer, String);
		return;
	}

	string RetVal = String;
	// Keep a copy of last output buffer
	CommandToExecute->m_sLastOutputBuffer = RetVal;
	EncodeCommandResult(iEncoding, RetVal);

	int iDataSize = (int)(RetVal.length());
	if(DataSize != NULL)
		*DataSize = iDataSize;

	// Keep a copy of last output buffer
	CommandToExecute->m_sLastOutputBufferEncoded = RetVal;

	if(CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT)
	{
		// We write the output buffer into command data for it to be returned to client
		// For now we use only memory message for that
		// We have to set the encoding comparing to the client encoding
		
		CommandToExecute->FreeData();
		if(iDataSize)
		{
			string RetValForClient = String;
			bool bMustConvert = false;
			if(CommandToExecute->bClientUnicode != GetMain()->IsUnicode())
				bMustConvert = true;
			if(bMustConvert)
			{
				int iEncodingHere = ENCODING_LATIN1;
				if(CommandToExecute->bClientUnicode)
					iEncodingHere = ENCODING_UTF8;
				EncodeCommandResult(iEncodingHere, RetValForClient);
			}
			unsigned long iDataSizeForClient = (int)(RetValForClient.length());

			CommandToExecute->Data = (char *)malloc(iDataSizeForClient+1);
			if(CommandToExecute->Data != NULL)
			{
				CommandToExecute->DataSize = iDataSizeForClient+1;
				memcpy(CommandToExecute->Data, RetValForClient.c_str(), iDataSizeForClient);
				CommandToExecute->Data[iDataSizeForClient] = 0;	// end of string added directly
			}
		}
	}

	if(Buffer == NULL)
		return;


	// Empty buffer by default;
	Buffer[0] = 0;


	if((int)(RetVal.length() + 1) > BufferSize)
		RetVal.erase(BufferSize-1);

	if(RetVal.length())
		strcpy(Buffer, RetVal.c_str());
}



// Internal function
void NvSession::GetLastOutputBuffer(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// All the local functions that return a value goes here
	if(DataSize != NULL)
		*DataSize = 0;
	if(CommandToExecute->m_sLastOutputBuffer.length() == 0)
	{
		if(Buffer == NULL)
			return;
		if(BufferSize == 0)
		{
			// Buffer contains a variable name (is it really used ?)
			m_oVariables.SetValue(Buffer, "");
			return;
		}
		Buffer[0] = 0;
		return;
	}

	// The return value is always a string
	// Even if the result string has to be returned into a buffer, we also
	// always set it into a request variable. This variable is NV_RESULT by default
	// but can be changed by the value of the parameter NV_VAR
	string VarName = CommandToExecute->Parameters.GetValue("NV_VAR");
	if(!VarName.length())
		VarName = "NV_RESULT";
	m_oVariables.SetValue(VarName, CommandToExecute->m_sLastOutputBuffer);

	if(DataSize != NULL)
		*DataSize = (int)(CommandToExecute->m_sLastOutputBuffer.length());
	if((CommandToExecute->m_iCommandSource != NV_SOURCE_CLIENT) && (Buffer == NULL))
		return;	// Nothing to do in this case

	if((Buffer != NULL) && (BufferSize == 0))
	{
		// Buffer contains a variable name (is it really used ?)
		string Variable = Buffer;
		m_oVariables.SetValue(Buffer, CommandToExecute->m_sLastOutputBuffer.c_str());
		return;
	}

	string RetVal = CommandToExecute->m_sLastOutputBuffer;
	EncodeCommandResult(iEncoding, RetVal);
	if(DataSize != NULL)
		*DataSize = (int)(RetVal.length());

	if(CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT)
	{
		// We write the output buffer into command data for it to be returned to client
		// For now we use only memory message for that
		// We have to set the encoding comparing to the client encoding
		
		CommandToExecute->FreeData();
		if(CommandToExecute->m_sLastOutputBuffer.length())
		{
			string RetValForClient = CommandToExecute->m_sLastOutputBuffer;
			bool bMustConvert = false;
			if(CommandToExecute->bClientUnicode != GetMain()->IsUnicode())
				bMustConvert = true;
			if(bMustConvert)
			{
				int iEncodingHere = ENCODING_LATIN1;
				if(CommandToExecute->bClientUnicode)
					iEncodingHere = ENCODING_UTF8;
				EncodeCommandResult(iEncodingHere, RetValForClient);
			}
			unsigned long iDataSizeForClient = (int)(RetValForClient.length());

			CommandToExecute->Data = (char *)malloc(iDataSizeForClient+1);
			if(CommandToExecute->Data != NULL)
			{
				CommandToExecute->DataSize = iDataSizeForClient+1;
				memcpy(CommandToExecute->Data, RetValForClient.c_str(), iDataSizeForClient);
				CommandToExecute->Data[iDataSizeForClient] = 0;	// end of string added directly
			}
		}
	}

	if(Buffer == NULL)
		return;


	// Empty buffer by default;
	Buffer[0] = 0;


	if((int)(RetVal.length() + 1) > BufferSize)
		RetVal.erase(BufferSize-1);

	if(RetVal.length())
		strcpy(Buffer, RetVal.c_str());
}



// Send object
// The object is sent by the client and goes to a session container
bool NvSession::SendObject(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// This command is only available from client
	if(CommandToExecute->m_iCommandSource != NV_SOURCE_CLIENT)
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:OBJECT:SEND");
		return false;
	}

	// Get object name and type that should be in parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	string ObjType = CommandToExecute->Parameters.GetValue("TYPE");
	CleanString(ObjName);
	CleanString(ObjType);
	if(!ObjName.length())
		ObjName = NV_OBJ_DEFAULT_NAME;

	if(!ObjType.length())
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_NO_TYPE);
		return false;
	}

	int iObjectType = atoi(ObjType.c_str());
	if((iObjectType < 0) || (iObjectType > NV_OBJTYPE_MAXTYPEVALUE))
		iObjectType = NV_OBJTYPE_UNKNOWN;

	if(iObjectType == NV_OBJTYPE_UNKNOWN)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_UNKNOWN_TYPE, ObjName);
		return false;
	}

	// The function uses only the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;


	// Now, the container has been found

	// We first remove the object if it ever exist in the container
	// In fact, the new object should replace the existing one
	InContainer->RemoveObject(ObjName.c_str());

	if(!InContainer->CreateObject(ObjName.c_str(), iObjectType))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, ObjName);
		return false;
	}

	if(iObjectType == NV_OBJTYPE_FILE)
	{
		// Check the persist parameter
		string PersistTimeFileInCommand = CommandToExecute->Parameters.GetValue("PERSIST");
		CleanString(PersistTimeFileInCommand);
		MakeUpper(PersistTimeFileInCommand);
		int iPersistTimeFileInCommand = 0;	// The file is not persistant by default on the server
		if(PersistTimeFileInCommand.length())
			iPersistTimeFileInCommand = atoi(PersistTimeFileInCommand.c_str());
		if(iPersistTimeFileInCommand < 0)
			iPersistTimeFileInCommand = -1;


		string FileExtensionInCommand = CommandToExecute->Parameters.GetValue("EXTENSION");
		CleanString(FileExtensionInCommand);
		if(!FileExtensionInCommand.length())
		{
			if(iPersistTimeFileInCommand == -1)
				FileExtensionInCommand = "obj";
			else
				FileExtensionInCommand = "tmp";
		}
		string FilePrefixInCommand = CommandToExecute->Parameters.GetValue("PREFIX");
		CleanString(FilePrefixInCommand);
		if(!FilePrefixInCommand.length())
			FilePrefixInCommand = Application->GetAppNameForFiles();
		string FileSuffixInCommand = CommandToExecute->Parameters.GetValue("SUFFIX");
		CleanString(FileSuffixInCommand);
		string TargetDir = Application->GetWorkDir();
		if(iPersistTimeFileInCommand == -1)
		{
			// If the file is persistant, it's created in the application File directory
			TargetDir = Application->GetFileDir();
		}

		NvObjFile *Object = (NvObjFile *)(InContainer->GetObject(ObjName.c_str()));
		if(Object != NULL)
		{

			if(!Object->SetAutoFileName(TargetDir, FileExtensionInCommand, FilePrefixInCommand, FileSuffixInCommand))
			{
				char *ErrorMessage = (char *)malloc(ObjName.length() + TargetDir.length() + FileExtensionInCommand.length()  + FilePrefixInCommand.length()  + FileSuffixInCommand.length() + 100);
				sprintf(ErrorMessage, "obj:%s - dir:%s - ext:%s - prefix:%s - suffix:%s", ObjName.c_str(), TargetDir.c_str(), FileExtensionInCommand.c_str(), FilePrefixInCommand.c_str(), FileSuffixInCommand.c_str());
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_FILE_CANNOT_CREATE, ErrorMessage);
				free(ErrorMessage);
				InContainer->RemoveObject(ObjName.c_str());
				return false;
			}
			Object->SetPersistTime(iPersistTimeFileInCommand);
		}
	}

	if(iObjectType == NV_OBJTYPE_INDSTRINGLIST)
	{
		NvObjIndStringList *Object = (NvObjIndStringList *)(InContainer->GetObject(ObjName.c_str()));
		if(Object != NULL)
		{
			string CaseSensitive = CommandToExecute->Parameters.GetValue("CASE_SENSITIVE");
			CleanString(CaseSensitive);
			MakeUpper(CaseSensitive);
			if(CaseSensitive == "YES")
				Object->m_bCaseSensitive = true;
		}
	}

	// PM TODO
	// We can call a Decode function here that operates on CommandToExecute->Data
	// This Decode function can also uncompress the data stream

	// We populate the object
	bool bPopulateFromData = true;
	if(iObjectType == NV_OBJTYPE_FILE)
	{
		if(CommandToExecute->DataFile.length())
		{
			bPopulateFromData = false;
			// We populate the file object directly from the data file in command
			NvObjFile *Object = (NvObjFile *)(InContainer->GetObject(ObjName.c_str()));
			if(Object != NULL)
			{
				string ObjectFileName = Object->GetFilePathName();
				if(!CopyFileEx(CommandToExecute->DataFile.c_str(), ObjectFileName.c_str()))
				{
					CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_POPULATE, ObjName);
					InContainer->RemoveObject(ObjName.c_str());
					return false;
				}
			}
		}
	}
	else
	{
		if(CommandToExecute->DataFile.length())
			bPopulateFromData = false;
	}

	// We have to eventually convert encoding
	bool bMustConvert = false;
	if(CommandToExecute->bClientUnicode != GetMain()->IsUnicode())
		bMustConvert = true;
	if(iObjectType == NV_OBJTYPE_FILE)
		bMustConvert = false;
	if(iObjectType == NV_OBJTYPE_BINARY)
		bMustConvert = false;
	string sNoConversion = CommandToExecute->Parameters.GetValue("NV_NO_CONVERSION");
	CleanString(sNoConversion);
	MakeUpper(sNoConversion);
	if(sNoConversion == "YES")
		bMustConvert = false;

	if(bMustConvert)
	{
		// We must convert the object encoding
		// This consist of loading object data that is in fact a table
		// and to convert it
		Intdbtable ProviTable;
		if(bPopulateFromData)
		{
			if(ProviTable.LoadFromBuffer(CommandToExecute->Data, CommandToExecute->DataSize, ENCODING_MODE_DECODE, CommandToExecute->bClientUnicode))
			{
				CommandToExecute->FreeData();
				CommandToExecute->Data = ProviTable.SaveToBuffer(&(CommandToExecute->DataSize));
			}
		}
		else
		{
			if(ProviTable.Load(CommandToExecute->DataFile.c_str(), ENCODING_MODE_DECODE, CommandToExecute->bClientUnicode))
				ProviTable.Save(CommandToExecute->DataFile.c_str());
		}
	}

	if(bPopulateFromData)
	{
		if(!InContainer->PopulateObject(ObjName.c_str(), CommandToExecute->Data, CommandToExecute->DataSize))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_POPULATE, ObjName);
			InContainer->RemoveObject(ObjName.c_str());
			return false;
		}
	}
	else if(iObjectType != NV_OBJTYPE_FILE)
	{
		// The work has ever been done for a file object
		if(!InContainer->PopulateObjectFromFile(ObjName.c_str(), CommandToExecute->DataFile.c_str(), false))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_POPULATE, ObjName);
			InContainer->RemoveObject(ObjName.c_str());
			return false;
		}
	}

	if(iObjectType == NV_OBJTYPE_FILE)
	{
		NvObjFile *Object = (NvObjFile *)(InContainer->GetObject(ObjName.c_str()));
		// Check the ASCII parameter
		string sAscii = CommandToExecute->Parameters.GetValue("ASCII");
		CleanString(sAscii);
		MakeUpper(sAscii);
		if(sAscii == "YES")
		{
			if(Object != NULL)
				Object->SetAscii();
		}
		// Get file origin
		string sFileOrigin = CommandToExecute->Parameters.GetValue("NV_FILE_ORIGIN");
		if(Object != NULL)
			Object->SetOrigin(sFileOrigin);
		CommandToExecute->Parameters.Remove("NV_FILE_ORIGIN");
	}


	// The command data is not needed any more
	CommandToExecute->FreeData();

	return true;
}

	

// Get object
// The object comes from the session container and goes the client
bool NvSession::GetObject(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if((CommandToExecute->m_iCommandSource == NV_SOURCE_PROCEDURE) || (CommandToExecute->m_iCommandSource == NV_SOURCE_SERVICE) || (CommandToExecute->m_iCommandSource == NV_SOURCE_SERVER))
	{
		CommandToExecute->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE, "SYSTEM:OBJECT:GET");
		return false;
	}

	// This command must block the standard xml output
	CommandToExecute->iOutputType = OUTPUT_BINARY;

	// Get object name and type that should be in parameters
	string ObjName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ObjName);
	if(!ObjName.length())
		ObjName = NV_OBJ_DEFAULT_NAME;

	CommandToExecute->FreeData();


	// The function uses only the output container
	if(!CheckOutputContainer(CommandToExecute))
		return false;

	// Now, the container has been found

	NvObj* Object = OutContainer->GetObject(ObjName.c_str());
	if(Object == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET, ObjName);
		return false;
	}

	int iObjectType = Object->GetType();

	// Get the remove option
	bool bRemoveObject = false;
	string sRemoveObject = CommandToExecute->Parameters.GetValue("REMOVE");
	CleanString(sRemoveObject);
	MakeUpper(sRemoveObject);
	if(sRemoveObject == "YES")
		bRemoveObject = true;

	// For the browser, we just return file objects
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER)
	{
		if((iObjectType != NV_OBJTYPE_FILE) && (iObjectType != NV_OBJTYPE_BINARY))
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET_TYPE, ObjName);
			return false;
		}
		if(iObjectType == NV_OBJTYPE_FILE)
			CommandToExecute->iOutputType = OUTPUT_BINARY;
	}

	if(iObjectType == NV_OBJTYPE_FILE)
	{
		NvInt64 FileSize = ((NvObjFile *)Object)->GetSize();
		bool bFileMessage = false;
		if(FileSize > (NvInt64)(GetMain()->GetHttpMaxMemBody()))	// for files grater than 1024 kbytes
			bFileMessage = true;
		
		if(bFileMessage)
		{
			string ObjectFileName = ((NvObjFile *)Object)->GetFilePathName();

			// We must work with a copy of the object because the session can be closed
			// before sending the file object to the client
			string DirHere = GetMain()->GetNirvaDir();
			DirHere += "Work";
			string FileNameHere;
			if(!GetTemporaryFile(FileNameHere, DirHere, "tmp", "", ""))
				return false;
			if(!CopyFileEx(ObjectFileName.c_str(), FileNameHere.c_str()))
			{
				RemoveFile(FileNameHere.c_str());
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET_DATA, ObjName);
				return false;
			}
			CommandToExecute->DataFile = FileNameHere;
			CommandToExecute->bDeleteDataFile = true;
		}
		else
		{
			// Not a too big file so we use memory data
			if(!((NvObjFile *)Object)->GetData(&(CommandToExecute->Data), &(CommandToExecute->DataSize)))
			{
				CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET_DATA, ObjName);
				return false;
			}
		}
	}
	else
	{
		// This is not a file object, so we take the object data into memory
		bool RetVal = OutContainer->GetObjectData(ObjName.c_str(), &(CommandToExecute->Data), &(CommandToExecute->DataSize));
		if(RetVal == false)
		{
			CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET_DATA, ObjName);
			return false;
		}
	}

	
	// For the browser, we just return file and binary objects
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER)
	{

		bool bWithCache = false;
		string sNoCache = CommandToExecute->Parameters.GetValue("NOCACHE");
		CleanString(sNoCache);
		MakeUpper(sNoCache);
		if(sNoCache == "NO")
			bWithCache = true;
		if(!bWithCache)
		{
			CommandToExecute->HttpOutHeaders.SetValue("Cache-Control", "no-cache");
			CommandToExecute->HttpOutHeaders.SetValue("Pragma", "no-cache");
			CommandToExecute->HttpOutHeaders.SetValue("Expires", "Wed, 26 Feb 1997 08:21:57 GMT"); // old date
		}
		bool bAsAttachment = false;
		string sAsAttachment = CommandToExecute->Parameters.GetValue("ATTACHMENT");
		CleanString(sAsAttachment);
		MakeUpper(sAsAttachment);
		if(sAsAttachment == "YES")
			bAsAttachment = true;
		if(bAsAttachment)
		{
			string HeaderValue = "attachment";
			string sFileName = CommandToExecute->Parameters.GetValue("FILENAME");
			if(!sFileName.length())
				sFileName = ((NvObjFile *)Object)->GetFileName();
			HeaderValue += "; filename=\"";
			HeaderValue += sFileName;
			HeaderValue += "\"";
			CommandToExecute->HttpOutHeaders.SetValue("Content-Disposition" , HeaderValue);
		}
		
		bool bAsInLine = false;
		string sAsInLine = CommandToExecute->Parameters.GetValue("INLINE");
		CleanString(sAsInLine);
		MakeUpper(sAsInLine);
		if(sAsInLine == "YES")
			bAsInLine = true;
		if(bAsInLine)
		{
			string HeaderValue = "inline";
			string sFileName = CommandToExecute->Parameters.GetValue("FILENAME");
			if(!sFileName.length())
				sFileName = ((NvObjFile *)Object)->GetFileName();
			HeaderValue += "; filename=\"";
			HeaderValue += sFileName;
			HeaderValue += "\"";
			CommandToExecute->HttpOutHeaders.SetValue("Content-Disposition" , HeaderValue);
		}

		string sExtensionInFile = ((NvObjFile *)Object)->GetFileExtension();
		CleanString(sExtensionInFile);
		MakeLower(sExtensionInFile);
		string sMimeType = CommandToExecute->Parameters.GetValue("MIME");
		CleanString(sMimeType);
		if(!sMimeType.length())
			sMimeType = GetMain()->GetMimeType(sExtensionInFile);
		CleanString(sMimeType);
		if(sMimeType.length())
			CommandToExecute->HttpOutHeaders.SetValue("Content-Type", sMimeType);


		if(bRemoveObject)
			OutContainer->RemoveObject(ObjName.c_str());

		return true;
	}

	// The object name and type should be returned in parameters
	CommandToExecute->Parameters.SetValue("NAME", Object->GetName());
	char Buff[50];
	sprintf(Buff, "%d", iObjectType);
	CommandToExecute->Parameters.SetValue("NV_SPECIAL_OBJECT_TYPE", Buff);
	

	// We have to eventually convert encoding
	bool bMustConvert = false;
	if(CommandToExecute->bClientUnicode != GetMain()->IsUnicode())
		bMustConvert = true;
	if(iObjectType == NV_OBJTYPE_FILE)
		bMustConvert = false;
	if(iObjectType == NV_OBJTYPE_BINARY)
		bMustConvert = false;
	if(CommandToExecute->m_iCommandSource != NV_SOURCE_CLIENT)
		bMustConvert = false;
	string sNoConversion = CommandToExecute->Parameters.GetValue("NV_NO_CONVERSION");
	CleanString(sNoConversion);
	MakeUpper(sNoConversion);
	if(sNoConversion == "YES")
		bMustConvert = false;
	if(bMustConvert)
	{
		// We must convert the object encoding
		// This consist of loading object data that is in fact a table
		// and to convert it
		Intdbtable ProviTable;
		if(ProviTable.LoadFromBuffer(CommandToExecute->Data, CommandToExecute->DataSize, ENCODING_MODE_ENCODE, CommandToExecute->bClientUnicode))
		{
			CommandToExecute->FreeData();
			CommandToExecute->Data = ProviTable.SaveToBuffer(&(CommandToExecute->DataSize));
		}
	}

	if(bRemoveObject)
		OutContainer->RemoveObject(ObjName.c_str());

	// PM TODO
	// We can call an Encode function here that operates on CommandToExecute->Data
	// This Encode function can also compress the data stream
	
	return true;
}

// Check if input container exists
// and sets the command error if not
bool NvSession::CheckInputContainer(NvCommand *CommandToExecute)
{
	if(InContainer == NULL)
	{
		// The container must exist
		string sContainerName = CommandToExecute->m_sInContainer;
		sContainerName = Containers.CleanContainerName(sContainerName);
		if(!sContainerName.length())
			sContainerName = NV_CONTAINER_DEFAULT_NAME;
		CommandToExecute->Error.SetError("SYSTEM", "CONTAINER", NV_CONTAINER_CANNOT_GET, sContainerName);
		return false;
	}

	return true;
}

// Check if output container exists
// and sets the command error if not
bool NvSession::CheckOutputContainer(NvCommand *CommandToExecute)
{
	if(OutContainer == NULL)
	{
		// The container must exist
		string sContainerName = CommandToExecute->m_sOutContainer;
		sContainerName = Containers.CleanContainerName(sContainerName);
		if(!sContainerName.length())
			sContainerName = NV_CONTAINER_DEFAULT_NAME;
		CommandToExecute->Error.SetError("SYSTEM", "CONTAINER", NV_CONTAINER_CANNOT_GET, sContainerName);
		return false;
	}

	return true;
}
	

	
// SYSTEM SYSTEM STOP command (undocumented)
bool NvSession::StopServer(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	string MachineNameInCommand = CommandToExecute->m_sMachineName;
	MakeUpper(MachineNameInCommand);
	string MachineNameHere = GetMain()->m_sMachineName;
	MakeUpper(MachineNameHere);
	if(MachineNameInCommand != MachineNameHere)
		if(!CheckPermission(CommandToExecute, "SYSTEM", "STOP_SERVER"))
			return false;

#ifdef WIN32
	// Cannot use that in window service
	if(!(GetMain()->b_ConsoleMode))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SYSTEM", NV_SYSTEM_CANNOT_STOP_SERVER);
		return false;
	}
#endif
	GetMain()->Stop();
	return true;
}

	
// SYSTEM SYSTEM SET_PARAM command (undocumented)
bool NvSession::SetSystemParam(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "SET_SYSTEM_PARAM"))
		return false;

	string Value;

	
	if(CommandToExecute->Parameters.KeyExist("HTTP_PORT"))
	{
		Value = CommandToExecute->Parameters.GetValue("HTTP_PORT");
		CleanString(Value);
		int iValue = atoi(Value.c_str());
		if(iValue < 1)
			iValue = 1;
		GetMain()->SetHttpPort(iValue);
	}

	
	if(CommandToExecute->Parameters.KeyExist("HTTPS_PORT"))
	{
		Value = CommandToExecute->Parameters.GetValue("HTTPS_PORT");
		CleanString(Value);
		int iValue = atoi(Value.c_str());
		if(iValue < 1)
			iValue = 1;
		GetMain()->SetHttpsPort(iValue);
	}

	if(CommandToExecute->Parameters.KeyExist("HTTP_TIME_OUT"))
	{
		Value = CommandToExecute->Parameters.GetValue("HTTP_TIME_OUT");
		CleanString(Value);
		int iValue = atoi(Value.c_str());
		if(iValue < 10)
			iValue = 10;
		GetMain()->SetHttpSessionTimeOut(iValue);
	}

	if(CommandToExecute->Parameters.KeyExist("HTTP_WAIT_STOP"))
	{
		Value = CommandToExecute->Parameters.GetValue("HTTP_WAIT_STOP");
		CleanString(Value);
		int iValue = atoi(Value.c_str());
		if(iValue < 5)
			iValue = 5;
		GetMain()->SetHttpStopWaitTime(iValue);
	}

	if(CommandToExecute->Parameters.KeyExist("HTTP_MAX_MEM_BODY"))
	{
		Value = CommandToExecute->Parameters.GetValue("HTTP_MAX_MEM_BODY");
		CleanString(Value);
		int iValue = atoi(Value.c_str());
		if(iValue < 10)
			iValue = 10;
		GetMain()->SetHttpMaxMemBody(iValue * 1024);
	}

	if(CommandToExecute->Parameters.KeyExist("SSL"))
	{
		Value = CommandToExecute->Parameters.GetValue("SSL");
		CleanString(Value);
		MakeUpper(Value);
		if(Value == "YES")
			GetMain()->SetSSL(true);
		else
			GetMain()->SetSSL(false);
	}

	if(CommandToExecute->Parameters.KeyExist("SSL_MIN_PROTOCOL"))
	{
		Value = CommandToExecute->Parameters.GetValue("SSL_MIN_PROTOCOL");
		if (Value.length())
		{
			CleanString(Value);
			MakeUpper(Value);
			if (Value == "TLS10")
				GetMain()->SetSSLMinProto(SSL_MINI_PROTO_TLS10);
			else if (Value == "TLS11")
				GetMain()->SetSSLMinProto(SSL_MINI_PROTO_TLS11);
			else if (Value == "TLS12")
				GetMain()->SetSSLMinProto(SSL_MINI_PROTO_TLS12);
			else
				GetMain()->SetSSLMinProto(SSL_MINI_PROTO_NONE);
		}
	}

	if(CommandToExecute->Parameters.KeyExist("HTTP_LOCAL"))
	{
		Value = CommandToExecute->Parameters.GetValue("HTTP_LOCAL");
		CleanString(Value);
		MakeUpper(Value);
		if(Value == "YES")
			GetMain()->SetHttpLocal(true);
		else
			GetMain()->SetHttpLocal(false);
	}

	if(CommandToExecute->Parameters.KeyExist("HTTP_IPV6"))
	{
		Value = CommandToExecute->Parameters.GetValue("HTTP_IPV6");
		CleanString(Value);
		MakeUpper(Value);
		if(Value == "YES")
			GetMain()->SetHttpIpV6(true);
		else
			GetMain()->SetHttpIpV6(false);
	}

	if(CommandToExecute->Parameters.KeyExist("HTTP_BLOCK_SERVER_HEADER"))
	{
		Value = CommandToExecute->Parameters.GetValue("HTTP_BLOCK_SERVER_HEADER");
		CleanString(Value);
		MakeUpper(Value);
		if(Value == "YES")
			GetMain()->SetHttpBlockServerHeader(true);
		else
			GetMain()->SetHttpBlockServerHeader(false);
	}

	if(CommandToExecute->Parameters.KeyExist("HTTPS_USE_CLIENT_CERTIFICATE"))
	{
		Value = CommandToExecute->Parameters.GetValue("HTTPS_USE_CLIENT_CERTIFICATE");
		CleanString(Value);
		MakeUpper(Value);
		if(Value == "YES")
			GetMain()->SetHttpsUseClientCertificate(true);
		else
			GetMain()->SetHttpsUseClientCertificate(false);
	}

	if(CommandToExecute->Parameters.KeyExist("HTTPS_CA_FILE"))
	{
		Value = CommandToExecute->Parameters.GetValue("HTTPS_CA_FILE");
		GetMain()->SetHttpsCAFile(Value.c_str());
	}

	if(CommandToExecute->Parameters.KeyExist("AUTO_DETECT_ENCODING"))
	{
		Value = CommandToExecute->Parameters.GetValue("AUTO_DETECT_ENCODING");
		CleanString(Value);
		MakeUpper(Value);
		if(Value == "YES")
			GetMain()->SetAutoDetectEncoding(true);
		else
			GetMain()->SetAutoDetectEncoding(false);
	}

	if(CommandToExecute->Parameters.KeyExist("DEFAULT_PAGE"))
	{
		Value = CommandToExecute->Parameters.GetValue("DEFAULT_PAGE");
		GetMain()->SetNirvaDefaultPage(Value.c_str());
	}

	if(CommandToExecute->Parameters.KeyExist("DEBUG"))
	{
		Value = CommandToExecute->Parameters.GetValue("DEBUG");
		CleanString(Value);
		MakeUpper(Value);
		if(Value == "YES")
			GetMain()->SetDebug(true);
		else
			GetMain()->SetDebug(false);
		bDebug = GetMain()->GetDebug();
	}
	
	if(CommandToExecute->Parameters.KeyExist("REMOTE_ADMIN"))
	{
		Value = CommandToExecute->Parameters.GetValue("REMOTE_ADMIN");
		CleanString(Value);
		MakeUpper(Value);
		if(Value == "YES")
			GetMain()->SetRemoteAdmin(true);
		else
			GetMain()->SetRemoteAdmin(false);
	}
	
	if(CommandToExecute->Parameters.KeyExist("LONG_SESSION_ID"))
	{
		Value = CommandToExecute->Parameters.GetValue("LONG_SESSION_ID");
		CleanString(Value);
		MakeUpper(Value);
		if(Value == "YES")
			GetMain()->SetLongSessionId(true);
		else
			GetMain()->SetLongSessionId(false);
	}

	if(CommandToExecute->Parameters.KeyExist("MACHINE_ID"))
	{
		Value = CommandToExecute->Parameters.GetValue("MACHINE_ID");
		GetMain()->SetMachineIdForSessions(Value.c_str());
	}

	if(CommandToExecute->Parameters.KeyExist("OLD_OBJECTS_UNICODE"))
	{
		Value = CommandToExecute->Parameters.GetValue("OLD_OBJECTS_UNICODE");
		CleanString(Value);
		MakeUpper(Value);
		if(Value == "YES")
			GetMain()->SetOldObjectsUnicode(true);
		else
			GetMain()->SetOldObjectsUnicode(false);
	}

	if(CommandToExecute->Parameters.KeyExist("INTERNALXSL"))
	{
		Value = CommandToExecute->Parameters.GetValue("INTERNALXSL");
		CleanString(Value);
		MakeUpper(Value);
		if(Value == "YES")
			GetMain()->SetInternalXsl(true);
		else
			GetMain()->SetInternalXsl(false);
	}

	if(CommandToExecute->Parameters.KeyExist("XSLT_ENGINE"))
	{
		Value = CommandToExecute->Parameters.GetValue("XSLT_ENGINE");
		CleanString(Value);
		if(!GetMain()->SetXslPath(Value.c_str()))
		{
			CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_INVALID_XSL_PROCESSOR_COMMAND, Value);
			return false;
		}
	}

	
	if(CommandToExecute->Parameters.KeyExist("XSLT_MAXDEPTH"))
	{
		Value = CommandToExecute->Parameters.GetValue("XSLT_MAXDEPTH");
		CleanString(Value);
		int iValue = atoi(Value.c_str());
		if(iValue < 1)
			iValue = 1;
		GetMain()->SetXslMaxDepth(iValue);
	}

	
	if(CommandToExecute->Parameters.KeyExist("XSLT_PARSERMAXDEPTH"))
	{
		Value = CommandToExecute->Parameters.GetValue("XSLT_PARSERMAXDEPTH");
		CleanString(Value);
		int iValue = atoi(Value.c_str());
		if(iValue < 1)
			iValue = 1;
		GetMain()->SetXslParserMaxDepth(iValue);
	}

	if(CommandToExecute->Parameters.KeyExist("XSLCACHE"))
	{
		Value = CommandToExecute->Parameters.GetValue("XSLCACHE");
		CleanString(Value);
		MakeUpper(Value);
		if(Value == "YES")
		{
			GetMain()->SetEnableXslCache(true);
		}
		else
		{
			GetMain()->SetEnableXslCache(false);
		}
	}

	if(CommandToExecute->Parameters.KeyExist("LOCAL_TIME"))
	{
		Value = CommandToExecute->Parameters.GetValue("LOCAL_TIME");
		CleanString(Value);
		MakeUpper(Value);
		if(Value == "YES")
			GetMain()->SetLocalTime(true);
		else
			GetMain()->SetLocalTime(false);
	}


	if(CommandToExecute->Parameters.KeyExist("INTERNALJVM"))
	{
		Value = CommandToExecute->Parameters.GetValue("INTERNALJVM");
		CleanString(Value);
		MakeUpper(Value);
		if(Value == "YES")
			GetMain()->SetInternalJvm(true);
		else
			GetMain()->SetInternalJvm(false);
	}


	if(CommandToExecute->Parameters.KeyExist("JAVA_USE_ENV_CLASSLOADER"))
	{
		Value = CommandToExecute->Parameters.GetValue("JAVA_USE_ENV_CLASSLOADER");
		CleanString(Value);
		MakeUpper(Value);
		if(Value == "YES")
			GetMain()->SetUseEnvClassLoader(true);
		else
			GetMain()->SetUseEnvClassLoader(false);
	}


	if(CommandToExecute->Parameters.KeyExist("JAVA_USE_CLASS_CACHE"))
	{
		Value = CommandToExecute->Parameters.GetValue("JAVA_USE_CLASS_CACHE");
		CleanString(Value);
		MakeUpper(Value);
		if(Value == "YES")
			GetMain()->SetUseClassCache(true);
		else
			GetMain()->SetUseClassCache(false);
	}

	if(CommandToExecute->Parameters.KeyExist("JVM_MIN_MEM"))
	{
		Value = CommandToExecute->Parameters.GetValue("JVM_MIN_MEM");
		CleanString(Value);
		int iValue = atoi(Value.c_str());
		if(iValue < 2)
			iValue = 2;
		GetMain()->SetJvmMinMem(iValue);
	}

	if(CommandToExecute->Parameters.KeyExist("JVM_MAX_MEM"))
	{
		Value = CommandToExecute->Parameters.GetValue("JVM_MAX_MEM");
		CleanString(Value);
		int iValue = atoi(Value.c_str());
		if(iValue < 2)
			iValue = 2;
		GetMain()->SetJvmMaxMem(iValue);
	}
	
	if(CommandToExecute->Parameters.KeyExist("JVM_PARAM"))
	{
		Value = CommandToExecute->Parameters.GetValue("JVM_PARAM");
		CleanString(Value);
		GetMain()->SetJvmParam(Value.c_str());
	}
	
	if(CommandToExecute->Parameters.KeyExist("JVM_HOME_DIR"))
	{
		Value = CommandToExecute->Parameters.GetValue("JVM_HOME_DIR");
		CleanString(Value);
		GetMain()->SetJvmHomeDir(Value.c_str());
	}
	
	if(CommandToExecute->Parameters.KeyExist("JVM_PATH"))
	{
		Value = CommandToExecute->Parameters.GetValue("JVM_PATH");
		CleanString(Value);
		GetMain()->SetJvmPath(Value.c_str());
	}


	if(CommandToExecute->Parameters.KeyExist("USE_TRANSACTION"))
	{
		Value = CommandToExecute->Parameters.GetValue("USE_TRANSACTION");
		CleanString(Value);
		MakeUpper(Value);
		if(Value == "YES")
			GetMain()->SetUseTransactionServer(true);
		else
			GetMain()->SetUseTransactionServer(false);
	}

	if(CommandToExecute->Parameters.KeyExist("MAX_TRANSACTIONS"))
	{
		Value = CommandToExecute->Parameters.GetValue("MAX_TRANSACTIONS");
		CleanString(Value);
		int iValue = atoi(Value.c_str());
		if(iValue < 1)
			iValue = 1;
		if(iValue > 500)
			iValue = 500;
		GetMain()->SetMaxTransactions(iValue);
	}

	if(CommandToExecute->Parameters.KeyExist("TRANSACTION_CHECK_EVERY"))
	{
		Value = CommandToExecute->Parameters.GetValue("TRANSACTION_CHECK_EVERY");
		CleanString(Value);
		int iValue = atoi(Value.c_str());
		if(iValue < 1)
			iValue = 1;
		GetMain()->SetTransactionCheckEvery(iValue);
	}
	
	if(CommandToExecute->Parameters.KeyExist("MAX_SCHEDULER_TASKS"))
	{
		Value = CommandToExecute->Parameters.GetValue("MAX_SCHEDULER_TASKS");
		CleanString(Value);
		int iValue = atoi(Value.c_str());
		if(iValue < 1)
			iValue = 1;
		if(iValue > 500)
			iValue = 500;
		GetMain()->SetMaxSchedulerTasks(iValue);
	}

	if(CommandToExecute->Parameters.KeyExist("SCHEDULER_CHECK_EVERY"))
	{
		Value = CommandToExecute->Parameters.GetValue("SCHEDULER_CHECK_EVERY");
		CleanString(Value);
		int iValue = atoi(Value.c_str());
		if(iValue < 1)
			iValue = 1;
		GetMain()->SetSchedulerCheckEvery(iValue);
	}

	if (CommandToExecute->Parameters.KeyExist("SCHEDULER_LOCAL_TIME"))
	{
		Value = CommandToExecute->Parameters.GetValue("SCHEDULER_LOCAL_TIME");
		CleanString(Value);
		MakeUpper(Value);
		bool bSchedulerLocalTime = true;
		if (Value == "NO")
			bSchedulerLocalTime = false;
		GetMain()->SetSchedulerLocalTime(bSchedulerLocalTime);
	}

	if(CommandToExecute->Parameters.KeyExist("DEFAULT_APP_TIMEOUT"))
	{
		Value = CommandToExecute->Parameters.GetValue("DEFAULT_APP_TIMEOUT");
		CleanString(Value);
		int iValue = atoi(Value.c_str());
		if(iValue < 30)
			iValue = 30;
		GetMain()->SetDefaultAppTimeOut(iValue);
	}

	if(CommandToExecute->Parameters.KeyExist("DEFAULT_APP_AUTH_MODE"))
	{
		string AuthMode = CommandToExecute->Parameters.GetValue("DEFAULT_APP_AUTH_MODE");
		int iAuthMode = NV_AUTHENTICATION_NIRVA;
		if(AuthMode == "SSO")
			iAuthMode = NV_AUTHENTICATION_SSO;
		if(AuthMode == "NIRVA_SSO")
			iAuthMode = NV_AUTHENTICATION_NIRVA_SSO;
		GetMain()->SetDefaultAppAuthMode(iAuthMode);
	}

	if(CommandToExecute->Parameters.KeyExist("DEFAULT_APP_NIRVA_IS_SSO_USER"))
	{
		Value = CommandToExecute->Parameters.GetValue("DEFAULT_APP_NIRVA_IS_SSO_USER");
		CleanString(Value);
		MakeUpper(Value);
		bool bNirvaIsSsoUser = false;
		if(Value == "YES")
			bNirvaIsSsoUser = true;
		GetMain()->SetDefaultAppNirvaIsSsoUser(bNirvaIsSsoUser);
	}

	if(CommandToExecute->Parameters.KeyExist("DEFAULT_APP_SECUAPP"))
	{
		Value = CommandToExecute->Parameters.GetValue("DEFAULT_APP_SECUAPP");
		CleanString(Value);
		GetMain()->SetDefaultAppSecuApp(Value.c_str());
	}

	if(CommandToExecute->Parameters.KeyExist("DEFAULT_APP_USE_SCHEDULED_TRANSAC"))
	{
		Value = CommandToExecute->Parameters.GetValue("DEFAULT_APP_USE_SCHEDULED_TRANSAC");
		CleanString(Value);
		MakeUpper(Value);
		bool bUseScheduled = true;
		if(Value == "NO")
			bUseScheduled = false;
		GetMain()->SetDefaultAppUseScheduled(bUseScheduled);
	}

	if(CommandToExecute->Parameters.KeyExist("DEFAULT_APP_USE_TRIGGER"))
	{
		Value = CommandToExecute->Parameters.GetValue("DEFAULT_APP_USE_TRIGGER");
		CleanString(Value);
		MakeUpper(Value);
		bool bUseTrigger = true;
		if(Value == "NO")
			bUseTrigger = false;
		GetMain()->SetDefaultAppUseTrigger(bUseTrigger);
	}

	if(CommandToExecute->Parameters.KeyExist("DEFAULT_APP_ALLOW_WEB_XML"))
	{
		Value = CommandToExecute->Parameters.GetValue("DEFAULT_APP_ALLOW_WEB_XML");
		CleanString(Value);
		MakeUpper(Value);
		bool bAllowWebXml = true;
		if(Value == "NO")
			bAllowWebXml = false;
		GetMain()->SetDefaultAppAllowWebXml(bAllowWebXml);
	}


	if(CommandToExecute->Parameters.KeyExist("DEFAULT_APP_ALLOW_NOT_OWNED_SESSIONS"))
	{
		Value = CommandToExecute->Parameters.GetValue("DEFAULT_APP_ALLOW_NOT_OWNED_SESSIONS");
		CleanString(Value);
		MakeUpper(Value);
		bool bAllowNotOwnedSession = true;
		if(Value == "NO")
			bAllowNotOwnedSession = false;
		GetMain()->SetDefaultAppAllowNotOwnedSession(bAllowNotOwnedSession);
	}

	if(CommandToExecute->Parameters.KeyExist("DEFAULT_APP_COMMAND_CLIENT_FILTER_MODE"))
	{
		string CommandClientFilterMode = CommandToExecute->Parameters.GetValue("DEFAULT_APP_COMMAND_CLIENT_FILTER_MODE");
		CleanString(CommandClientFilterMode);
		MakeUpper(CommandClientFilterMode);
		int iCommandClientFilterMode = NV_COMMAND_FILTER_NEVER;
		if(CommandClientFilterMode == "ALWAYS")
			iCommandClientFilterMode = NV_COMMAND_FILTER_ALWAYS;
		if(CommandClientFilterMode == "BYSECU")
			iCommandClientFilterMode = NV_COMMAND_FILTER_BYSECU;
		GetMain()->SetDefaultAppCommandClientFilterMode(iCommandClientFilterMode);
	}

	if(CommandToExecute->Parameters.KeyExist("DEFAULT_APP_COMMAND_WEB_FILTER_MODE"))
	{
		string CommandWebFilterMode = CommandToExecute->Parameters.GetValue("DEFAULT_APP_COMMAND_WEB_FILTER_MODE");
		CleanString(CommandWebFilterMode);
		MakeUpper(CommandWebFilterMode);
		int iCommandWebFilterMode = NV_COMMAND_FILTER_NEVER;
		if(CommandWebFilterMode == "ALWAYS")
			iCommandWebFilterMode = NV_COMMAND_FILTER_ALWAYS;
		if(CommandWebFilterMode == "BYSECU")
			iCommandWebFilterMode = NV_COMMAND_FILTER_BYSECU;
		GetMain()->SetDefaultAppCommandWebFilterMode(iCommandWebFilterMode);
	}

	if(CommandToExecute->Parameters.KeyExist("DEFAULT_APP_COMMAND_CLIENT_FILTER"))
	{
		string CommandClientFilter = CommandToExecute->Parameters.GetValue("DEFAULT_APP_COMMAND_CLIENT_FILTER");
		GetMain()->SetDefaultAppCommandClientFilter(CommandClientFilter.c_str());
	}

	if(CommandToExecute->Parameters.KeyExist("DEFAULT_APP_COMMAND_WEB_FILTER"))
	{
		string CommandWebFilter = CommandToExecute->Parameters.GetValue("DEFAULT_APP_COMMAND_WEB_FILTER");
		GetMain()->SetDefaultAppCommandWebFilter(CommandWebFilter.c_str());
	}

	if(CommandToExecute->Parameters.KeyExist("REG_HOME_DIR"))
	{
		Value = CommandToExecute->Parameters.GetValue("REG_HOME_DIR");
		CleanString(Value);
		GetMain()->SetRegistryHomeDir(Value.c_str());
	}

	if(CommandToExecute->Parameters.KeyExist("LOG_HOME_DIR"))
	{
		Value = CommandToExecute->Parameters.GetValue("LOG_HOME_DIR");
		CleanString(Value);
		GetMain()->SetLogHomeDir(Value.c_str());
	}

	if(CommandToExecute->Parameters.KeyExist("APPWORK_HOME_DIR"))
	{
		Value = CommandToExecute->Parameters.GetValue("APPWORK_HOME_DIR");
		CleanString(Value);
		GetMain()->SetAppWorkHomeDir(Value.c_str());
	}

	if(CommandToExecute->Parameters.KeyExist("DEFAULT_MAIL_SERVER"))
	{
		Value = CommandToExecute->Parameters.GetValue("DEFAULT_MAIL_SERVER");
		CleanString(Value);
		GetMain()->SetDefaultMailServer(Value.c_str());
	}

	if(CommandToExecute->Parameters.KeyExist("DEFAULT_MAIL_USER"))
	{
		Value = CommandToExecute->Parameters.GetValue("DEFAULT_MAIL_USER");
		CleanString(Value);
		GetMain()->SetDefaultMailUser(Value.c_str());
	}

	if(CommandToExecute->Parameters.KeyExist("DEFAULT_MAIL_PASSWORD"))
	{
		Value = CommandToExecute->Parameters.GetValue("DEFAULT_MAIL_PASSWORD");
		CleanString(Value);
		GetMain()->SetDefaultMailPassword(Value.c_str());
	}

	if(CommandToExecute->Parameters.KeyExist("HOST_WEBS"))
	{
		Value = CommandToExecute->Parameters.GetValue("HOST_WEBS");
		CleanString(Value);
		GetMain()->SetHostWebs(Value.c_str());
	}

	if(CommandToExecute->Parameters.KeyExist("DEFAULT_UNICODE"))
	{
		Value = CommandToExecute->Parameters.GetValue("DEFAULT_UNICODE");
		CleanString(Value);
		MakeUpper(Value);
		if(Value == "YES")
			GetMain()->SetDefaultUnicode(true);
		else
			GetMain()->SetDefaultUnicode(false);
	}
	return true;
}

	
// SYSTEM SYSTEM GET_PARAM command (undocumented)
bool NvSession::GetSystemParam(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!CheckPermission(CommandToExecute, "SYSTEM", "GET_SYSTEM_PARAM"))
		return false;

	OutContainer->RemoveObject("SYSTEM_PARAM");
	if(!OutContainer->CreateObject("SYSTEM_PARAM", NV_OBJTYPE_INDSTRINGLIST))
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_CREATE, "SYSTEM_PARAM");
		return false;
	}

	NvObjIndStringList *SystemParam = (NvObjIndStringList *)(OutContainer->GetObject("SYSTEM_PARAM"));
	if(SystemParam == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET, "SYSTEM_PARAM");
		return false;
	}

	char Buff[128];
	sprintf(Buff, "%d", GetMain()->GetHttpPort());
	SystemParam->SetValue("HTTP_PORT", Buff);
	sprintf(Buff, "%d", GetMain()->GetHttpsPort());
	SystemParam->SetValue("HTTPS_PORT", Buff);
	sprintf(Buff, "%d", GetMain()->GetHttpSessionTimeOut());
	SystemParam->SetValue("HTTP_TIME_OUT", Buff);
	sprintf(Buff, "%d", GetMain()->GetHttpStopWaitTime());
	SystemParam->SetValue("HTTP_WAIT_STOP", Buff);
	sprintf(Buff, "%d", (GetMain()->GetHttpMaxMemBody() / 1024));
	SystemParam->SetValue("HTTP_MAX_MEM_BODY", Buff);
	SystemParam->SetValue("DEFAULT_PAGE", GetMain()->GetNirvaDefaultPage().c_str());


	if(GetMain()->GetSSL())
		SystemParam->SetValue("SSL", "YES");
	else
		SystemParam->SetValue("SSL", "NO");

	int iMinProto = GetMain()->GetSSLMinProto();
	if(iMinProto == SSL_MINI_PROTO_TLS10)
		SystemParam->SetValue("SSL_MIN_PROTOCOL", "TLS10");
	else if (iMinProto == SSL_MINI_PROTO_TLS11)
		SystemParam->SetValue("SSL_MIN_PROTOCOL", "TLS11");
	else if (iMinProto == SSL_MINI_PROTO_TLS12)
		SystemParam->SetValue("SSL_MIN_PROTOCOL", "TLS12");
	else
		SystemParam->SetValue("SSL_MIN_PROTOCOL", "NONE");

	if(GetMain()->GetHttpLocal())
		SystemParam->SetValue("HTTP_LOCAL", "YES");
	else
		SystemParam->SetValue("HTTP_LOCAL", "NO");

	if(GetMain()->GetHttpIpV6())
		SystemParam->SetValue("HTTP_IPV6", "YES");
	else
		SystemParam->SetValue("HTTP_IPV6", "NO");

	if(GetMain()->GetHttpBlockServerHeader())
		SystemParam->SetValue("HTTP_BLOCK_SERVER_HEADER", "YES");
	else
		SystemParam->SetValue("HTTP_BLOCK_SERVER_HEADER", "NO");

	if(GetMain()->GetHttpsUseClientCertificate())
		SystemParam->SetValue("HTTPS_USE_CLIENT_CERTIFICATE", "YES");
	else
		SystemParam->SetValue("HTTPS_USE_CLIENT_CERTIFICATE", "NO");

	SystemParam->SetValue("HTTPS_CA_FILE", GetMain()->GetHttpsCAFile().c_str());

	if(GetMain()->GetAutoDetectEncoding())
		SystemParam->SetValue("AUTO_DETECT_ENCODING", "YES");
	else
		SystemParam->SetValue("AUTO_DETECT_ENCODING", "NO");

	if(GetMain()->GetDebug())
		SystemParam->SetValue("DEBUG", "YES");
	else
		SystemParam->SetValue("DEBUG", "NO");
	
	if(GetMain()->GetRemoteAdmin())
		SystemParam->SetValue("REMOTE_ADMIN", "YES");
	else
		SystemParam->SetValue("REMOTE_ADMIN", "NO");
	
	if(GetMain()->GetLongSessionId())
		SystemParam->SetValue("LONG_SESSION_ID", "YES");
	else
		SystemParam->SetValue("LONG_SESSION_ID", "NO");
	SystemParam->SetValue("MACHINE_ID", GetMain()->GetMachineIdForSessions().c_str());

	if(GetMain()->GetOldObjectsUnicode())
		SystemParam->SetValue("OLD_OBJECTS_UNICODE", "YES");
	else
		SystemParam->SetValue("OLD_OBJECTS_UNICODE", "NO");

	if(GetMain()->GetInternalXsl())
		SystemParam->SetValue("INTERNALXSL", "YES");
	else
		SystemParam->SetValue("INTERNALXSL", "NO");

	if(GetMain()->GetEnableXslCache())
		SystemParam->SetValue("XSLCACHE", "YES");
	else
		SystemParam->SetValue("XSLCACHE", "NO");

	if(GetMain()->GetLocalTime())
		SystemParam->SetValue("LOCAL_TIME", "YES");
	else
		SystemParam->SetValue("LOCAL_TIME", "NO");

	SystemParam->SetValue("XSLT_ENGINE", GetMain()->GetXslPath().c_str());
	sprintf(Buff, "%d", GetMain()->GetXslMaxDepth());
	SystemParam->SetValue("XSLT_MAXDEPTH", Buff);
	sprintf(Buff, "%d", GetMain()->GetXslParserMaxDepth());
	SystemParam->SetValue("XSLT_PARSERMAXDEPTH", Buff);
	
	if(GetMain()->GetInternalJvm())
		SystemParam->SetValue("INTERNALJVM", "YES");
	else
		SystemParam->SetValue("INTERNALJVM", "NO");
	if(GetMain()->GetUseEnvClassLoader())
		SystemParam->SetValue("JAVA_USE_ENV_CLASSLOADER", "YES");
	else
		SystemParam->SetValue("JAVA_USE_ENV_CLASSLOADER", "NO");
	if(GetMain()->GetUseClassCache())
		SystemParam->SetValue("JAVA_USE_CLASS_CACHE", "YES");
	else
		SystemParam->SetValue("JAVA_USE_CLASS_CACHE", "NO");
	sprintf(Buff, "%d", GetMain()->GetJvmMinMem());
	SystemParam->SetValue("JVM_MIN_MEM", Buff);
	sprintf(Buff, "%d", GetMain()->GetJvmMaxMem());
	SystemParam->SetValue("JVM_MAX_MEM", Buff);
	SystemParam->SetValue("JVM_PARAM", GetMain()->GetJvmParam().c_str());
	SystemParam->SetValue("JVM_HOME_DIR", GetMain()->GetJvmHomeDir().c_str());
	SystemParam->SetValue("JVM_PATH", GetMain()->GetJvmPath().c_str());
	
	
	if(GetMain()->GetUseTransactionServer())
		SystemParam->SetValue("USE_TRANSACTION", "YES");
	else
		SystemParam->SetValue("USE_TRANSACTION", "NO");
	sprintf(Buff, "%d", GetMain()->GetMaxTransactions());
	SystemParam->SetValue("MAX_TRANSACTIONS", Buff);
	sprintf(Buff, "%d", GetMain()->GetTransactionCheckEvery());
	SystemParam->SetValue("TRANSACTION_CHECK_EVERY", Buff);
	
	sprintf(Buff, "%d", GetMain()->GetMaxSchedulerTasks());
	SystemParam->SetValue("MAX_SCHEDULER_TASKS", Buff);
	sprintf(Buff, "%d", GetMain()->GetSchedulerCheckEvery());
	SystemParam->SetValue("SCHEDULER_CHECK_EVERY", Buff);
	if (GetMain()->GetSchedulerLocalTime())
		SystemParam->SetValue("SCHEDULER_LOCAL_TIME", "YES");
	else
		SystemParam->SetValue("SCHEDULER_LOCAL_TIME", "NO");

	sprintf(Buff, "%d", GetMain()->GetDefaultAppTimeOut());
	SystemParam->SetValue("DEFAULT_APP_TIMEOUT", Buff);

	SystemParam->SetValue("DEFAULT_APP_SECUAPP", GetMain()->GetDefaultAppSecuApp().c_str());
	
	int AuthMode = GetMain()->GetDefaultAppAuthMode();
	strcpy(Buff, "NIRVA");
	if(AuthMode == NV_AUTHENTICATION_SSO)
		strcpy(Buff, "SSO");
	if(AuthMode == NV_AUTHENTICATION_NIRVA_SSO)
		strcpy(Buff, "NIRVA_SSO");
	SystemParam->SetValue("DEFAULT_APP_AUTH_MODE", Buff);

	if(GetMain()->GetDefaultAppNirvaIsSsoUser())
		SystemParam->SetValue("DEFAULT_APP_NIRVA_IS_SSO_USER", "YES");
	else
		SystemParam->SetValue("DEFAULT_APP_NIRVA_IS_SSO_USER", "NO");

	if(GetMain()->GetDefaultAppUseScheduled())
		SystemParam->SetValue("DEFAULT_APP_USE_SCHEDULED_TRANSAC", "YES");
	else
		SystemParam->SetValue("DEFAULT_APP_USE_SCHEDULED_TRANSAC", "NO");
	if(GetMain()->GetDefaultAppUseTrigger())
		SystemParam->SetValue("DEFAULT_APP_USE_TRIGGER", "YES");
	else
		SystemParam->SetValue("DEFAULT_APP_USE_TRIGGER", "NO");
	if(GetMain()->GetDefaultAppAllowWebXml())
		SystemParam->SetValue("DEFAULT_APP_ALLOW_WEB_XML", "YES");
	else
		SystemParam->SetValue("DEFAULT_APP_ALLOW_WEB_XML", "NO");
	if(GetMain()->GetDefaultAppAllowNotOwnedSession())
		SystemParam->SetValue("DEFAULT_APP_ALLOW_NOT_OWNED_SESSIONS", "YES");
	else
		SystemParam->SetValue("DEFAULT_APP_ALLOW_NOT_OWNED_SESSIONS", "NO");
	
	int CommandClientFilterMode = GetMain()->GetDefaultAppCommandClientFilterMode();
	strcpy(Buff, "NEVER");
	if(CommandClientFilterMode == NV_COMMAND_FILTER_ALWAYS)
		strcpy(Buff, "ALWAYS");
	if(CommandClientFilterMode == NV_COMMAND_FILTER_BYSECU)
		strcpy(Buff, "BYSECU");
	SystemParam->SetValue("DEFAULT_APP_COMMAND_CLIENT_FILTER_MODE", Buff);
	
	int CommandWebFilterMode = GetMain()->GetDefaultAppCommandWebFilterMode();
	strcpy(Buff, "NEVER");
	if(CommandWebFilterMode == NV_COMMAND_FILTER_ALWAYS)
		strcpy(Buff, "ALWAYS");
	if(CommandWebFilterMode == NV_COMMAND_FILTER_BYSECU)
		strcpy(Buff, "BYSECU");
	SystemParam->SetValue("DEFAULT_APP_COMMAND_WEB_FILTER_MODE", Buff);
	SystemParam->SetValue("DEFAULT_APP_COMMAND_CLIENT_FILTER", GetMain()->GetDefaultAppCommandClientFilter().c_str());
	SystemParam->SetValue("DEFAULT_APP_COMMAND_WEB_FILTER", GetMain()->GetDefaultAppCommandWebFilter().c_str());

	SystemParam->SetValue("REG_HOME_DIR", GetMain()->GetRegistryHomeDir(true).c_str());
	SystemParam->SetValue("LOG_HOME_DIR", GetMain()->GetLogHomeDir(true).c_str());
	SystemParam->SetValue("APPWORK_HOME_DIR", GetMain()->GetAppWorkHomeDir(true).c_str());
	
	SystemParam->SetValue("DEFAULT_MAIL_SERVER", GetMain()->GetDefaultMailServer().c_str());
	SystemParam->SetValue("DEFAULT_MAIL_USER", GetMain()->GetDefaultMailUser().c_str());
	SystemParam->SetValue("DEFAULT_MAIL_PASSWORD", GetMain()->GetDefaultMailPassword().c_str());
	SystemParam->SetValue("HOST_WEBS", GetMain()->GetHostWebs().c_str());
	if(GetMain()->GetDefaultUnicode())
		SystemParam->SetValue("DEFAULT_UNICODE", "YES");
	else
		SystemParam->SetValue("DEFAULT_UNICODE", "NO");
	
	return true;
}
	


	
// SYSTEM SYSTEM DEBUG_STATE command (undocumented)
bool NvSession::SystemDebugState(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
#ifdef WIN32
#ifdef _DEBUG
	GetMain()->DebugState();
#endif
#endif
	return true;
}
	
// SYSTEM SYSTEM DEBUG_DUMP command (undocumented)
bool NvSession::SystemDebugDump(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
#ifdef WIN32
#ifdef _DEBUG
	GetMain()->DebugDump();
#endif
#endif
	return true;
}
	
// SYSTEM DEBUG DISPLAY_MESSAGE
bool NvSession::DebugDisplayMessage(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!bDebug)
		return true;

	WriteDebugMessageToAttachedConsole(CommandToExecute);

	if(!GetMain()->b_ConsoleMode)
		return true;

	string Message = CommandToExecute->Parameters.GetValue("MESSAGE");
	NV_LOCKLOCK(&ProtectConsoleLog);
	printf("Debug: ");
	printf("%s", Message.c_str());
	printf("\n");
	NV_UNLOCKLOCK(&ProtectConsoleLog);
	return true;
}
	
// SYSTEM DEBUG DISPLAY_OBJECT
bool NvSession::DebugDisplayObject(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!bDebug)
		return true;

	WriteDebugObjectToAttachedConsole(CommandToExecute);

	if(!GetMain()->b_ConsoleMode)
		return true;

	string ObjName = CommandToExecute->Parameters.GetValue("NAME");

	// The function uses only the input container
	if(!CheckInputContainer(CommandToExecute))
		return false;

	NvObj* Object = InContainer->GetObject(ObjName.c_str());
	if(Object == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "OBJECT", NV_OBJECT_CANNOT_GET, ObjName);
		return false;
	}

	NV_LOCKLOCK(&ProtectConsoleLog);
	int iObjectType = Object->GetType();
	if(iObjectType == NV_OBJTYPE_BOOLEAN)
	{
		printf("Debug: object %s of container %s - Type BOOLEAN:\n", Object->GetName().c_str(), CommandToExecute->m_sInContainer.c_str());
		printf("\t ");
		if(((NvObjBoolean *)Object)->GetValue())
			printf("\t TRUE\n");
		else
			printf("\t FALSE\n");
	}
	if(iObjectType == NV_OBJTYPE_INTEGER)
	{
		printf("Debug: object %s of container %s - Type INTEGER:\n", Object->GetName().c_str(), CommandToExecute->m_sInContainer.c_str());
		printf("\t %ld\n", (unsigned long)(((NvObjInteger *)Object)->GetValue()));
	}
	if(iObjectType == NV_OBJTYPE_STRING)
	{
		printf("Debug: object %s of container %s - Type STRING:\n", Object->GetName().c_str(), CommandToExecute->m_sInContainer.c_str());
		printf("\t %s\n", ((NvObjString *)Object)->GetValue().c_str());
	}
	if(iObjectType == NV_OBJTYPE_STRINGLIST)
	{
		printf("Debug: object %s of container %s - Type STRINGLIST:\n", Object->GetName().c_str(), CommandToExecute->m_sInContainer.c_str());
		NvObjStringList* ObjectHere = (NvObjStringList*)Object;
		int i;
		for(i=0;i<ObjectHere->GetSize();i++)
			printf("\t %s\n", ObjectHere->GetValue(i).c_str());
	}
	if(iObjectType == NV_OBJTYPE_INDSTRINGLIST)
	{
		printf("Debug: object %s of container %s - Type INDSTRINGLIST:\n", Object->GetName().c_str(), CommandToExecute->m_sInContainer.c_str());
		NvObjIndStringList* ObjectHere = (NvObjIndStringList*)Object;
		map<string,string>::iterator pos;
		for(pos = ObjectHere->m_mList.begin(); pos != ObjectHere->m_mList.end(); ++pos)
			printf("\t%s = %s\n",pos->first.c_str(),pos->second.c_str());
	}
	if(iObjectType == NV_OBJTYPE_TABLE)
	{

		// Get eventual column separator
		string ColSep = CommandToExecute->Parameters.GetValue("COLSEP");
		if(!ColSep.length())
			ColSep = ";";

		// Get eventual line separator
		string LineSep = CommandToExecute->Parameters.GetValue("LINESEP");
		if(!LineSep.length())
			LineSep = "|";

		printf("Debug: object %s of container %s - Type TABLE:\n", Object->GetName().c_str(), CommandToExecute->m_sInContainer.c_str());
		NvObjTable* ObjectHere = (NvObjTable*)Object;
		int NumRows = ObjectHere->GetNumRow();
		printf("\tNumber of rows = %ld\n", (unsigned long)NumRows);
		int NumCols = ObjectHere->GetNumColumn();
		printf("\tNumber of columns = %ld\n", (unsigned long)NumCols);
		printf("\tcolumns: ");
		int i;
		for(i=0;i<NumCols;i++)
		{
			if(i==0)
				printf("%s", ObjectHere->GetColumnName(i).c_str());
			else
				printf(";%s", ObjectHere->GetColumnName(i).c_str());
		}
		printf("\n");

		if(NumRows)
		{
			// Get specific rows
			string Rows = CommandToExecute->Parameters.GetValue("ROWS");
			CleanString(Rows);
			int MinRow = 0;
			int MaxRow = NumRows-1;
			if(Rows.length())
			{
				string sMinRow = Rows;
				string sMaxRow = Rows;
				string::size_type index = sMinRow.find('-');
				if(index == string::npos)
				{
					// Single row is requested
					MinRow = atoi(sMinRow.c_str());
					MinRow--;
					if(MinRow >= NumRows)
						MinRow = NumRows-1;
					if(MinRow < 0)
						MinRow = 0;
					MaxRow = MinRow;
				}
				else
				{
					// Range is requested
					sMaxRow = sMinRow.substr(index+1);
					sMinRow = sMinRow.substr(0, index);
					CleanString(sMaxRow);
					CleanString(sMinRow);
					if(sMinRow.length())
					{
						MinRow = atoi(sMinRow.c_str());
						MaxRow = atoi(sMaxRow.c_str());
						MinRow--;
						MaxRow--;
						if(MinRow < 0)
							MinRow = 0;
						if(MaxRow < 0)
							MaxRow = NumRows-1;
						if(MaxRow >= NumRows)
							MaxRow = NumRows-1;
						if(MinRow > MaxRow)
							MinRow = MaxRow;
					}
					else
					{
						// A single negative number has been given
						// So we get the last row
						MinRow = NumRows-1;	// Last row
						MaxRow = MinRow;
					}
				}
			}
			string Value;
			for(i=MinRow;i<=MaxRow;i++)
			{
				Value = "";
				int j;
				for(j=0;j<NumCols;j++)
				{
					if(j!=0)
						Value += ColSep;
					int NumCells = ObjectHere->GetNumCellLines(i, j);
					int k;
					for(k=0;k<NumCells;k++)
					{
						if(k!=0)
							Value += LineSep;
						Value += ObjectHere->GetCellLine(i, j, k);
					}
				}
				printf("\trow %d: %s\n", i+1, Value.c_str());
			}
		}
	}
	if(iObjectType == NV_OBJTYPE_FILE)
	{
		printf("Debug: object %s of container %s - Type FILE:\n", Object->GetName().c_str(), CommandToExecute->m_sInContainer.c_str());
		NvObjFile* ObjectHere = (NvObjFile*)Object;
		printf("\tPathname = %s\n", ObjectHere->GetFilePathName().c_str());
		printf("\tSize = %llu\n", ObjectHere->GetSize());
		printf("\tOrigin = %s\n", ObjectHere->GetOrigin().c_str());

/*		bool bWithData = false;
		string sWithData = CommandToExecute->Parameters.GetValue("WITH_DATA");
		CleanString(sWithData);
		MakeUpper(sWithData);
		if(sWithData == "YES")
			bWithData = true;
		if(bWithData)
		{
			if(ObjectHere->GetFilePathName().length())
			{
				char* Buffer = (char *)(malloc(4096));
				FILE *File = Nvfopen(ObjectHere->GetFilePathName().c_str(), "rb");
				if(File != NULL)
				{
					int NumBytesRead;
					while(NumBytesRead=fread(Buffer, 1, 4095, File))
					{
						Buffer[NumBytesRead+1] = '\0';
						printf(Buffer);
					}
					fclose(File);
					printf("\n");
				}
				free(Buffer);
			}
		}*/
	}
	if(iObjectType == NV_OBJTYPE_BINARY)
	{
		printf("Debug: object %s of container %s - Type BINARY:\n", Object->GetName().c_str(), CommandToExecute->m_sInContainer.c_str());
		NvObjBinary* ObjectHere = (NvObjBinary*)Object;
		printf("\tSize = %ld\n", (unsigned long)(ObjectHere->GetSize()));

		/*
		bool bWithData = false;
		string sWithData = CommandToExecute->Parameters.GetValue("WITH_DATA");
		CleanString(sWithData);
		MakeUpper(sWithData);
		if(sWithData == "YES")
			bWithData = true;
		if(bWithData)
		{
			if(ObjectHere->m_cdata != NULL)
			{
				printf(ObjectHere->m_cdata);
				printf("\n");
			}
		}*/
	}
	
	NV_UNLOCKLOCK(&ProtectConsoleLog);

	return true;
}
	
// SYSTEM DEBUG DISPLAY_CONTAINER
bool NvSession::DebugDisplayContainer(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!bDebug)
		return true;

	WriteDebugContainerToAttachedConsole(CommandToExecute);

	if(!GetMain()->b_ConsoleMode)
		return true;
	
	string ContainerName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ContainerName);

	bool bIsOutputContainer = false;
	if(!ContainerName.length())
	{
		bIsOutputContainer = true;
		ContainerName = CommandToExecute->m_sOutContainer;
	}

	NvContainer *Container = OutContainer;
	if(!bIsOutputContainer)
		Container = GetContainer(ContainerName, false, CommandToExecute);

	if(!ContainerName.length())
		ContainerName = "nvdef";

	if(Container != NULL)
	{
		NV_LOCKLOCK(&ProtectConsoleLog);
		printf("Debug: container %s:\n", ContainerName.c_str());
		map<string,void *>::iterator pos;
		for(pos = Container->m_mSubcontainers.begin(); pos != Container->m_mSubcontainers.end(); ++pos)
		{
			NvContainer* SubContainer = (NvContainer*)(pos->second);
			printf("\t%s (container)\n",SubContainer->GetName().c_str());
		}
		for(pos = Container->m_mObjects.begin(); pos != Container->m_mObjects.end(); ++pos)
		{
			NvObj* Object = (NvObj*)(pos->second);
			int type = Object->GetType();
			char Buf[100];
			switch(Object->GetType())
			{
				case NV_OBJTYPE_BOOLEAN:
					strcpy(Buf, "boolean");
					break;
				case NV_OBJTYPE_INTEGER:
					strcpy(Buf, "integer");
					break;
				case NV_OBJTYPE_STRING:
					strcpy(Buf, "string");
					break;
				case NV_OBJTYPE_STRINGLIST:
					strcpy(Buf, "stringlist");
					break;
				case NV_OBJTYPE_INDSTRINGLIST:
					strcpy(Buf, "indstringlist");
					break;
				case NV_OBJTYPE_TABLE:
					strcpy(Buf, "table");
					break;
				case NV_OBJTYPE_FILE:
					strcpy(Buf, "file");
					break;
				case NV_OBJTYPE_BINARY:
					strcpy(Buf, "binary");
					break;
				default:
					strcpy(Buf, "unknown");
					break;
			}
			printf("\t%s (%s)\n",Object->GetName().c_str(), Buf);
		}

		NV_UNLOCKLOCK(&ProtectConsoleLog);
		if(!bIsOutputContainer)
			FreeContainer(Container, ContainerName, false, CommandToExecute);
	}
	return true;
}
	
// SYSTEM DEBUG DISPLAY_VARIABLES
bool NvSession::DebugDisplayVariables(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!bDebug)
		return true;

	WriteDebugVariablesToAttachedConsole(CommandToExecute);

	if(!GetMain()->b_ConsoleMode)
		return true;

	NV_LOCKLOCK(&ProtectConsoleLog);
	printf("Debug: variables:\n");
	map<string,string>::iterator pos;
	for(pos = m_oVariables.m_mList.begin(); pos != m_oVariables.m_mList.end(); ++pos)
		printf("\t%s = %s\n",pos->first.c_str(),pos->second.c_str());
	NV_UNLOCKLOCK(&ProtectConsoleLog);
	return true;
}
	
// SYSTEM DEBUG DISPLAY_PARAMETERS (internal use only)
bool NvSession::DebugDisplayParameters(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if(!bDebug)
		return true;

	if(!GetMain()->b_ConsoleMode)
		return true;

	string sDebugDisplayParam = CommandToExecute->Parameters.GetValue("NV_DEBUG_PARAM");
	CleanString(sDebugDisplayParam);
	MakeUpper(sDebugDisplayParam);
	if(sDebugDisplayParam != "YES")
			return true;

	NV_LOCKLOCK(&ProtectConsoleLog);
	printf("Debug: parameters for command %s:%s:%s:\n", CommandToExecute->m_sService.c_str(), CommandToExecute->m_sClass.c_str(), CommandToExecute->m_sCommand.c_str());
	map<string,string>::iterator pos;
	string Name,Value;
	for(pos = CommandToExecute->Parameters.m_mList.begin(); pos != CommandToExecute->Parameters.m_mList.end(); ++pos)
	{
		Name = pos->first;
		CleanString(Name);
		MakeUpper(Name);
		if(Name == "NV_DEBUG_ONLY")
			continue;
		if(Name == "NV_DEBUG_PARAM")
			continue;
		// We must eliminate passwords if there are ones
		bool IsPassword = false;
		if(Name.find("PASSWORD") != string::npos)
			IsPassword = true;
		else if(Name.find("PWD") != string::npos)
			IsPassword = true;

		// Do not display also the unique service id for generating licenses
		if(CommandToExecute->m_sClass == "LICENSE")
			if(CommandToExecute->m_sService == "SYSTEM")
				if((CommandToExecute->m_sCommand == "SET") || (CommandToExecute->m_sCommand == "GET"))
					if(Name == "SERVICE_ID")
						continue;
		if(IsPassword)
			printf("\t%s = ********\n",Name.c_str());
		else
			printf("\t%s = %s\n",Name.c_str(), pos->second.c_str());
	}
	NV_UNLOCKLOCK(&ProtectConsoleLog);
	if(HasSessionAttachedConsole())
	{
		string Message = "Debug: parameters:\n";
		string Name,Value;
		for(pos = CommandToExecute->Parameters.m_mList.begin(); pos != CommandToExecute->Parameters.m_mList.end(); ++pos)
		{
			Name = pos->first;
			CleanString(Name);
			MakeUpper(Name);
			if(Name == "NV_DEBUG_ONLY")
				continue;
			if(Name == "NV_DEBUG_PARAM")
				continue;
			Message += "\t";
			Message += Name;
			Message += " = ";
			Message += pos->second;
			Message += "\n";
		}
		WriteAttachedConsole(Message.c_str());
	}
	return true;
}

// Check if the command is SYSTEM:SESSION:ATTACH_CONSOLE
bool NvSession::IsSessionAttachConsoleCommand(NvCommand *CommandToExecute)
{
	if(CommandToExecute == NULL)
		return false;
	if(CommandToExecute->m_iCommandSource != NV_SOURCE_CLIENT)
		return false;
	if(CommandToExecute->m_sService != "SYSTEM")
		return false;
	if(CommandToExecute->m_sClass != "SESSION")
		return false;
	if(CommandToExecute->m_sCommand != "ATTACH_CONSOLE")
		return false;

	return true;
}


bool NvSession::SessionAttachConsole(NvCommand *CommandToExecute, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	if((CommandToExecute->m_sClientAddress != "127.0.0.1") && (CommandToExecute->m_sClientAddress != "::1"))
	{
		CommandToExecute->Error.SetError("SYSTEM", "SESSION", NV_SESSION_CANNOT_ATTACH_CONSOLE, "Console can be attached localy only");
		return false;
	}
	if(CommandToExecute->m_iCommandSource != NV_SOURCE_CLIENT)
	{
		CommandToExecute->Error.SetError("SYSTEM", "SESSION", NV_SESSION_CANNOT_ATTACH_CONSOLE, "Console can be attached only from nvc client");
		return false;
	}
	
	string sClientPid = CommandToExecute->Parameters.GetValue("PID");
	CleanString(sClientPid);
	if(!sClientPid.length())
	{
		CleanupAttachedConsole(false);
		NV_UNLOCKLOCK(&ProtectPipe);
		CommandToExecute->Error.SetError("SYSTEM", "SESSION", NV_SESSION_CANNOT_ATTACH_CONSOLE, "No process ID given");
		return false;
	}
	int ClientPid = atoi(sClientPid.c_str());
	
	int Level = 1;
	string sLevel = CommandToExecute->Parameters.GetValue("LEVEL");
	CleanString(sLevel);
	if(sLevel.length())
		Level = atoi(sLevel.c_str());
	if(Level < 1)
		Level = 1;
	
	NV_LOCKLOCK(&ProtectPipe);
	// test if we already have a console attached
	if(m_bHasPipe)
	{
		//PM To be tested
		// There is already a console attached
		// try to write to it
#ifdef WIN32
		// Under windows we just try to write to the console
		if(WriteAttachedConsole("Another console attemps to attach to this session\n", 1, false))
		{
			NV_UNLOCKLOCK(&ProtectPipe);
			CommandToExecute->Error.SetError("SYSTEM", "SESSION", NV_SESSION_CANNOT_ATTACH_CONSOLE, "Console already attached to another console");
			return false;
		}
#else
		int killresult = kill(m_iAttachedConsolePid, 0);
		if((killresult == 0) || (errno != ESRCH))
		{
			WriteAttachedConsole("Another console attemps to attach to this session\n", 1, false);
			NV_UNLOCKLOCK(&ProtectPipe);
			CommandToExecute->Error.SetError("SYSTEM", "SESSION", NV_SESSION_CANNOT_ATTACH_CONSOLE, "Console already attached to another console");
			return false;
		}
#endif
		// We didn't succed so probably the console application has been removed
		CleanupAttachedConsole(false);
	}

	m_iAttachedConsolePid = ClientPid;
#ifdef WIN32
	HANDLE hClientProcess = OpenProcess(PROCESS_DUP_HANDLE, false, m_iAttachedConsolePid);
	if(hClientProcess == NULL)
	{
		CleanupAttachedConsole(false);
		NV_UNLOCKLOCK(&ProtectPipe);
		CommandToExecute->Error.SetError("SYSTEM", "SESSION", NV_SESSION_CANNOT_ATTACH_CONSOLE, "Cannot open client process");
		return false;
	}

	HANDLE hClientPipehandle = NULL;
	SECURITY_ATTRIBUTES saAttr;
	
	// Set the bInheritHandle flag so pipe handles are not inherited. 
	saAttr.nLength = sizeof(SECURITY_ATTRIBUTES); 
	saAttr.bInheritHandle = FALSE; 
	saAttr.lpSecurityDescriptor = NULL; 

	if(!CreatePipe(&hClientPipehandle, &hServerWritePipehandle, &saAttr, 0))
	{
		CloseHandle(hClientProcess);
		CleanupAttachedConsole(false);
		NV_UNLOCKLOCK(&ProtectPipe);
		CommandToExecute->Error.SetError("SYSTEM", "SESSION", NV_SESSION_CANNOT_ATTACH_CONSOLE, "Cannot create pipe");
		return false;
	}

	// Ensure the read handle to the client is not inherited.
	if(!SetHandleInformation(hClientPipehandle, HANDLE_FLAG_INHERIT, 0))
	{
		CloseHandle(hClientPipehandle);
		CloseHandle(hClientProcess);
		CleanupAttachedConsole(false);
		NV_UNLOCKLOCK(&ProtectPipe);
		CommandToExecute->Error.SetError("SYSTEM", "SESSION", NV_SESSION_CANNOT_ATTACH_CONSOLE, "Cannot set handle information");
		return false;
	}

	// Duplicate handle for the client process so we can close our side
	HANDLE hClientTempPipehandle = NULL;
	if(!DuplicateHandle(GetCurrentProcess(),
					hClientPipehandle,
					hClientProcess,
					&hClientTempPipehandle,
					0,
					FALSE,
					DUPLICATE_SAME_ACCESS))
	{
		CloseHandle(hClientPipehandle);
		CloseHandle(hClientProcess);
		CleanupAttachedConsole(false);
		NV_UNLOCKLOCK(&ProtectPipe);
		CommandToExecute->Error.SetError("SYSTEM", "SESSION", NV_SESSION_CANNOT_ATTACH_CONSOLE, "Cannot duplicate handle");
		return false;
	}

	// Close process handle
	CloseHandle(hClientProcess);
	// Close our client handle
	CloseHandle(hClientPipehandle);

	// the client duplicated handle doesn(t have to be closed because it has been created for the
	// client process so it's under its responsability

	// We send back the client pipe handle to the client (in the output buffer)
	char sClientPipehandle[50];
	sprintf(sClientPipehandle, "%llu", (NvInt64)hClientTempPipehandle);
	GetStringValue(CommandToExecute, sClientPipehandle, Buffer, BufferSize, DataSize, iEncoding);
#else
	// Under unix we use fifo (naed pipes)
	// We create a fifo in the Nirva/Work directory having the session and cleint PID in its name
	m_sPipeFile = GetMain()->GetNirvaDir();
	m_sPipeFile += "Work";
	AddDirSuffix(m_sPipeFile);
	m_sPipeFile += m_sIdent;
	m_sPipeFile += "_";
	m_sPipeFile += sClientPid;
	m_sPipeFile += ".fifo";
	if(mkfifo(m_sPipeFile.c_str(), 0777) != 0)
	{
		m_sPipeFile = "";
		CleanupAttachedConsole(false);
		NV_UNLOCKLOCK(&ProtectPipe);
		CommandToExecute->Error.SetError("SYSTEM", "SESSION", NV_SESSION_CANNOT_ATTACH_CONSOLE, "Cannot open fifo file");
		return false;
	}
	GetStringValue(CommandToExecute, m_sPipeFile.c_str(), Buffer, BufferSize, DataSize, iEncoding);
#endif
	m_bHasPipe = true;
	bFirstAttachedCommand = true;
	m_iAttachedLevel = Level;
	
	NV_UNLOCKLOCK(&ProtectPipe);

	return true;
}


bool NvSession::WriteAttachedConsole(const char *String, int Level, bool Lock)
{
	if(Lock)
		NV_LOCKLOCK(&ProtectPipe);
	if(!m_bHasPipe)
	{
		if(Lock)
			NV_UNLOCKLOCK(&ProtectPipe);
		return true;
	}
	if(m_iAttachedLevel < Level)
	{
		if(Lock)
			NV_UNLOCKLOCK(&ProtectPipe);
		return true;
	}

	unsigned long lWritten;
	int Length;
	if(bFirstAttachedCommand)
	{
		// This is the first command after attachment
		// For Unix we must open the fifo here in non blocking mode. its opened in read/write but the read is not used
		// We must do that otherwise the open fails because there is nobody who opened in read mode
#ifndef WIN32
		time_t InitialTime = time(NULL);
		while(hServerWritePipehandle <= 0)
		{
//			hServerWritePipehandle = open(m_sPipeFile.c_str(), O_WRONLY | O_NONBLOCK);
			hServerWritePipehandle = open(m_sPipeFile.c_str(), O_RDWR | O_NONBLOCK);
			SLEEP(10);
			if((time(NULL) - InitialTime) > 3)
				break;
		}
		if(hServerWritePipehandle <= 0)
		{
			// The client didn't open the pipe within 3 seconds
			CleanupAttachedConsole(false);
			if(Lock)
				NV_UNLOCKLOCK(&ProtectPipe);
			return false;
		}
		fcntl(hServerWritePipehandle, F_SETFD, FD_CLOEXEC);
#endif		
		// Write banner information
		char Buff2[256];
		sprintf(Buff2, "Session %s has been attached\n\n", m_sIdent.c_str());
		Length = (int)(strlen(Buff2));
#ifdef WIN32
		WriteFile(hServerWritePipehandle, Buff2, Length, &lWritten, NULL);
#else
		write(hServerWritePipehandle, Buff2, Length);
#endif
		// We display the command stack
		vector<string> Stack;
		GetCommandStack(Stack);
		int i;
		string Message = "Current command stack:\n";
		Length = (int)(Message.length());
#ifdef WIN32
		WriteFile(hServerWritePipehandle, Message.c_str(), Length, &lWritten, NULL);
#else
		write(hServerWritePipehandle, Message.c_str(), Length);
#endif
		for(i=0; i<(int)Stack.size(); i++)
		{
			string StackItem = Stack[i];
			StackItem += "\n";
			if(i == Stack.size()-1)
				StackItem += "\n"; // Last item
			Length = (int)(StackItem.length());
#ifdef WIN32
			if(!WriteFile(hServerWritePipehandle, StackItem.c_str(), Length, &lWritten, NULL))
#else
			if(write(hServerWritePipehandle, StackItem.c_str(), Length) != Length)
#endif
			{
				// If we cannot write we close the pipe
				CleanupAttachedConsole(false);
				if(Lock)
					NV_UNLOCKLOCK(&ProtectPipe);
				return false;
			}
		}
		// Reset the first attached command flag
		bFirstAttachedCommand = false;
	}

	Length = (int)(strlen(String));
#ifdef WIN32
	if(!WriteFile(hServerWritePipehandle, String, Length, &lWritten, NULL))
#else
	if(write(hServerWritePipehandle, String, Length) != Length)
#endif
	{
		// If we cannot write we close the pipe
		CleanupAttachedConsole(false);
		if(Lock)
			NV_UNLOCKLOCK(&ProtectPipe);
		return false;
	}
	
	if(Lock)
		NV_UNLOCKLOCK(&ProtectPipe);

	return true;
}

bool NvSession::HasSessionAttachedConsole()
{
	bool RetVal = false;
	NV_LOCKLOCK(&ProtectPipe);
	RetVal = m_bHasPipe;
	NV_UNLOCKLOCK(&ProtectPipe);
	return RetVal;
}

int NvSession::GetAttachedConsoleLevel()
{
	int RetVal = 1;
	NV_LOCKLOCK(&ProtectPipe);
	RetVal = m_iAttachedLevel;
	NV_UNLOCKLOCK(&ProtectPipe);
	return RetVal;
}

void NvSession::CleanupAttachedConsole(bool Lock)
{
	if(Lock)
		NV_LOCKLOCK(&ProtectPipe);
	if(hServerWritePipehandle > 0)
	{
#ifdef WIN32
		CloseHandle(hServerWritePipehandle);
#else
		close(hServerWritePipehandle);
#endif
	}
#ifndef WIN32
	if(m_sPipeFile.length())
		unlink(m_sPipeFile.c_str());
#endif
	hServerWritePipehandle = 0;
	m_sPipeFile = "";
	bFirstAttachedCommand = false;
	m_bHasPipe = false;
	m_iAttachedConsolePid = 0;
	m_iAttachedLevel = 1;
	if(Lock)
		NV_UNLOCKLOCK(&ProtectPipe);
}

// Mode = 1 -> Start command
// Mode = 2 -> End command
void NvSession::WriteCommandToAttachedConsole(NvCommand *CommandToExecute, int Mode, NvRequest *Request)
{
	int AttachedLevel = GetAttachedConsoleLevel();
	if(AttachedLevel < Mode)
		return;
	char Buff[50];
	time_t tTime = time(NULL);
	struct tm tCurrentTime;
	localtime_r(&tTime, &tCurrentTime);
	sprintf(Buff, "%02d:%02d:%02d",
					tCurrentTime.tm_hour,
					tCurrentTime.tm_min,
					tCurrentTime.tm_sec);
	string Message = Buff;
	Message += " ";
	if((Mode == 1) && (AttachedLevel > 1))
		Message += "start ";
	if((Mode == 2) && (AttachedLevel > 1))
		Message += "end   ";
	Message += CommandToExecute->m_sService + ":" + CommandToExecute->m_sClass + ":" + CommandToExecute->m_sCommand;
	if(Request != NULL)
	{
		Message += " -> ";
		Message += Request->m_sServerAddress;
	}
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_CLIENT)
		Message += "(C)";
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_BROWSER)
		Message += "(B)";
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_PROCEDURE)
		Message += "(P)";
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_SERVICE)
		Message += "(S)";
	if(CommandToExecute->m_iCommandSource == NV_SOURCE_SERVER)
		Message += "(V)";
	Message += "\n";
	WriteAttachedConsole(Message.c_str());
	bool DoDisplayParameters = false;
	bool ForceDisplayParameters = false;
	if((Mode == 1) && (AttachedLevel > 2))
	{
		DoDisplayParameters = true;
		ForceDisplayParameters = true;
	}
	if(bDebug)
		DoDisplayParameters = true;
	WriteDebugParametersToAttachedConsole(CommandToExecute, ForceDisplayParameters);
	if((Mode == 2) && (AttachedLevel > 2))
	{
		// We write the output buffer if not empty
		if(CommandToExecute->m_sLastOutputBuffer.length())
		{
			string Message = "Debug: output buffer: ";
			Message += CommandToExecute->m_sLastOutputBuffer.c_str();
			Message += "\n";
			WriteAttachedConsole(Message.c_str());
		}
	}
}

void NvSession::WriteDebugParametersToAttachedConsole(NvCommand *CommandToExecute, bool Force)
{
	if(!HasSessionAttachedConsole())
		return;

	// If not forced we must see if we have NV_DEBUG_PARAM set to yes
	if(!Force)
	{
		if(!bDebug)
			return;
		string sDebugDisplayParam = CommandToExecute->Parameters.GetValue("NV_DEBUG_PARAM");
		CleanString(sDebugDisplayParam);
		MakeUpper(sDebugDisplayParam);
		if(sDebugDisplayParam != "YES")
			return;
	}
	if(!CommandToExecute->Parameters.m_mList.size())
		return;
	string Message = "Debug: parameters:\n";
	string Name,Value;
	map<string,string>::iterator pos;
	for(pos = CommandToExecute->Parameters.m_mList.begin(); pos != CommandToExecute->Parameters.m_mList.end(); ++pos)
	{
		Name = pos->first;
		CleanString(Name);
		MakeUpper(Name);
		if(Name == "NV_DEBUG_ONLY")
			continue;
		if(Name == "NV_DEBUG_PARAM")
			continue;

		// We must eliminate passwords if there are ones
		bool IsPassword = false;
		if(Name.find("PASSWORD") != string::npos)
			IsPassword = true;
		else if(Name.find("PWD") != string::npos)
			IsPassword = true;

		// Do not display also the unique service id for generating licenses
		if(CommandToExecute->m_sClass == "LICENSE")
			if(CommandToExecute->m_sService == "SYSTEM")
				if((CommandToExecute->m_sCommand == "SET") || (CommandToExecute->m_sCommand == "GET"))
					if(Name == "SERVICE_ID")
						continue;
	
		Message += "\t";
		Message += Name;
		Message += " = ";
		if(IsPassword)
			Message += "********";
		else
			Message += pos->second;
		Message += "\n";
	}
	WriteAttachedConsole(Message.c_str());
}

void NvSession::WriteDebugVariablesToAttachedConsole(NvCommand *CommandToExecute)
{
	if(!HasSessionAttachedConsole())
		return;

	string Message = "Debug: variables:\n";
	map<string,string>::iterator pos;
	for(pos = CommandToExecute->Parameters.m_mList.begin(); pos != CommandToExecute->Parameters.m_mList.end(); ++pos)
	{
		Message += "\t";
		Message += pos->first;
		Message += " = ";
		Message += pos->second;
		Message += "\n";
	}
	WriteAttachedConsole(Message.c_str());
}

void NvSession::WriteDebugMessageToAttachedConsole(NvCommand *CommandToExecute)
{
	if(!HasSessionAttachedConsole())
		return;

	string MessageToDisplay = CommandToExecute->Parameters.GetValue("MESSAGE");
	string Message = "Debug: ";
	Message += MessageToDisplay;
	Message += "\n";
	WriteAttachedConsole(Message.c_str());
}
	
// SYSTEM DEBUG DISPLAY_OBJECT
void NvSession::WriteDebugObjectToAttachedConsole(NvCommand *CommandToExecute)
{
	if(!HasSessionAttachedConsole())
		return;

	string ObjName = CommandToExecute->Parameters.GetValue("NAME");

	// The function uses only the input container
	if(!CheckInputContainer(CommandToExecute))
		return;

	NvObj* Object = InContainer->GetObject(ObjName.c_str());
	if(Object == NULL)
		return;

	string Message = "Debug: object: ";
	Message += Object->GetName();
	Message += " of container ";
	Message += CommandToExecute->m_sInContainer;
	Message += " - Type ";
	int iObjectType = Object->GetType();
	if(iObjectType == NV_OBJTYPE_BOOLEAN)
	{
		Message += "BOOLEAN\n";
		if(((NvObjBoolean *)Object)->GetValue())
			Message += "\t TRUE\n";
		else
			Message += "\t FALSE\n";
		WriteAttachedConsole(Message.c_str());
	}
	if(iObjectType == NV_OBJTYPE_INTEGER)
	{
		Message += "INTEGER\n\t";
		char Buf[50];
		sprintf(Buf, "%ld", (unsigned long)(((NvObjInteger *)Object)->GetValue()));
		Message += Buf;
		Message += "\n";
		WriteAttachedConsole(Message.c_str());
	}
	if(iObjectType == NV_OBJTYPE_STRING)
	{
		Message += "STRING\n\t";
		Message += ((NvObjString *)Object)->GetValue();
		Message += "\n";
		WriteAttachedConsole(Message.c_str());
	}
	if(iObjectType == NV_OBJTYPE_STRINGLIST)
	{
		Message += "STRINGLIST\n";
		WriteAttachedConsole(Message.c_str());
		NvObjStringList* ObjectHere = (NvObjStringList*)Object;
		int i;
		for(i=0;i<ObjectHere->GetSize();i++)
		{
			Message = "\t ";
			Message += ObjectHere->GetValue(i);
			Message += "\n";
			WriteAttachedConsole(Message.c_str());
		}
	}
	if(iObjectType == NV_OBJTYPE_INDSTRINGLIST)
	{
		Message += "INDSTRINGLIST\n";
		WriteAttachedConsole(Message.c_str());
		NvObjIndStringList* ObjectHere = (NvObjIndStringList*)Object;
		map<string,string>::iterator pos;
		for(pos = ObjectHere->m_mList.begin(); pos != ObjectHere->m_mList.end(); ++pos)
		{
			Message = "\t ";
			Message += pos->first;
			Message += " = ";
			Message += pos->second;
			Message += "\n";
			WriteAttachedConsole(Message.c_str());
		}
	}
	if(iObjectType == NV_OBJTYPE_TABLE)
	{

		// Get eventual column separator
		string ColSep = CommandToExecute->Parameters.GetValue("COLSEP");
		if(!ColSep.length())
			ColSep = ";";

		// Get eventual line separator
		string LineSep = CommandToExecute->Parameters.GetValue("LINESEP");
		if(!LineSep.length())
			LineSep = "|";

		Message += "TABLE\n";
		WriteAttachedConsole(Message.c_str());
		NvObjTable* ObjectHere = (NvObjTable*)Object;
		int NumRows = ObjectHere->GetNumRow();
		int NumCols = ObjectHere->GetNumColumn();
		Message = "\tNumber of rows = ";
		char Buf[50];
		sprintf(Buf, "%ld", (unsigned long)NumRows);
		Message += Buf;
		Message += "\n\tNumber of columns = ";
		sprintf(Buf, "%ld", (unsigned long)NumCols);
		Message += Buf;
		Message += "\n\tcolumns: ";
		WriteAttachedConsole(Message.c_str());
		int i;
		Message = "";
		for(i=0;i<NumCols;i++)
		{
			if(i!=0)
				Message += ";";
			Message += ObjectHere->GetColumnName(i);
		}
		Message += "\n";
		WriteAttachedConsole(Message.c_str());

		if(NumRows)
		{
			// Get specific rows
			string Rows = CommandToExecute->Parameters.GetValue("ROWS");
			CleanString(Rows);
			int MinRow = 0;
			int MaxRow = NumRows-1;
			if(Rows.length())
			{
				string sMinRow = Rows;
				string sMaxRow = Rows;
				string::size_type index = sMinRow.find('-');
				if(index == string::npos)
				{
					// Single row is requested
					MinRow = atoi(sMinRow.c_str());
					MinRow--;
					if(MinRow >= NumRows)
						MinRow = NumRows-1;
					if(MinRow < 0)
						MinRow = 0;
					MaxRow = MinRow;
				}
				else
				{
					// Range is requested
					sMaxRow = sMinRow.substr(index+1);
					sMinRow = sMinRow.substr(0, index);
					CleanString(sMaxRow);
					CleanString(sMinRow);
					if(sMinRow.length())
					{
						MinRow = atoi(sMinRow.c_str());
						MaxRow = atoi(sMaxRow.c_str());
						MinRow--;
						MaxRow--;
						if(MinRow < 0)
							MinRow = 0;
						if(MaxRow < 0)
							MaxRow = NumRows-1;
						if(MaxRow >= NumRows)
							MaxRow = NumRows-1;
						if(MinRow > MaxRow)
							MinRow = MaxRow;
					}
					else
					{
						// A single negative number has been given
						// So we get the last row
						MinRow = NumRows-1;	// Last row
						MaxRow = MinRow;
					}
				}
			}
			string Value;
			for(i=MinRow;i<=MaxRow;i++)
			{
				Value = "";
				int j;
				for(j=0;j<NumCols;j++)
				{
					if(j!=0)
						Value += ColSep;
					int NumCells = ObjectHere->GetNumCellLines(i, j);
					int k;
					for(k=0;k<NumCells;k++)
					{
						if(k!=0)
							Value += LineSep;
						Value += ObjectHere->GetCellLine(i, j, k);
					}
				}
				Message = "\trow ";
				sprintf(Buf, "%ld", (unsigned long)(i+1));
				Message += Buf;
				Message += ": ";
				Message += Value;
				Message += "\n";
				WriteAttachedConsole(Message.c_str());
			}
		}
	}
	if(iObjectType == NV_OBJTYPE_FILE)
	{
		Message += "FILE\n";
		NvObjFile* ObjectHere = (NvObjFile*)Object;
		Message += "\tPathname = ";
		Message += ObjectHere->GetFilePathName();
		Message += "\n\tSize = ";
		char Buf[50];
		sprintf(Buf, "%llu", ObjectHere->GetSize());
		Message += Buf;
		Message += "\n\tOrigin = ";
		Message += ObjectHere->GetOrigin();
		Message += "\n";
		WriteAttachedConsole(Message.c_str());
	}
	if(iObjectType == NV_OBJTYPE_BINARY)
	{
		Message += "BINARY\n";
		NvObjBinary* ObjectHere = (NvObjBinary*)Object;
		Message += "\tSize = ";
		char Buf[50];
		sprintf(Buf, "%ld", (unsigned long)(ObjectHere->GetSize()));
		Message += Buf;
		Message += "\n";
		WriteAttachedConsole(Message.c_str());
	}
}
	
// SYSTEM DEBUG DISPLAY_CONTAINER
void NvSession::WriteDebugContainerToAttachedConsole(NvCommand *CommandToExecute)
{
	if(!HasSessionAttachedConsole())
		return;
	
	string ContainerName = CommandToExecute->Parameters.GetValue("NAME");
	CleanString(ContainerName);

	bool bIsOutputContainer = false;
	if(!ContainerName.length())
	{
		bIsOutputContainer = true;
		ContainerName = CommandToExecute->m_sOutContainer;
	}

	NvContainer *Container = OutContainer;
	if(!bIsOutputContainer)
		Container = GetContainer(ContainerName, false, CommandToExecute);

	if(!ContainerName.length())
		ContainerName = "nvdef";

	if(Container != NULL)
	{
		string Message = "Debug: container: ";
		Message += ContainerName;
		Message += "\n";
		WriteAttachedConsole(Message.c_str());
		map<string,void *>::iterator pos;
		for(pos = Container->m_mSubcontainers.begin(); pos != Container->m_mSubcontainers.end(); ++pos)
		{
			NvContainer* SubContainer = (NvContainer*)(pos->second);
			Message = "\t";
			Message += SubContainer->GetName();
			Message += " (container)\n";
			WriteAttachedConsole(Message.c_str());
		}
		for(pos = Container->m_mObjects.begin(); pos != Container->m_mObjects.end(); ++pos)
		{
			NvObj* Object = (NvObj*)(pos->second);
			int type = Object->GetType();
			char Buf[100];
			switch(Object->GetType())
			{
				case NV_OBJTYPE_BOOLEAN:
					strcpy(Buf, "boolean");
					break;
				case NV_OBJTYPE_INTEGER:
					strcpy(Buf, "integer");
					break;
				case NV_OBJTYPE_STRING:
					strcpy(Buf, "string");
					break;
				case NV_OBJTYPE_STRINGLIST:
					strcpy(Buf, "stringlist");
					break;
				case NV_OBJTYPE_INDSTRINGLIST:
					strcpy(Buf, "indstringlist");
					break;
				case NV_OBJTYPE_TABLE:
					strcpy(Buf, "table");
					break;
				case NV_OBJTYPE_FILE:
					strcpy(Buf, "file");
					break;
				case NV_OBJTYPE_BINARY:
					strcpy(Buf, "binary");
					break;
				default:
					strcpy(Buf, "unknown");
					break;
			}
			Message = "\t";
			Message += Object->GetName();
			Message += " (";
			Message += Buf;
			Message += ")\n";
			WriteAttachedConsole(Message.c_str());
		}

		if(!bIsOutputContainer)
			FreeContainer(Container, ContainerName, false, CommandToExecute);
	}
}
